@@ -38,20 +38,18 @@
 
 #include "intel_batchbuffer.h"
 
-#ifdef ANDROID
-#ifndef HAVE_MMAP64
-extern void*  __mmap2(void *, size_t, int, int, int, off_t);
-
-/* mmap64 is a recent addition to bionic and not available in all android builds. */
-/* I can find no reliable way to know if it is defined or not - so just avoid it */
-#define mmap64 igt_mmap64
-static inline void *igt_mmap64(void *addr, size_t length, int prot, int flags,
-        int fd, off64_t offset)
-{
-    return __mmap2(addr, length, prot, flags, fd, offset >> 12);
-}
-#endif
-#endif
+#define DRIVER_INTEL	(1 << 0)
+#define DRIVER_VC4	(1 << 1)
+#define DRIVER_VGEM	(1 << 2)
+#define DRIVER_VIRTIO	(1 << 3)
+#define DRIVER_AMDGPU	(1 << 4)
+/*
+ * Exclude DRVER_VGEM from DRIVER_ANY since if you run on a system
+ * with vgem as well as a supported driver, you can end up with a
+ * near-100% skip rate if you don't explicitly specify the device,
+ * depending on device-load ordering.
+ */
+#define DRIVER_ANY 	~(DRIVER_VGEM)
 
 /**
  * ARRAY_SIZE:
@@ -71,12 +69,17 @@ static inline void *igt_mmap64(void *addr, size_t length, int prot, int flags,
 #define ALIGN(v, a) (((v) + (a)-1) & ~((a)-1))
 
 int drm_get_card(void);
-int drm_open_any(void);
-int drm_open_any_master(void);
-int drm_open_any_render(void);
+int drm_open_driver(int chipset);
+int drm_open_driver_master(int chipset);
+int drm_open_driver_render(int chipset);
+int __drm_open_driver(int chipset);
 
 void gem_quiescent_gpu(int fd);
 
+void igt_require_intel(int fd);
+
+bool is_i915_device(int fd);
+
 /**
  * do_or_die:
  * @x: command
@@ -97,7 +100,23 @@ void gem_quiescent_gpu(int fd);
  * successfully executed.
  */
 #define do_ioctl(fd, ioc, ioc_data) do { \
-	igt_assert(drmIoctl((fd), (ioc), (ioc_data)) == 0); \
+	igt_assert_eq(igt_ioctl((fd), (ioc), (ioc_data)), 0); \
+	errno = 0; \
+} while (0)
+
+/**
+ * do_ioctl_err:
+ * @fd: open i915 drm file descriptor
+ * @ioc: ioctl op definition from drm headers
+ * @ioc_data: data pointer for the ioctl operation
+ * @err: value to expect in errno
+ *
+ * This macro wraps drmIoctl() and uses igt_assert to check that it fails,
+ * returning a particular value in errno.
+ */
+#define do_ioctl_err(fd, ioc, ioc_data, err) do { \
+	igt_assert_eq(igt_ioctl((fd), (ioc), (ioc_data)), -1); \
+	igt_assert_eq(errno, err); \
 	errno = 0; \
 } while (0)
 
