@@ -50,6 +50,7 @@
 #include <assert.h>
 
 #include <proc/readproc.h>
+#include <libudev.h>
 
 #include "drmtest.h"
 #include "i915_drm.h"
@@ -425,8 +426,6 @@ void igt_stop_shrink_helper(void)
 	igt_stop_helper(&shrink_helper);
 }
 
-#ifdef HAVE_UDEV
-#include <libudev.h>
 
 static struct igt_helper_process hang_detector;
 static void __attribute__((noreturn))
@@ -506,17 +505,6 @@ void igt_stop_hang_detector(void)
 {
 	igt_stop_helper(&hang_detector);
 }
-#else
-void igt_fork_hang_detector(int fd)
-{
-	if (igt_only_list_subtests())
-		return;
-}
-
-void igt_stop_hang_detector(void)
-{
-}
-#endif
 
 /**
  * igt_check_boolean_env_var:
@@ -577,6 +565,22 @@ void igt_exchange_int(void *array, unsigned i, unsigned j)
 	int_arr[j] = tmp;
 }
 
+/**
+ * igt_exchange_int64:
+ * @array: pointer to the array of int64_t
+ * @i: first position
+ * @j: second position
+ *
+ * Exchanges the two values at array indices @i and @j. Useful as an exchange
+ * function for igt_permute_array().
+ */
+void igt_exchange_int64(void *array, unsigned i, unsigned j)
+{
+	int64_t *a = array;
+
+	igt_swap(a[i], a[j]);
+}
+
 /**
  * igt_permute_array:
  * @array: pointer to array
