@@ -70,6 +70,7 @@
 #include "igt_core.h"
 #include "igt_aux.h"
 #include "igt_sysfs.h"
+#include "igt_sysrq.h"
 #include "igt_rc.h"
 
 #define UNW_LOCAL_ONLY
@@ -386,16 +387,7 @@ void igt_log_buffer_inspect(igt_buffer_log_handler_t check, void *data)
 	pthread_mutex_unlock(&log_buffer_mutex);
 }
 
-__attribute__((format(printf, 1, 2)))
-static void kmsg(const char *format, ...)
-#define KERN_EMER	"<0>"
-#define KERN_ALERT	"<1>"
-#define KERN_CRIT	"<2>"
-#define KERN_ERR	"<3>"
-#define KERN_WARNING	"<4>"
-#define KERN_NOTICE	"<5>"
-#define KERN_INFO	"<6>"
-#define KERN_DEBUG	"<7>"
+void igt_kmsg(const char *format, ...)
 {
 	va_list ap;
 	FILE *file;
@@ -525,7 +517,7 @@ bool igt_exit_called;
 static void common_exit_handler(int sig)
 {
 	if (!igt_only_list_subtests()) {
-		kick_fbcon(true);
+		bind_fbcon(true);
 	}
 
 	/* When not killed by a signal check that igt_exit() has been properly
@@ -810,8 +802,8 @@ out:
 		exit(ret == -1 ? 0 : IGT_EXIT_INVALID);
 
 	if (!list_subtests) {
-		kick_fbcon(false);
-		kmsg(KERN_INFO "[IGT] %s: executing\n", command_str);
+		bind_fbcon(false);
+		igt_kmsg(KMSG_INFO "%s: executing\n", command_str);
 		print_version();
 
 		sync();
@@ -938,7 +930,8 @@ bool __igt_run_subtest(const char *subtest_name)
 		return false;
 	}
 
-	kmsg(KERN_INFO "[IGT] %s: starting subtest %s\n", command_str, subtest_name);
+	igt_kmsg(KMSG_INFO "%s: starting subtest %s\n",
+		 command_str, subtest_name);
 	igt_debug("Starting subtest: %s\n", subtest_name);
 
 	_igt_log_buffer_reset();
@@ -1144,6 +1137,28 @@ void igt_fail(int exitcode)
 	}
 }
 
+/**
+ * igt_fatal_error: Stop test execution on fatal errors
+ *
+ * Stop test execution or optionally, if the IGT_REBOOT_ON_FATAL_ERROR
+ * environment variable is set, reboot the machine.
+ *
+ * Since out test runner (piglit) does support fatal test exit codes, we
+ * implement the default behaviour by waiting endlessly.
+ */
+void  __attribute__((noreturn)) igt_fatal_error(void)
+{
+	if (igt_check_boolean_env_var("IGT_REBOOT_ON_FATAL_ERROR", false)) {
+		igt_warn("FATAL ERROR - REBOOTING\n");
+		igt_sysrq_reboot();
+	} else {
+		igt_warn("FATAL ERROR\n");
+		for (;;)
+			pause();
+	}
+}
+
+
 /**
  * igt_can_fail:
  *
@@ -1447,8 +1462,8 @@ void igt_exit(void)
 	}
 
 	if (command_str)
-		kmsg(KERN_INFO "[IGT] %s: exiting, ret=%d\n",
-		     command_str, igt_exitcode);
+		igt_kmsg(KMSG_INFO "%s: exiting, ret=%d\n",
+			 command_str, igt_exitcode);
 	igt_debug("Exiting with status code %d\n", igt_exitcode);
 
 	for (int c = 0; c < num_test_children; c++)
