@@ -110,16 +110,7 @@ static bool is_mountpoint(const char *path)
 	return dev != st.st_dev;
 }
 
-/**
- * igt_debugfs_mount:
- *
- * This attempts to locate where debugfs is mounted on the filesystem,
- * and if not found, will then try to mount debugfs at /sys/kernel/debug.
- *
- * Returns:
- * The path to the debugfs mount point (e.g. /sys/kernel/debug)
- */
-const char *igt_debugfs_mount(void)
+static const char *__igt_debugfs_mount(void)
 {
 	if (is_mountpoint("/sys/kernel/debug"))
 		return "/sys/kernel/debug";
@@ -133,6 +124,25 @@ const char *igt_debugfs_mount(void)
 	return "/sys/kernel/debug";
 }
 
+/**
+ * igt_debugfs_mount:
+ *
+ * This attempts to locate where debugfs is mounted on the filesystem,
+ * and if not found, will then try to mount debugfs at /sys/kernel/debug.
+ *
+ * Returns:
+ * The path to the debugfs mount point (e.g. /sys/kernel/debug)
+ */
+const char *igt_debugfs_mount(void)
+{
+	static const char *path;
+
+	if (!path)
+		path = __igt_debugfs_mount();
+
+	return path;
+}
+
 /**
  * igt_debugfs_path:
  * @device: fd of the device
@@ -315,12 +325,10 @@ bool igt_debugfs_search(int device, const char *filename, const char *substring)
 static bool igt_find_crc_mismatch(const igt_crc_t *a, const igt_crc_t *b,
 				  int *index)
 {
+	int nwords = min(a->n_words, b->n_words);
 	int i;
 
-	if (a->n_words != b->n_words)
-		return true;
-
-	for (i = 0; i < a->n_words; i++) {
+	for (i = 0; i < nwords; i++) {
 		if (a->crc[i] != b->crc[i]) {
 			if (index)
 				*index = i;
@@ -329,6 +337,12 @@ static bool igt_find_crc_mismatch(const igt_crc_t *a, const igt_crc_t *b,
 		}
 	}
 
+	if (a->n_words != b->n_words) {
+		if (index)
+			*index = i;
+		return true;
+	}
+
 	return false;
 }
 
@@ -706,15 +720,12 @@ static int read_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out)
 	bytes_read = read(pipe_crc->crc_fd, &buf, MAX_LINE_LEN);
 	igt_reset_timeout();
 
-	if (bytes_read < 0 && errno == EAGAIN)
-		igt_assert(pipe_crc->flags & O_NONBLOCK);
-
 	if (bytes_read < 0)
-		bytes_read = 0;
+		bytes_read = -errno;
+	else
+		buf[bytes_read] = '\0';
 
-	buf[bytes_read] = '\0';
-
-	if (bytes_read && !pipe_crc_init_from_string(pipe_crc, out, buf))
+	if (bytes_read > 0 && !pipe_crc_init_from_string(pipe_crc, out, buf))
 		return -EINVAL;
 
 	return bytes_read;
@@ -722,7 +733,9 @@ static int read_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out)
 
 static void read_one_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out)
 {
-	while (read_crc(pipe_crc, out) == 0)
+	int ret;
+
+	while ((ret = read_crc(pipe_crc, out)) <= 0 && ret != -EINVAL)
 		usleep(1000);
 }
 
@@ -795,10 +808,11 @@ igt_pipe_crc_get_crcs(igt_pipe_crc_t *pipe_crc, int n_crcs,
 		int ret;
 
 		ret = read_crc(pipe_crc, crc);
+		if (ret == -EAGAIN)
+			break;
+
 		if (ret < 0)
 			continue;
-		if (ret == 0)
-			break;
 
 		n++;
 	} while (n < n_crcs);
@@ -854,6 +868,68 @@ void igt_pipe_crc_collect_crc(igt_pipe_crc_t *pipe_crc, igt_crc_t *out_crc)
 	crc_sanity_checks(out_crc);
 }
 
+/**
+ * igt_pipe_crc_drain:
+ * @pipe_crc: pipe CRC object
+ *
+ * Discards all currently queued CRC values from @pipe_crc. This function does
+ * not block, and is useful to flush @pipe_crc. Afterwards you can get a fresh
+ * CRC with igt_pipe_crc_get_single().
+ */
+void igt_pipe_crc_drain(igt_pipe_crc_t *pipe_crc)
+{
+	int ret;
+	igt_crc_t crc;
+
+	fcntl(pipe_crc->crc_fd, F_SETFL, pipe_crc->flags | O_NONBLOCK);
+
+	do {
+		ret = read_crc(pipe_crc, &crc);
+	} while (ret > 0 || ret == -EINVAL);
+
+	fcntl(pipe_crc->crc_fd, F_SETFL, pipe_crc->flags);
+}
+
+/**
+ * igt_pipe_crc_get_single:
+ * @pipe_crc: pipe CRC object
+ * @crc: buffer pointer for the captured CRC value
+ *
+ * Read a single @crc from @pipe_crc. This function does not block
+ * when nonblocking CRC is requested, and will return false if no CRC
+ * can be captured.
+ *
+ * If opened in blocking mode it will always block until a new CRC is read, like
+ * igt_pipe_crc_collect_crc().
+ *
+ * Callers must start and stop the capturing themselves by calling
+ * igt_pipe_crc_start() and igt_pipe_crc_stop(). For one-shot CRC collecting
+ * look at igt_pipe_crc_collect_crc().
+ *
+ * If capturing has been going on for a while and a fresh crc is required,
+ * you will need to call igt_pipe_crc_drain() first to remove stale entries.
+ *
+ * Returns:
+ * Whether a crc is captured, only false in non-blocking mode.
+ */
+bool
+igt_pipe_crc_get_single(igt_pipe_crc_t *pipe_crc, igt_crc_t *crc)
+{
+	bool found = true;
+
+	if (pipe_crc->flags & O_NONBLOCK) {
+		int ret = read_crc(pipe_crc, crc);
+
+		found = ret > 0;
+	} else
+		read_one_crc(pipe_crc, crc);
+
+	if (found)
+		crc_sanity_checks(crc);
+
+	return found;
+}
+
 /*
  * Drop caches
  */
