@@ -126,11 +126,10 @@ void igt_require_hang_ring(int fd, int ring)
 
 static unsigned context_get_ban(int fd, unsigned ctx)
 {
-	struct drm_i915_gem_context_param param;
-
-	param.param = I915_CONTEXT_PARAM_BANNABLE;
-	param.value = 0;
-	param.size = 0;
+	struct drm_i915_gem_context_param param = {
+		.ctx_id = ctx,
+		.param = I915_CONTEXT_PARAM_BANNABLE,
+	};
 
 	if (__gem_context_get_param(fd, &param) == -EINVAL) {
 		igt_assert(param.value == 0);
@@ -143,13 +142,11 @@ static unsigned context_get_ban(int fd, unsigned ctx)
 
 static void context_set_ban(int fd, unsigned ctx, unsigned ban)
 {
-	struct drm_i915_gem_context_param param;
-
-	memset(&param, 0, sizeof(param));
-	param.ctx_id = ctx;
-	param.value = ban;
-	param.size = 0;
-	param.param = I915_CONTEXT_PARAM_BANNABLE;
+	struct drm_i915_gem_context_param param = {
+		.ctx_id = ctx,
+		.param = I915_CONTEXT_PARAM_BANNABLE,
+		.value = ban,
+	};
 
 	if(__gem_context_set_param(fd, &param) == -EINVAL) {
 		igt_assert(param.value == ban);
@@ -160,7 +157,9 @@ static void context_set_ban(int fd, unsigned ctx, unsigned ban)
 
 igt_hang_t igt_allow_hang(int fd, unsigned ctx, unsigned flags)
 {
-	struct drm_i915_gem_context_param param;
+	struct drm_i915_gem_context_param param = {
+		.ctx_id = ctx,
+	};
 	unsigned ban;
 
 	igt_assert(igt_sysfs_set_parameter
@@ -172,9 +171,6 @@ igt_hang_t igt_allow_hang(int fd, unsigned ctx, unsigned flags)
 	if (!igt_check_boolean_env_var("IGT_HANG_WITHOUT_RESET", false))
 		igt_require(has_gpu_reset(fd));
 
-	param.ctx_id = ctx;
-	param.size = 0;
-
 	if ((flags & HANG_ALLOW_CAPTURE) == 0) {
 		param.param = I915_CONTEXT_PARAM_NO_ERROR_CAPTURE;
 		param.value = 1;
@@ -194,7 +190,6 @@ igt_hang_t igt_allow_hang(int fd, unsigned ctx, unsigned flags)
 
 void igt_disallow_hang(int fd, igt_hang_t arg)
 {
-
 	context_set_ban(fd, arg.ctx, arg.ban);
 
 	if ((arg.flags & HANG_ALLOW_CAPTURE) == 0) {
@@ -276,6 +271,7 @@ igt_hang_t igt_hang_ctx(int fd,
 	uint32_t b[16];
 	unsigned ban;
 	unsigned len;
+	int gen;
 
 	igt_require_hang_ring(fd, ring);
 
@@ -296,7 +292,6 @@ igt_hang_t igt_hang_ctx(int fd,
 	}
 
 	ban = context_get_ban(fd, ctx);
-
 	if ((flags & HANG_ALLOW_BAN) == 0)
 		context_set_ban(fd, ctx, 0);
 
@@ -310,12 +305,26 @@ igt_hang_t igt_hang_ctx(int fd,
 
 	memset(b, 0xc5, sizeof(b));
 
-	len = 2;
-	if (intel_gen(intel_get_drm_devid(fd)) >= 8)
+	len = 0;
+	gen = intel_gen(intel_get_drm_devid(fd));
+	if (gen >= 8) {
+		b[len++] = MI_BATCH_BUFFER_START | 1 << 8 | 1;
+		b[len++] = 0;
+		b[len++] = 0;
+	} else if (gen >= 6) {
+		b[len++] = MI_BATCH_BUFFER_START | 1 << 8;
+		b[len++] = 0;
+	} else {
+		b[len++] = MI_BATCH_BUFFER_START | 2 << 6;
+		b[len] = 0;
+		if (gen < 4) {
+			b[len] |= 1;
+			reloc.delta = 1;
+		}
 		len++;
-	b[0] = MI_BATCH_BUFFER_START | (len - 2);
-	b[len] = MI_BATCH_BUFFER_END;
-	b[len+1] = MI_NOOP;
+	}
+	b[len++] = MI_BATCH_BUFFER_END;
+	b[len] = MI_NOOP;
 	gem_write(fd, exec.handle, 0, b, sizeof(b));
 
 	reloc.offset = sizeof(uint32_t);
@@ -364,8 +373,7 @@ void igt_post_hang_ring(int fd, igt_hang_t arg)
 	if (arg.handle == 0)
 		return;
 
-	gem_set_domain(fd, arg.handle,
-		       I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
+	gem_sync(fd, arg.handle);
 	gem_close(fd, arg.handle);
 
 	context_set_ban(fd, arg.ctx, arg.ban);
@@ -402,6 +410,7 @@ void igt_force_gpu_reset(int drm_fd)
 	igt_sysfs_scanf(dir, "i915_wedged", "%d", &wedged);
 
 	close(dir);
+	errno = 0;
 
 	igt_assert(!wedged);
 }
@@ -659,3 +668,26 @@ bool gem_has_engine(int gem_fd,
 			    gem_class_instance_to_eb_flags(gem_fd, class,
 							   instance));
 }
+
+bool gem_ring_is_physical_engine(int fd, unsigned ring)
+{
+	if (ring == I915_EXEC_DEFAULT)
+		return false;
+
+	/* BSD uses an extra flag to chose between aliasing modes */
+	if ((ring & 63) == I915_EXEC_BSD) {
+		bool explicit_bsd = ring & (3 << 13);
+		bool has_bsd2 = gem_has_bsd2(fd);
+		return explicit_bsd ? has_bsd2 : !has_bsd2;
+	}
+
+	return true;
+}
+
+bool gem_ring_has_physical_engine(int fd, unsigned ring)
+{
+	if (!gem_ring_is_physical_engine(fd, ring))
+		return false;
+
+	return gem_has_ring(fd, ring);
+}
