@@ -47,7 +47,9 @@
  * @PIPE_A: First crtc.
  * @PIPE_B: Second crtc.
  * @PIPE_C: Third crtc.
- * ... and so on.
+ * @PIPE_D: Fourth crtc.
+ * @PIPE_E: Fifth crtc.
+ * @PIPE_F: Sixth crtc.
  * @IGT_MAX_PIPES: Max number of pipes allowed.
  */
 enum pipe {
@@ -495,11 +497,12 @@ static inline bool igt_output_is_connected(igt_output_t *output)
 #define IGT_FIXED(i,f)	((i) << 16 | (f))
 
 /**
- * igt_plane_has_prop - Check whether plane supports a given property.
- *
+ * igt_plane_has_prop:
  * @plane: Plane to check.
  * @prop: Property to check.
  *
+ * Check whether plane supports a given property.
+ *
  * Returns: True if the property is supported, otherwise false.
  */
 static inline bool
@@ -530,11 +533,12 @@ extern void igt_plane_replace_prop_blob(igt_plane_t *plane,
 					const void *ptr, size_t length);
 
 /**
- * igt_output_has_prop - Check whether output supports a given property.
- *
+ * igt_output_has_prop:
  * @output: Output to check.
  * @prop: Property to check.
  *
+ * Check whether output supports a given property.
+ *
  * Returns: True if the property is supported, otherwise false.
  */
 static inline bool
@@ -564,11 +568,12 @@ extern void igt_output_replace_prop_blob(igt_output_t *output,
 					 const void *ptr, size_t length);
 
 /**
- * igt_pipe_obj_has_prop - Check whether pipe supports a given property.
- *
+ * igt_pipe_obj_has_prop:
  * @pipe: Pipe to check.
  * @prop: Property to check.
  *
+ * Check whether pipe supports a given property.
+ *
  * Returns: True if the property is supported, otherwise false.
  */
 static inline bool
@@ -580,12 +585,13 @@ igt_pipe_obj_has_prop(igt_pipe_t *pipe, enum igt_atomic_crtc_properties prop)
 uint64_t igt_pipe_obj_get_prop(igt_pipe_t *pipe, enum igt_atomic_crtc_properties prop);
 
 /**
- * igt_pipe_get_prop - Return current value on a pipe for a given property.
- *
+ * igt_pipe_get_prop:
  * @display: Pointer to display.
  * @pipe: Target pipe.
  * @prop: Property to return.
  *
+ * Return current value on a pipe for a given property.
+ *
  * Returns: The value the property is set to, if this
  * is a blob, the blob id is returned. This can be passed
  * to drmModeGetPropertyBlob() to get the contents of the blob.
@@ -598,12 +604,13 @@ igt_pipe_get_prop(igt_display_t *display, enum pipe pipe,
 }
 
 /**
- * igt_pipe_has_prop - Check whether pipe supports a given property.
- *
+ * igt_pipe_has_prop:
  * @display: Pointer to display.
  * @pipe: Pipe to check.
  * @prop: Property to check.
  *
+ * Check whether pipe supports a given property.
+ *
  * Returns: True if the property is supported, otherwise false.
  */
 static inline bool
@@ -640,7 +647,7 @@ igt_pipe_has_prop(igt_display_t *display, enum pipe pipe,
 #define igt_pipe_set_prop_value(display, pipe, prop, value) \
 	igt_pipe_obj_set_prop_value(&(display)->pipes[(pipe)], prop, value)
 
-extern void igt_pipe_obj_replace_prop_blob(igt_pipe_t *pipe_obj,
+extern void igt_pipe_obj_replace_prop_blob(igt_pipe_t *pipe,
 					   enum igt_atomic_crtc_properties prop,
 					   const void *ptr, size_t length);
 
@@ -658,12 +665,10 @@ uint32_t kmstest_get_vbl_flag(uint32_t pipe_id);
 const unsigned char* igt_kms_get_base_edid(void);
 const unsigned char* igt_kms_get_alt_edid(void);
 
-#ifdef HAVE_UDEV
 struct udev_monitor *igt_watch_hotplug(void);
 bool igt_hotplug_detected(struct udev_monitor *mon,
 			  int timeout_secs);
 void igt_flush_hotplugs(struct udev_monitor *mon);
 void igt_cleanup_hotplug(struct udev_monitor *mon);
-#endif
 
 #endif /* __IGT_KMS_H__ */
