@@ -208,10 +208,10 @@ static int __gem_set_caching(int fd, uint32_t handle, uint32_t caching)
 	arg.caching = caching;
 
 	err = 0;
-	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_SET_CACHING, &arg)) {
+	if (igt_ioctl(fd, DRM_IOCTL_I915_GEM_SET_CACHING, &arg))
 		err = -errno;
-		igt_assert(errno == ENOTTY || errno == EINVAL);
-	}
+
+	errno = 0;
 	return err;
 }
 
@@ -229,7 +229,6 @@ static int __gem_set_caching(int fd, uint32_t handle, uint32_t caching)
 void gem_set_caching(int fd, uint32_t handle, uint32_t caching)
 {
 	igt_require(__gem_set_caching(fd, handle, caching) == 0);
-	errno = 0;
 }
 
 /**
@@ -1405,6 +1404,22 @@ void gem_require_caching(int fd)
 	errno = 0;
 }
 
+static void reset_device(int fd)
+{
+	int dir;
+
+	dir = igt_debugfs_dir(fd);
+	igt_require(dir >= 0);
+
+	if (ioctl(fd, DRM_IOCTL_I915_GEM_THROTTLE)) {
+		igt_info("Found wedged device, trying to reset and continue\n");
+		igt_sysfs_set(dir, "i915_wedged", "-1");
+	}
+	igt_sysfs_set(dir, "i915_next_seqno", "1");
+
+	close(dir);
+}
+
 void igt_require_gem(int fd)
 {
 	char path[256];
@@ -1427,15 +1442,12 @@ void igt_require_gem(int fd)
 	 * itself, which avoids accidentally hanging when setting up long
 	 * sequences of batches.
 	 */
-	err = igt_debugfs_dir(fd);
-	if (err != -1) {
-		igt_sysfs_printf(err, "i915_next_seqno", "1");
-		close(err);
-	}
+	reset_device(fd);
 
 	err = 0;
 	if (ioctl(fd, DRM_IOCTL_I915_GEM_THROTTLE))
 		err = -errno;
+
 	close(fd);
 
 	igt_require_f(err == 0, "Unresponsive i915/GEM device\n");
@@ -1659,10 +1671,10 @@ void igt_require_fb_modifiers(int fd)
 
 int __kms_addfb(int fd, uint32_t handle, uint32_t width, uint32_t height,
 		uint32_t stride, uint32_t pixel_format, uint64_t modifier,
-		uint32_t flags, uint32_t *buf_id)
+		uint32_t *offsets, uint32_t flags, uint32_t *buf_id)
 {
 	struct drm_mode_fb_cmd2 f;
-	int ret;
+	int ret, i;
 
 	igt_require_fb_modifiers(fd);
 
@@ -1676,6 +1688,13 @@ int __kms_addfb(int fd, uint32_t handle, uint32_t width, uint32_t height,
 	f.pitches[0] = stride;
 	f.modifier[0] = modifier;
 
+	for (i = 1; i < 4 && offsets && offsets[i]; i++) {
+		f.handles[i] = handle;
+		f.pitches[i] = stride;
+		f.modifier[i] = modifier;
+		f.offsets[i] = offsets[i];
+	}
+
 	ret = igt_ioctl(fd, DRM_IOCTL_MODE_ADDFB2, &f);
 
 	*buf_id = f.fb_id;
