@@ -45,6 +45,7 @@
  * GPU will hang.
  */
 
+#include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -56,13 +57,7 @@
 
 #include <drm.h>
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
 #include "intel_bufmgr.h"
-#include "intel_batchbuffer.h"
-#include "intel_io.h"
-#include "intel_chipset.h"
-#include "igt_aux.h"
 
 IGT_TEST_DESCRIPTION("Test the relocations through the CPU domain.");
 
@@ -79,7 +74,7 @@ static void copy(int fd, uint32_t batch, uint32_t src, uint32_t dst)
 	gem_reloc[0].target_handle = dst;
 	gem_reloc[0].read_domains = I915_GEM_DOMAIN_RENDER;
 	gem_reloc[0].write_domain = I915_GEM_DOMAIN_RENDER;
-	gem_reloc[0].presumed_offset = 0;
+	gem_reloc[0].presumed_offset = -1;
 
 	gem_reloc[1].offset = 7 * sizeof(uint32_t);
 	if (intel_gen(intel_get_drm_devid(fd)) >= 8)
@@ -88,22 +83,22 @@ static void copy(int fd, uint32_t batch, uint32_t src, uint32_t dst)
 	gem_reloc[1].target_handle = src;
 	gem_reloc[1].read_domains = I915_GEM_DOMAIN_RENDER;
 	gem_reloc[1].write_domain = 0;
-	gem_reloc[1].presumed_offset = 0;
+	gem_reloc[1].presumed_offset = -1;
 
 	memset(gem_exec, 0, sizeof(gem_exec));
 	gem_exec[0].handle = src;
 	gem_exec[1].handle = dst;
 	gem_exec[2].handle = batch;
 	gem_exec[2].relocation_count = 2;
-	gem_exec[2].relocs_ptr = (uintptr_t)gem_reloc;
+	gem_exec[2].relocs_ptr = to_user_pointer(gem_reloc);
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)gem_exec;
+	execbuf.buffers_ptr = to_user_pointer(gem_exec);
 	execbuf.buffer_count = 3;
 	execbuf.batch_len = 4096;
 	execbuf.flags = use_blt;
 
-	do_or_die(drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &execbuf));
+	gem_execbuf(fd, &execbuf);
 }
 
 static void exec(int fd, uint32_t handle)
@@ -115,11 +110,11 @@ static void exec(int fd, uint32_t handle)
 	gem_exec.handle = handle;
 
 	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = (uintptr_t)&gem_exec;
+	execbuf.buffers_ptr = to_user_pointer(&gem_exec);
 	execbuf.buffer_count = 1;
 	execbuf.batch_len = 4096;
 
-	do_or_die(drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &execbuf));
+	gem_execbuf(fd, &execbuf);
 }
 
 uint32_t gen6_batch[] = {
@@ -159,16 +154,14 @@ uint32_t gen8_batch[] = {
 uint32_t *batch = gen6_batch;
 uint32_t batch_size = sizeof(gen6_batch);
 
-igt_simple_main
+static void run_test(int fd, int count)
 {
 	const uint32_t hang[] = {-1, -1, -1, -1};
 	const uint32_t end[] = {MI_BATCH_BUFFER_END, 0};
-	uint64_t aper_size;
 	uint32_t noop;
 	uint32_t *handles;
-	int fd, i, count;
+	int i;
 
-	fd = drm_open_any();
 	noop = intel_get_drm_devid(fd);
 
 	use_blt = 0;
@@ -180,13 +173,6 @@ igt_simple_main
 		batch_size += 2 * 4;
 	}
 
-	aper_size = gem_mappable_aperture_size();
-	count = aper_size / 4096 * 2;
-	if (igt_run_in_simulation())
-		count = 10;
-
-	intel_require_memory(1+count, 4096, CHECK_RAM);
-
 	handles = malloc (count * sizeof(uint32_t));
 	igt_assert(handles);
 
@@ -246,6 +232,41 @@ igt_simple_main
 		igt_progress("gem_cpu_reloc: ", 2*count+i, 3*count);
 	}
 
-	igt_info("Test suceeded, cleanup up - this might take a while.\n");
-	close(fd);
+	igt_info("Subtest suceeded, cleanup up - this might take a while.\n");
+	for (i = 0; i < count; i++) {
+		gem_close(fd, handles[i]);
+	}
+	gem_close(fd, noop);
+	free(handles);
+}
+
+igt_main
+{
+	uint64_t aper_size;
+	int fd, count;
+
+	igt_fixture {
+		fd = drm_open_driver(DRIVER_INTEL);
+		igt_require_gem(fd);
+	}
+
+	igt_subtest("basic") {
+		run_test (fd, 10);
+	}
+
+
+	igt_subtest("full") {
+		aper_size = gem_mappable_aperture_size();
+		count = aper_size / 4096 * 2;
+
+		/* count + 2 (noop & bad) buffers. A gem object appears to
+                   require about 2kb + buffer + kernel overhead */
+		intel_require_memory(2+count, 2048+4096, CHECK_RAM);
+
+		run_test (fd, count);
+	}
+
+	igt_fixture {
+		close(fd);
+	}
 }
