@@ -25,6 +25,7 @@
 #include "igt_sysfs.h"
 #include "igt_vgem.h"
 #include "sw_sync.h"
+#include "i915/gem_ring.h"
 
 #include <sys/ioctl.h>
 #include <sys/poll.h>
@@ -273,7 +274,7 @@ static void test_fence_await(int fd, unsigned ring, unsigned flags)
 	igt_assert(fence != -1);
 
 	i = 0;
-	for_each_engine(fd, engine) {
+	for_each_physical_engine(fd, engine) {
 		if (!gem_can_store_dword(fd, engine))
 			continue;
 
@@ -321,99 +322,20 @@ static void resubmit(int fd, uint32_t handle, unsigned int ring, int count)
 		gem_execbuf(fd, &execbuf);
 }
 
-struct cork {
-	int device;
-	uint32_t handle;
-	uint32_t fence;
-};
-
-static void plug(int fd, struct cork *c)
-{
-	struct vgem_bo bo;
-	int dmabuf;
-
-	c->device = drm_open_driver(DRIVER_VGEM);
-
-	bo.width = bo.height = 1;
-	bo.bpp = 4;
-	vgem_create(c->device, &bo);
-	c->fence = vgem_fence_attach(c->device, &bo, VGEM_FENCE_WRITE);
-
-	dmabuf = prime_handle_to_fd(c->device, bo.handle);
-	c->handle = prime_fd_to_handle(fd, dmabuf);
-	close(dmabuf);
-}
-
-static void unplug(int fd, struct cork *c)
-{
-	vgem_fence_signal(c->device, c->fence);
-	gem_close(fd, c->handle);
-	close(c->device);
-}
-
 static void alarm_handler(int sig)
 {
 }
 
 static int __execbuf(int fd, struct drm_i915_gem_execbuffer2 *execbuf)
 {
-	return ioctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, execbuf);
-}
+	int err;
 
-static unsigned int measure_ring_size(int fd)
-{
-	struct sigaction sa = { .sa_handler = alarm_handler };
-	struct drm_i915_gem_exec_object2 obj[2];
-	struct drm_i915_gem_execbuffer2 execbuf;
-	const uint32_t bbe = MI_BATCH_BUFFER_END;
-	unsigned int count, last;
-	struct itimerval itv;
-	struct cork c;
+	err = 0;
+	if (ioctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2_WR, execbuf))
+		err = -errno;
 
-	memset(obj, 0, sizeof(obj));
-	obj[1].handle = gem_create(fd, 4096);
-	gem_write(fd, obj[1].handle, 0, &bbe, sizeof(bbe));
-
-	memset(&execbuf, 0, sizeof(execbuf));
-	execbuf.buffers_ptr = to_user_pointer(&obj[1]);
-	execbuf.buffer_count = 1;
-	gem_execbuf(fd, &execbuf);
-	gem_sync(fd, obj[1].handle);
-
-	plug(fd, &c);
-	obj[0].handle = c.handle;
-
-	execbuf.buffers_ptr = to_user_pointer(obj);
-	execbuf.buffer_count = 2;
-
-	sigaction(SIGALRM, &sa, NULL);
-	itv.it_interval.tv_sec = 0;
-	itv.it_interval.tv_usec = 100;
-	itv.it_value.tv_sec = 0;
-	itv.it_value.tv_usec = 1000;
-	setitimer(ITIMER_REAL, &itv, NULL);
-
-	last = -1;
-	count = 0;
-	do {
-		if (__execbuf(fd, &execbuf) == 0) {
-			count++;
-			continue;
-		}
-
-		if (last == count)
-			break;
-
-		last = count;
-	} while (1);
-
-	memset(&itv, 0, sizeof(itv));
-	setitimer(ITIMER_REAL, &itv, NULL);
-
-	unplug(fd, &c);
-	gem_close(fd, obj[1].handle);
-
-	return count;
+	errno = 0;
+	return err;
 }
 
 static void test_parallel(int fd, unsigned int master)
@@ -430,15 +352,16 @@ static void test_parallel(int fd, unsigned int master)
 	uint32_t batch[16];
 	igt_spin_t *spin;
 	unsigned engine;
-	struct cork c;
+	IGT_CORK_HANDLE(c);
+	uint32_t plug;
 	int i, x = 0;
 
-	plug(fd, &c);
+	plug = igt_cork_plug(&c, fd);
 
 	/* Fill the queue with many requests so that the next one has to
 	 * wait before it can be executed by the hardware.
 	 */
-	spin = igt_spin_batch_new(fd, 0, master, c.handle);
+	spin = igt_spin_batch_new(fd, 0, master, plug);
 	resubmit(fd, spin->handle, master, 16);
 
 	/* Now queue the master request and its secondaries */
@@ -514,10 +437,7 @@ static void test_parallel(int fd, unsigned int master)
 	obj[BATCH].relocation_count = 1;
 
 	/* Queue all secondaries */
-	for_each_engine(fd, engine) {
-		if (engine == 0 || engine == I915_EXEC_BSD)
-			continue;
-
+	for_each_physical_engine(fd, engine) {
 		if (engine == master)
 			continue;
 
@@ -559,7 +479,8 @@ static void test_parallel(int fd, unsigned int master)
 	}
 
 	/* Unblock the master */
-	unplug(fd, &c);
+	igt_cork_unplug(&c);
+	gem_close(fd, plug);
 	igt_spin_batch_end(spin);
 
 	/* Wait for all secondaries to complete. If we used a regular fence
@@ -586,6 +507,94 @@ static void test_parallel(int fd, unsigned int master)
 	gem_close(fd, handle[0]);
 }
 
+static uint32_t batch_create(int fd)
+{
+	const uint32_t bbe = MI_BATCH_BUFFER_END;
+	uint32_t handle;
+
+	handle = gem_create(fd, 4096);
+	gem_write(fd, handle, 0, &bbe, sizeof(bbe));
+
+	return handle;
+}
+
+static inline uint32_t lower_32_bits(uint64_t x)
+{
+	return x & 0xffffffff;
+}
+
+static inline uint32_t upper_32_bits(uint64_t x)
+{
+	return x >> 32;
+}
+
+static void test_keep_in_fence(int fd, unsigned int engine, unsigned int flags)
+{
+	struct sigaction sa = { .sa_handler = alarm_handler };
+	struct drm_i915_gem_exec_object2 obj = {
+		.handle = batch_create(fd),
+	};
+	struct drm_i915_gem_execbuffer2 execbuf = {
+		.buffers_ptr = to_user_pointer(&obj),
+		.buffer_count = 1,
+		.flags = engine | LOCAL_EXEC_FENCE_OUT,
+	};
+	unsigned long count, last;
+	struct itimerval itv;
+	igt_spin_t *spin;
+	int fence;
+
+	spin = igt_spin_batch_new(fd, 0, engine, 0);
+
+	gem_execbuf_wr(fd, &execbuf);
+	fence = upper_32_bits(execbuf.rsvd2);
+
+	sigaction(SIGALRM, &sa, NULL);
+	itv.it_interval.tv_sec = 0;
+	itv.it_interval.tv_usec = 1000;
+	itv.it_value.tv_sec = 0;
+	itv.it_value.tv_usec = 10000;
+	setitimer(ITIMER_REAL, &itv, NULL);
+
+	execbuf.flags |= LOCAL_EXEC_FENCE_IN;
+	execbuf.rsvd2 = fence;
+
+	last = -1;
+	count = 0;
+	do {
+		int err = __execbuf(fd, &execbuf);
+
+		igt_assert_eq(lower_32_bits(execbuf.rsvd2), fence);
+
+		if (err == 0) {
+			close(fence);
+
+			fence = upper_32_bits(execbuf.rsvd2);
+			execbuf.rsvd2 = fence;
+
+			count++;
+			continue;
+		}
+
+		igt_assert_eq(err, -EINTR);
+		igt_assert_eq(upper_32_bits(execbuf.rsvd2), 0);
+
+		if (last == count)
+			break;
+
+		last = count;
+	} while (1);
+
+	memset(&itv, 0, sizeof(itv));
+	setitimer(ITIMER_REAL, &itv, NULL);
+
+	gem_close(fd, obj.handle);
+	close(fence);
+
+	igt_spin_batch_free(fd, spin);
+	gem_quiescent_gpu(fd);
+}
+
 #define EXPIRED 0x10000
 static void test_long_history(int fd, long ring_size, unsigned flags)
 {
@@ -597,22 +606,15 @@ static void test_long_history(int fd, long ring_size, unsigned flags)
 	unsigned int nengine, n, s;
 	unsigned long limit;
 	int all_fences;
-	struct cork c;
+	IGT_CORK_HANDLE(c);
 
 	limit = -1;
 	if (!gem_uses_full_ppgtt(fd))
 		limit = ring_size / 3;
 
 	nengine = 0;
-	for_each_engine(fd, engine) {
-		if (engine == 0)
-			continue;
-
-		if (engine == I915_EXEC_BSD)
-			continue;
-
+	for_each_physical_engine(fd, engine)
 		engines[nengine++] = engine;
-	}
 	igt_require(nengine);
 
 	gem_quiescent_gpu(fd);
@@ -632,8 +634,7 @@ static void test_long_history(int fd, long ring_size, unsigned flags)
 	execbuf.buffers_ptr = to_user_pointer(obj);
 	execbuf.buffer_count = 2;
 
-	plug(fd, &c);
-	obj[0].handle = c.handle;
+	obj[0].handle = igt_cork_plug(&c, fd);
 
 	igt_until_timeout(5) {
 		execbuf.rsvd1 = gem_context_create(fd);
@@ -661,7 +662,7 @@ static void test_long_history(int fd, long ring_size, unsigned flags)
 		if (!--limit)
 			break;
 	}
-	unplug(fd, &c);
+	igt_cork_unplug(&c);
 
 	igt_info("History depth = %d\n", sync_fence_count(all_fences));
 
@@ -685,6 +686,7 @@ static void test_long_history(int fd, long ring_size, unsigned flags)
 
 	gem_sync(fd, obj[1].handle);
 	gem_close(fd, obj[1].handle);
+	gem_close(fd, obj[0].handle);
 }
 
 static void test_fence_flip(int i915)
@@ -698,7 +700,7 @@ static bool has_submit_fence(int fd)
 	int value = 0;
 
 	memset(&gp, 0, sizeof(gp));
-	gp.param = 50; /* I915_PARAM_HAS_EXEC_SUBMIT_FENCE */
+	gp.param = 0xdeadbeef ^ 51; /* I915_PARAM_HAS_EXEC_SUBMIT_FENCE */
 	gp.value = &value;
 
 	ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp, sizeof(gp));
@@ -1490,6 +1492,9 @@ igt_main
 				igt_subtest_f("nb-await-%s", e->name)
 					test_fence_await(i915, e->exec_id | e->flags, NONBLOCK);
 
+				igt_subtest_f("keep-in-fence-%s", e->name)
+					test_keep_in_fence(i915, e->exec_id | e->flags, 0);
+
 				if (e->exec_id &&
 				    !(e->exec_id == I915_EXEC_BSD && !e->flags)) {
 					igt_subtest_f("parallel-%s", e->name) {
@@ -1528,22 +1533,22 @@ igt_main
 		}
 	}
 
-	igt_subtest("long-history") {
-		long ring_size = measure_ring_size(i915) - 1;
+	igt_subtest_group {
+		long ring_size = 0;
 
-		igt_info("Ring size: %ld batches\n", ring_size);
-		igt_require(ring_size);
+		igt_fixture {
+			ring_size = gem_measure_ring_inflight(i915, 0, 0) - 1;
+			igt_info("Ring size: %ld batches\n", ring_size);
+			igt_require(ring_size);
 
-		test_long_history(i915, ring_size, 0);
-	}
+			gem_require_contexts(i915);
+		}
 
-	igt_subtest("expired-history") {
-		long ring_size = measure_ring_size(i915) - 1;
+		igt_subtest("long-history")
+			test_long_history(i915, ring_size, 0);
 
-		igt_info("Ring size: %ld batches\n", ring_size);
-		igt_require(ring_size);
-
-		test_long_history(i915, ring_size, EXPIRED);
+		igt_subtest("expired-history")
+			test_long_history(i915, ring_size, EXPIRED);
 	}
 
 	igt_subtest("flip") {
