@@ -25,19 +25,14 @@
  *
  */
 
-#define _GNU_SOURCE
-
 #include <stdlib.h>
-#include <sys/ioctl.h>
-#include <stdio.h>
 #include <string.h>
-#include <fcntl.h>
 #include <inttypes.h>
-#include <drm.h>
-#include <i915_drm.h>
+#include <errno.h>
 
-#include "drmtest.h"
-#include "intel_gpu_tools.h"
+#include "igt.h"
+
+IGT_TEST_DESCRIPTION("Test execbuf fence accounting.");
 
 #define WIDTH 1024
 #define HEIGHT 1024
@@ -45,7 +40,7 @@
 
 #define BATCH_SIZE 4096
 
-#define MAX_FENCES 32
+#define MAX_FENCES 64
 
 /*
  * Testcase: execbuf fence accounting
@@ -84,18 +79,6 @@ batch_create (int fd)
 	return batch_handle;
 }
 
-static int get_num_fences(int fd)
-{
-	int val;
-
-	val = gem_available_fences(fd);
-
-	printf ("total %d fences\n", val);
-	igt_assert(val > 4);
-
-	return val;
-}
-
 static void fill_reloc(struct drm_i915_gem_relocation_entry *reloc, uint32_t handle)
 {
 	reloc->offset = 2 * sizeof(uint32_t);
@@ -104,14 +87,21 @@ static void fill_reloc(struct drm_i915_gem_relocation_entry *reloc, uint32_t han
 	reloc->write_domain = 0;
 }
 
-static void run_test(int fd, int num_fences, int expected_errno)
+#define BUSY_LOAD (1 << 0)
+#define INTERRUPTIBLE (1 << 1)
+
+static void run_test(int fd, int num_fences, int expected_errno,
+		     unsigned flags)
 {
 	struct drm_i915_gem_execbuffer2 execbuf[2];
-	struct drm_i915_gem_exec_object2 exec[2][2*MAX_FENCES+3];
-	struct drm_i915_gem_relocation_entry reloc[2*MAX_FENCES+2];
+	struct drm_i915_gem_exec_object2 exec[2][2*MAX_FENCES+1];
+	struct drm_i915_gem_relocation_entry reloc[2*MAX_FENCES];
 
+	unsigned long count;
 	int i, n;
-	int loop = 1000;
+
+	igt_assert(2*num_fences+1 <= ARRAY_SIZE(exec[0]));
+	igt_assert(2*num_fences <= ARRAY_SIZE(reloc));
 
 	memset(execbuf, 0, sizeof(execbuf));
 	memset(exec, 0, sizeof(exec));
@@ -128,52 +118,72 @@ static void run_test(int fd, int num_fences, int expected_errno)
 			exec[i][n].flags = EXEC_OBJECT_NEEDS_FENCE;
 
 		exec[i][2*num_fences].handle = batch_create(fd);
-		exec[i][2*num_fences].relocs_ptr = (uintptr_t)reloc;
+		exec[i][2*num_fences].relocs_ptr = to_user_pointer(reloc);
 		exec[i][2*num_fences].relocation_count = 2*num_fences;
 
-		execbuf[i].buffers_ptr = (uintptr_t)exec[i];
+		execbuf[i].buffers_ptr = to_user_pointer(exec[i]);
 		execbuf[i].buffer_count = 2*num_fences+1;
 		execbuf[i].batch_len = 2*sizeof(uint32_t);
 	}
 
-	do {
-		int ret;
+	count = 0;
+	igt_until_timeout(2) {
+		for (i = 0; i < 2; i++) {
+			igt_spin_t *spin = NULL;
 
-		ret = drmIoctl(fd,
-			       DRM_IOCTL_I915_GEM_EXECBUFFER2,
-			       &execbuf[0]);
-		igt_assert(expected_errno ?
-		       ret < 0 && errno == expected_errno :
-		       ret == 0);
+			if (flags & BUSY_LOAD)
+				spin = __igt_spin_batch_new(fd, 0, 0, 0);
 
-		ret = drmIoctl(fd,
-			       DRM_IOCTL_I915_GEM_EXECBUFFER2,
-			       &execbuf[1]);
-		igt_assert(expected_errno ?
-		       ret < 0 && errno == expected_errno :
-		       ret == 0);
-	} while (--loop);
+			igt_while_interruptible(flags & INTERRUPTIBLE) {
+				igt_assert_eq(__gem_execbuf(fd, &execbuf[i]),
+					      -expected_errno);
+			}
+
+			igt_spin_batch_free(fd, spin);
+			gem_quiescent_gpu(fd);
+		}
+		count++;
+	}
+	igt_info("Completed %lu cycles\n", count);
+
+	/* Cleanup */
+	for (n = 0; n < 2*num_fences; n++)
+		gem_close(fd, exec[0][n].handle);
+
+	for (i = 0; i < 2; i++)
+		gem_close(fd, exec[i][2*num_fences].handle);
 }
 
-int
-main(int argc, char **argv)
+igt_main
 {
-	int fd;
-	int num_fences;
-	uint32_t devid;
+	uint32_t devid = 0;
+	unsigned int num_fences = 0;
+	int fd = -1;
 
 	igt_skip_on_simulation();
 
-	fd = drm_open_any();
-	num_fences = get_num_fences(fd);
-	devid = intel_get_drm_devid(fd);
+	igt_fixture {
+		fd = drm_open_driver(DRIVER_INTEL);
+		igt_require_gem(fd);
 
-	igt_assert(num_fences <= MAX_FENCES);
+		num_fences = gem_available_fences(fd);
+		igt_assert(num_fences > 4);
+		igt_assert(num_fences <= MAX_FENCES);
 
-	run_test(fd, num_fences - 2, 0);
-	run_test(fd, num_fences + 1, intel_gen(devid) >= 4 ? 0 : EDEADLK);
+		devid = intel_get_drm_devid(fd);
+	}
 
-	close(fd);
+	igt_subtest("2-spare-fences")
+		run_test(fd, num_fences - 2, 0, 0);
+	for (unsigned flags = 0; flags < 4; flags++) {
+		igt_subtest_f("no-spare-fences%s%s",
+			      flags & BUSY_LOAD ? "-busy" : "",
+			      flags & INTERRUPTIBLE ? "-interruptible" : "")
+			run_test(fd, num_fences, 0, flags);
+	}
+	igt_subtest("too-many-fences")
+		run_test(fd, num_fences + 1, intel_gen(devid) >= 4 ? 0 : EDEADLK, 0);
 
-	return 0;
+	igt_fixture
+		close(fd);
 }
