@@ -26,6 +26,8 @@
  *
  */
 
+#include "igt.h"
+#include "igt_x86.h"
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -38,8 +40,6 @@
 #include <sys/ioctl.h>
 #include <sys/time.h>
 #include "drm.h"
-#include "i915_drm.h"
-#include "drmtest.h"
 
 #define OBJECT_SIZE 16384
 
@@ -50,35 +50,80 @@ static double elapsed(const struct timeval *start,
 	return (1e6*(end->tv_sec - start->tv_sec) + (end->tv_usec - start->tv_usec))/loop;
 }
 
+#if defined(__x86_64__) && !defined(__clang__)
+#pragma GCC push_options
+#pragma GCC target("sse4.1")
+#include <smmintrin.h>
+__attribute__((noinline))
+static void streaming_load(void *src, int len)
+{
+	__m128i tmp, *s = src;
+
+	igt_assert((len & 15) == 0);
+	igt_assert((((uintptr_t)src) & 15) == 0);
+
+	while (len >= 16) {
+		tmp += _mm_stream_load_si128(s++);
+		len -= 16;
+
+	}
+
+	*(volatile __m128i *)src = tmp;
+}
+static inline unsigned x86_64_features(void)
+{
+	return igt_x86_features();
+}
+#pragma GCC pop_options
+#else
+static inline unsigned x86_64_features(void)
+{
+	return 0;
+}
+static void streaming_load(void *src, int len)
+{
+	igt_assert(!"reached");
+}
+#endif
+
 int main(int argc, char **argv)
 {
 	struct timeval start, end;
 	uint8_t *buf;
 	uint32_t handle;
+	unsigned cpu = x86_64_features();
 	int size = OBJECT_SIZE;
 	int loop, i, tiling;
 	int fd;
 
+	igt_simple_init(argc, argv);
+
 	igt_skip_on_simulation();
 
 	if (argc > 1)
 		size = atoi(argv[1]);
 	if (size == 0) {
-		fprintf(stderr, "Invalid object size specified\n");
+		igt_warn("Invalid object size specified\n");
 		return 1;
 	}
 
+	if (cpu) {
+		char str[1024];
+		igt_info("Detected cpu faatures: %s\n",
+			 igt_x86_features_to_string(cpu, str));
+	}
+
 	buf = malloc(size);
 	memset(buf, 0, size);
-	fd = drm_open_any();
+	fd = drm_open_driver(DRIVER_INTEL);
 
 	handle = gem_create(fd, size);
 	igt_assert(handle);
 
 	for (tiling = I915_TILING_NONE; tiling <= I915_TILING_Y; tiling++) {
 		if (tiling != I915_TILING_NONE) {
-			printf("\nSetting tiling mode to %s\n",
-			       tiling == I915_TILING_X ? "X" : "Y");
+			igt_info("\nSetting tiling mode to %s\n",
+				 tiling == I915_TILING_X ? "X" : "Y");
 			gem_set_tiling(fd, handle, tiling, 512);
 		}
 
@@ -88,14 +133,14 @@ int main(int argc, char **argv)
 				       I915_GEM_DOMAIN_CPU);
 
 			{
-				uint32_t *base = gem_mmap__cpu(fd, handle, size, PROT_READ | PROT_WRITE);
+				uint32_t *base = gem_mmap__cpu(fd, handle, 0, size, PROT_READ | PROT_WRITE);
 				volatile uint32_t *ptr = base;
 				int x = 0;
 
 				for (i = 0; i < size/sizeof(*ptr); i++)
 					x += ptr[i];
 
-				/* force overtly clever gcc to actually compute x */
+				/* force overly clever gcc to actually compute x */
 				ptr[0] = x;
 
 				munmap(base, size);
@@ -103,26 +148,51 @@ int main(int argc, char **argv)
 				/* mmap read */
 				gettimeofday(&start, NULL);
 				for (loop = 0; loop < 1000; loop++) {
-					base = gem_mmap__cpu(fd, handle, size, PROT_READ | PROT_WRITE);
+					base = gem_mmap__cpu(fd, handle, 0,
+							     size,
+							     PROT_READ | PROT_WRITE);
 					ptr = base;
 					x = 0;
 
 					for (i = 0; i < size/sizeof(*ptr); i++)
 						x += ptr[i];
 
-					/* force overtly clever gcc to actually compute x */
+					/* force overly clever gcc to actually compute x */
 					ptr[0] = x;
 
 					munmap(base, size);
 				}
 				gettimeofday(&end, NULL);
-				printf("Time to read %dk through a CPU map:		%7.3fµs\n",
-				       size/1024, elapsed(&start, &end, loop));
+				igt_info("Time to read %dk through a CPU map:		%7.3fµs\n",
+					 size/1024, elapsed(&start, &end, loop));
+				{
+					base = gem_mmap__cpu(fd, handle, 0,
+							     size,
+							     PROT_READ | PROT_WRITE);
+					gettimeofday(&start, NULL);
+					for (loop = 0; loop < 1000; loop++) {
+						ptr = base;
+						x = 0;
+
+						for (i = 0; i < size/sizeof(*ptr); i++)
+							x += ptr[i];
+
+						/* force overly clever gcc to actually compute x */
+						ptr[0] = x;
+
+					}
+					gettimeofday(&end, NULL);
+					munmap(base, size);
+					igt_info("Time to read %dk through a cached CPU map:	%7.3fµs\n",
+						 size/1024, elapsed(&start, &end, loop));
+				}
 
 				/* mmap write */
 				gettimeofday(&start, NULL);
 				for (loop = 0; loop < 1000; loop++) {
-					base = gem_mmap__cpu(fd, handle, size, PROT_READ | PROT_WRITE);
+					base = gem_mmap__cpu(fd, handle, 0,
+							     size,
+							     PROT_READ | PROT_WRITE);
 					ptr = base;
 
 					for (i = 0; i < size/sizeof(*ptr); i++)
@@ -131,27 +201,30 @@ int main(int argc, char **argv)
 					munmap(base, size);
 				}
 				gettimeofday(&end, NULL);
-				printf("Time to write %dk through a CPU map:		%7.3fµs\n",
-				       size/1024, elapsed(&start, &end, loop));
+				igt_info("Time to write %dk through a CPU map:		%7.3fµs\n",
+					 size/1024, elapsed(&start, &end, loop));
 
 				gettimeofday(&start, NULL);
 				for (loop = 0; loop < 1000; loop++) {
-					base = gem_mmap__cpu(fd, handle, size, PROT_READ | PROT_WRITE);
+					base = gem_mmap__cpu(fd, handle, 0,
+							     size,
+							     PROT_READ | PROT_WRITE);
 					memset(base, 0, size);
 					munmap(base, size);
 				}
 				gettimeofday(&end, NULL);
-				printf("Time to clear %dk through a CPU map:		%7.3fµs\n",
-				       size/1024, elapsed(&start, &end, loop));
+				igt_info("Time to clear %dk through a CPU map:		%7.3fµs\n",
+					 size/1024, elapsed(&start, &end, loop));
 
 				gettimeofday(&start, NULL);
-				base = gem_mmap__cpu(fd, handle, size, PROT_READ | PROT_WRITE);
+				base = gem_mmap__cpu(fd, handle, 0, size,
+						     PROT_READ | PROT_WRITE);
 				for (loop = 0; loop < 1000; loop++)
 					memset(base, 0, size);
 				munmap(base, size);
 				gettimeofday(&end, NULL);
-				printf("Time to clear %dk through a cached CPU map:	%7.3fµs\n",
-				       size/1024, elapsed(&start, &end, loop));
+				igt_info("Time to clear %dk through a cached CPU map:	%7.3fµs\n",
+					 size/1024, elapsed(&start, &end, loop));
 			}
 
 			/* CPU pwrite */
@@ -159,28 +232,28 @@ int main(int argc, char **argv)
 			for (loop = 0; loop < 1000; loop++)
 				gem_write(fd, handle, 0, buf, size);
 			gettimeofday(&end, NULL);
-			printf("Time to pwrite %dk through the CPU:		%7.3fµs\n",
-			       size/1024, elapsed(&start, &end, loop));
+			igt_info("Time to pwrite %dk through the CPU:		%7.3fµs\n",
+				 size/1024, elapsed(&start, &end, loop));
 
 			/* CPU pread */
 			gettimeofday(&start, NULL);
 			for (loop = 0; loop < 1000; loop++)
 				gem_read(fd, handle, 0, buf, size);
 			gettimeofday(&end, NULL);
-			printf("Time to pread %dk through the CPU:		%7.3fµs\n",
-			       size/1024, elapsed(&start, &end, loop));
+			igt_info("Time to pread %dk through the CPU:		%7.3fµs\n",
+				 size/1024, elapsed(&start, &end, loop));
 		}
 
 		/* prefault into gtt */
 		{
-			uint32_t *base = gem_mmap(fd, handle, size, PROT_READ | PROT_WRITE);
+			uint32_t *base = gem_mmap__gtt(fd, handle, size, PROT_READ | PROT_WRITE);
 			volatile uint32_t *ptr = base;
 			int x = 0;
 
 			for (i = 0; i < size/sizeof(*ptr); i++)
 				x += ptr[i];
 
-			/* force overtly clever gcc to actually compute x */
+			/* force overly clever gcc to actually compute x */
 			ptr[0] = x;
 
 			munmap(base, size);
@@ -188,26 +261,92 @@ int main(int argc, char **argv)
 		/* mmap read */
 		gettimeofday(&start, NULL);
 		for (loop = 0; loop < 1000; loop++) {
-			uint32_t *base = gem_mmap(fd, handle, size, PROT_READ | PROT_WRITE);
+			uint32_t *base = gem_mmap__gtt(fd, handle, size, PROT_READ | PROT_WRITE);
 			volatile uint32_t *ptr = base;
 			int x = 0;
 
 			for (i = 0; i < size/sizeof(*ptr); i++)
 				x += ptr[i];
 
-			/* force overtly clever gcc to actually compute x */
+			/* force overly clever gcc to actually compute x */
 			ptr[0] = x;
 
 			munmap(base, size);
 		}
 		gettimeofday(&end, NULL);
-		printf("Time to read %dk through a GTT map:		%7.3fµs\n",
-		       size/1024, elapsed(&start, &end, loop));
+		igt_info("Time to read %dk through a GTT map:		%7.3fµs\n",
+			 size/1024, elapsed(&start, &end, loop));
+
+		if (gem_mmap__has_wc(fd)) {
+			gettimeofday(&start, NULL);
+			for (loop = 0; loop < 1000; loop++) {
+				uint32_t *base = gem_mmap__wc(fd, handle, 0, size, PROT_READ | PROT_WRITE);
+				volatile uint32_t *ptr = base;
+				int x = 0;
+
+				for (i = 0; i < size/sizeof(*ptr); i++)
+					x += ptr[i];
+
+				/* force overly clever gcc to actually compute x */
+				ptr[0] = x;
+
+				munmap(base, size);
+			}
+			gettimeofday(&end, NULL);
+			igt_info("Time to read %dk through a WC map:		%7.3fµs\n",
+					size/1024, elapsed(&start, &end, loop));
+
+			{
+				uint32_t *base = gem_mmap__wc(fd, handle, 0, size, PROT_READ | PROT_WRITE);
+				gettimeofday(&start, NULL);
+				for (loop = 0; loop < 1000; loop++) {
+					volatile uint32_t *ptr = base;
+					int x = 0;
+
+					for (i = 0; i < size/sizeof(*ptr); i++)
+						x += ptr[i];
+
+					/* force overly clever gcc to actually compute x */
+					ptr[0] = x;
+
+				}
+				gettimeofday(&end, NULL);
+				munmap(base, size);
+			}
+			igt_info("Time to read %dk through a cached WC map:	%7.3fµs\n",
+				 size/1024, elapsed(&start, &end, loop));
+
+			/* Check streaming loads from WC */
+			if (cpu & SSE4_1) {
+				gettimeofday(&start, NULL);
+				for (loop = 0; loop < 1000; loop++) {
+					uint32_t *base = gem_mmap__wc(fd, handle, 0, size, PROT_READ | PROT_WRITE);
+					streaming_load(base, size);
+
+					munmap(base, size);
+				}
+				gettimeofday(&end, NULL);
+				igt_info("Time to stream %dk from a WC map:		%7.3fµs\n",
+					 size/1024, elapsed(&start, &end, loop));
+
+				{
+					uint32_t *base = gem_mmap__wc(fd, handle, 0, size, PROT_READ | PROT_WRITE);
+					gettimeofday(&start, NULL);
+					for (loop = 0; loop < 1000; loop++)
+						streaming_load(base, size);
+					gettimeofday(&end, NULL);
+					munmap(base, size);
+				}
+				igt_info("Time to stream %dk from a cached WC map:	%7.3fµs\n",
+					 size/1024, elapsed(&start, &end, loop));
+			}
+		}
+
 
 		/* mmap write */
 		gettimeofday(&start, NULL);
 		for (loop = 0; loop < 1000; loop++) {
-			uint32_t *base = gem_mmap(fd, handle, size, PROT_READ | PROT_WRITE);
+			uint32_t *base = gem_mmap__gtt(fd, handle, size, PROT_READ | PROT_WRITE);
 			volatile uint32_t *ptr = base;
 
 			for (i = 0; i < size/sizeof(*ptr); i++)
@@ -216,47 +355,88 @@ int main(int argc, char **argv)
 			munmap(base, size);
 		}
 		gettimeofday(&end, NULL);
-		printf("Time to write %dk through a GTT map:		%7.3fµs\n",
-		       size/1024, elapsed(&start, &end, loop));
+		igt_info("Time to write %dk through a GTT map:		%7.3fµs\n",
+			 size/1024, elapsed(&start, &end, loop));
+
+		if (gem_mmap__has_wc(fd)) {
+			/* mmap write */
+			gettimeofday(&start, NULL);
+			for (loop = 0; loop < 1000; loop++) {
+				uint32_t *base = gem_mmap__wc(fd, handle, 0, size, PROT_READ | PROT_WRITE);
+				volatile uint32_t *ptr = base;
+
+				for (i = 0; i < size/sizeof(*ptr); i++)
+					ptr[i] = i;
+
+				munmap(base, size);
+			}
+			gettimeofday(&end, NULL);
+			igt_info("Time to write %dk through a WC map:		%7.3fµs\n",
+					size/1024, elapsed(&start, &end, loop));
+		}
 
 		/* mmap clear */
 		gettimeofday(&start, NULL);
 		for (loop = 0; loop < 1000; loop++) {
-			uint32_t *base = gem_mmap(fd, handle, size, PROT_READ | PROT_WRITE);
+			uint32_t *base = gem_mmap__gtt(fd, handle, size, PROT_READ | PROT_WRITE);
 			memset(base, 0, size);
 			munmap(base, size);
 		}
 		gettimeofday(&end, NULL);
-		printf("Time to clear %dk through a GTT map:		%7.3fµs\n",
-		       size/1024, elapsed(&start, &end, loop));
+		igt_info("Time to clear %dk through a GTT map:		%7.3fµs\n",
+			 size/1024, elapsed(&start, &end, loop));
+
+		if (gem_mmap__has_wc(fd)) {
+			/* mmap clear */
+			gettimeofday(&start, NULL);
+			for (loop = 0; loop < 1000; loop++) {
+				uint32_t *base = gem_mmap__wc(fd, handle, 0, size, PROT_READ | PROT_WRITE);
+				memset(base, 0, size);
+				munmap(base, size);
+			}
+			gettimeofday(&end, NULL);
+			igt_info("Time to clear %dk through a WC map:		%7.3fµs\n",
+					size/1024, elapsed(&start, &end, loop));
+		}
 
 		gettimeofday(&start, NULL);{
-			uint32_t *base = gem_mmap(fd, handle, size, PROT_READ | PROT_WRITE);
+			uint32_t *base = gem_mmap__gtt(fd, handle, size, PROT_READ | PROT_WRITE);
 			for (loop = 0; loop < 1000; loop++)
 				memset(base, 0, size);
 			munmap(base, size);
 		} gettimeofday(&end, NULL);
-		printf("Time to clear %dk through a cached GTT map:	%7.3fµs\n",
-		       size/1024, elapsed(&start, &end, loop));
+		igt_info("Time to clear %dk through a cached GTT map:	%7.3fµs\n",
+			 size/1024, elapsed(&start, &end, loop));
+
+		if (gem_mmap__has_wc(fd)) {
+			gettimeofday(&start, NULL);{
+				uint32_t *base = gem_mmap__wc(fd, handle, 0, size, PROT_READ | PROT_WRITE);
+				for (loop = 0; loop < 1000; loop++)
+					memset(base, 0, size);
+				munmap(base, size);
+			} gettimeofday(&end, NULL);
+			igt_info("Time to clear %dk through a cached WC map:	%7.3fµs\n",
+					size/1024, elapsed(&start, &end, loop));
+		}
 
 		/* mmap read */
 		gettimeofday(&start, NULL);
 		for (loop = 0; loop < 1000; loop++) {
-			uint32_t *base = gem_mmap(fd, handle, size, PROT_READ | PROT_WRITE);
+			uint32_t *base = gem_mmap__gtt(fd, handle, size, PROT_READ | PROT_WRITE);
 			volatile uint32_t *ptr = base;
 			int x = 0;
 
 			for (i = 0; i < size/sizeof(*ptr); i++)
 				x += ptr[i];
 
-			/* force overtly clever gcc to actually compute x */
+			/* force overly clever gcc to actually compute x */
 			ptr[0] = x;
 
 			munmap(base, size);
 		}
 		gettimeofday(&end, NULL);
-		printf("Time to read %dk (again) through a GTT map:	%7.3fµs\n",
-		       size/1024, elapsed(&start, &end, loop));
+		igt_info("Time to read %dk (again) through a GTT map:	%7.3fµs\n",
+			 size/1024, elapsed(&start, &end, loop));
 
 		if (tiling == I915_TILING_NONE) {
 			/* GTT pwrite */
@@ -264,16 +444,16 @@ int main(int argc, char **argv)
 			for (loop = 0; loop < 1000; loop++)
 				gem_write(fd, handle, 0, buf, size);
 			gettimeofday(&end, NULL);
-			printf("Time to pwrite %dk through the GTT:		%7.3fµs\n",
-			       size/1024, elapsed(&start, &end, loop));
+			igt_info("Time to pwrite %dk through the GTT:		%7.3fµs\n",
+				 size/1024, elapsed(&start, &end, loop));
 
 			/* GTT pread */
 			gettimeofday(&start, NULL);
 			for (loop = 0; loop < 1000; loop++)
 				gem_read(fd, handle, 0, buf, size);
 			gettimeofday(&end, NULL);
-			printf("Time to pread %dk through the GTT:		%7.3fµs\n",
-			       size/1024, elapsed(&start, &end, loop));
+			igt_info("Time to pread %dk through the GTT:		%7.3fµs\n",
+				 size/1024, elapsed(&start, &end, loop));
 
 			/* GTT pwrite, including clflush */
 			gettimeofday(&start, NULL);
@@ -282,8 +462,8 @@ int main(int argc, char **argv)
 				gem_sync(fd, handle);
 			}
 			gettimeofday(&end, NULL);
-			printf("Time to pwrite %dk through the GTT (clflush):	%7.3fµs\n",
-			       size/1024, elapsed(&start, &end, loop));
+			igt_info("Time to pwrite %dk through the GTT (clflush):	%7.3fµs\n",
+				 size/1024, elapsed(&start, &end, loop));
 
 			/* GTT pread, including clflush */
 			gettimeofday(&start, NULL);
@@ -292,11 +472,11 @@ int main(int argc, char **argv)
 				gem_read(fd, handle, 0, buf, size);
 			}
 			gettimeofday(&end, NULL);
-			printf("Time to pread %dk through the GTT (clflush):	%7.3fµs\n",
-			       size/1024, elapsed(&start, &end, loop));
+			igt_info("Time to pread %dk through the GTT (clflush):	%7.3fµs\n",
+				 size/1024, elapsed(&start, &end, loop));
 
 			/* partial writes */
-			printf("Now partial writes.\n");
+			igt_info("Now partial writes.\n");
 			size /= 4;
 
 			/* partial GTT pwrite, including clflush */
@@ -306,7 +486,7 @@ int main(int argc, char **argv)
 				gem_sync(fd, handle);
 			}
 			gettimeofday(&end, NULL);
-			printf("Time to pwrite %dk through the GTT (clflush):	%7.3fµs\n",
+			igt_info("Time to pwrite %dk through the GTT (clflush):	%7.3fµs\n",
 			       size/1024, elapsed(&start, &end, loop));
 
 			/* partial GTT pread, including clflush */
@@ -316,16 +496,15 @@ int main(int argc, char **argv)
 				gem_read(fd, handle, 0, buf, size);
 			}
 			gettimeofday(&end, NULL);
-			printf("Time to pread %dk through the GTT (clflush):	%7.3fµs\n",
+			igt_info("Time to pread %dk through the GTT (clflush):	%7.3fµs\n",
 			       size/1024, elapsed(&start, &end, loop));
 
 			size *= 4;
 		}
-
 	}
 
 	gem_close(fd, handle);
 	close(fd);
 
-	return 0;
+	igt_exit();
 }
