@@ -25,6 +25,7 @@
  *
  */
 
+#include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -34,13 +35,11 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include "drm.h"
-#include "i915_drm.h"
-#include "drmtest.h"
 
 /* Should take 64 pages to store the page pointers on 64 bit */
 #define OBJ_SIZE (128 * 1024 * 1024)
 
-unsigned char data[OBJ_SIZE];
+unsigned char *data;
 
 static void
 test_large_object(int fd)
@@ -48,7 +47,7 @@ test_large_object(int fd)
 	struct drm_i915_gem_create create;
 	struct drm_i915_gem_pin pin;
 	uint32_t obj_size;
-	int ret;
+	char *ptr;
 
 	memset(&create, 0, sizeof(create));
 	memset(&pin, 0, sizeof(pin));
@@ -60,37 +59,32 @@ test_large_object(int fd)
 	else
 		obj_size = OBJ_SIZE;
 	create.size = obj_size;
-	printf("obj size %i\n", obj_size);
+	igt_info("obj size %i\n", obj_size);
 
-	ret = ioctl(fd, DRM_IOCTL_I915_GEM_CREATE, &create);
-	if (ret) {
-		fprintf(stderr, "object creation failed: %s\n",
-			strerror(errno));
-		igt_fail(ret);
-	}
+	igt_assert(ioctl(fd, DRM_IOCTL_I915_GEM_CREATE, &create) == 0);
 
-	pin.handle = create.handle;
-	ret = ioctl(fd, DRM_IOCTL_I915_GEM_PIN, &pin);
-	if (ret) {
-		fprintf(stderr, "pin failed: %s\n",
-			strerror(errno));
-		igt_fail(ret);
-	}
+	/* prefault */
+	ptr = gem_mmap__gtt(fd, create.handle, obj_size,
+			    PROT_WRITE | PROT_READ);
+	*ptr = 0;
 
 	gem_write(fd, create.handle, 0, data, obj_size);
 
 	/* kernel should clean this up for us */
 }
 
-int main(int argc, char **argv)
+igt_simple_main
 {
 	int fd;
 
 	igt_skip_on_simulation();
 
-	fd = drm_open_any();
+	data = malloc(OBJ_SIZE);
+	igt_assert(data);
+
+	fd = drm_open_driver(DRIVER_INTEL);
 
 	test_large_object(fd);
 
-	return 0;
+	free(data);
 }
