@@ -27,6 +27,7 @@
 
 /* Exercises the basic execbuffer using the handle LUT interface */
 
+#include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -36,8 +37,6 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include "drm.h"
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
 
 IGT_TEST_DESCRIPTION("Exercises the basic execbuffer using the handle LUT"
 		     " interface.");
@@ -66,14 +65,14 @@ static int exec(int fd, uint32_t handle, unsigned int flags)
 
 	gem_exec[0].handle = handle;
 	gem_exec[0].relocation_count = 1;
-	gem_exec[0].relocs_ptr = (uintptr_t) gem_reloc;
+	gem_exec[0].relocs_ptr = to_user_pointer(gem_reloc);
 	gem_exec[0].alignment = 0;
 	gem_exec[0].offset = 0;
 	gem_exec[0].flags = 0;
 	gem_exec[0].rsvd1 = 0;
 	gem_exec[0].rsvd2 = 0;
 
-	execbuf.buffers_ptr = (uintptr_t)gem_exec;
+	execbuf.buffers_ptr = to_user_pointer(gem_exec);
 	execbuf.buffer_count = 1;
 	execbuf.batch_start_offset = 0;
 	execbuf.batch_len = 8;
@@ -85,12 +84,7 @@ static int exec(int fd, uint32_t handle, unsigned int flags)
 	i915_execbuffer2_set_context_id(execbuf, 0);
 	execbuf.rsvd2 = 0;
 
-	if (drmIoctl(fd,
-		     DRM_IOCTL_I915_GEM_EXECBUFFER2,
-		     &execbuf))
-		return -errno;
-
-	return 0;
+	return __gem_execbuf(fd, &execbuf);
 }
 
 static int many_exec(int fd, uint32_t batch, int num_exec, int num_reloc, unsigned flags)
@@ -120,7 +114,7 @@ static int many_exec(int fd, uint32_t batch, int num_exec, int num_reloc, unsign
 
 	gem_exec[n].handle = batch;
 	gem_exec[n].relocation_count = num_reloc;
-	gem_exec[n].relocs_ptr = (uintptr_t) gem_reloc;
+	gem_exec[n].relocs_ptr = to_user_pointer(gem_reloc);
 
 	if (flags & USE_LUT)
 		max_handle = num_exec + 1;
@@ -145,7 +139,7 @@ static int many_exec(int fd, uint32_t batch, int num_exec, int num_reloc, unsign
 		gem_reloc[n].presumed_offset = 0;
 	}
 
-	execbuf.buffers_ptr = (uintptr_t)gem_exec;
+	execbuf.buffers_ptr = to_user_pointer(gem_exec);
 	execbuf.buffer_count = num_exec + 1;
 	execbuf.batch_start_offset = 0;
 	execbuf.batch_len = 8;
@@ -157,11 +151,7 @@ static int many_exec(int fd, uint32_t batch, int num_exec, int num_reloc, unsign
 	i915_execbuffer2_set_context_id(execbuf, 0);
 	execbuf.rsvd2 = 0;
 
-	ret = drmIoctl(fd,
-		       DRM_IOCTL_I915_GEM_EXECBUFFER2,
-		       &execbuf);
-	if (ret < 0)
-		ret = -errno;
+	ret = __gem_execbuf(fd, &execbuf);
 
 	for (n = 0; n < num_exec; n++)
 		gem_close(fd, gem_exec[n].handle);
@@ -181,7 +171,7 @@ igt_simple_main
 	uint32_t handle;
 	int fd, i;
 
-	fd = drm_open_any();
+	fd = drm_open_driver(DRIVER_INTEL);
 
 	handle = gem_create(fd, 4096);
 	gem_write(fd, handle, 0, batch, sizeof(batch));
