@@ -25,7 +25,7 @@
  *
  */
 
-#define _GNU_SOURCE
+#include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -39,18 +39,14 @@
 
 #include <drm.h>
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "intel_chipset.h"
-#include "intel_io.h"
-#include "igt_debugfs.h"
-#include "igt_aux.h"
 
 IGT_TEST_DESCRIPTION("Test persistent relocations as used by uxa/libva.");
 
 /*
  * Testcase: Persistent relocations as used by uxa/libva
  *
+ * Note: this currently fails on byt/full-ppgtt
+ * https://bugs.freedesktop.org/show_bug.cgi?id=84859
  */
 
 static drm_intel_bufmgr *bufmgr;
@@ -105,7 +101,7 @@ static drm_intel_bo *create_special_bo(void)
 		BATCH(CMD_POLY_STIPPLE_OFFSET << 16);
 		BATCH(0);
 	}
-	igt_assert(len % 2 == 0);
+	igt_assert_eq(len % 2, 0);
 	BATCH(MI_NOOP);
 	BATCH(MI_BATCH_BUFFER_END);
 
@@ -174,14 +170,14 @@ static void faulting_reloc_and_emit(int fd, drm_intel_bo *target_bo,
 	exec[1].handle = special_bo->handle;
 	exec[1].relocation_count = 1;
 	/* A newly mmap gtt bo will fault on first access. */
-	exec[1].relocs_ptr = (uintptr_t)gtt_relocs;
+	exec[1].relocs_ptr = to_user_pointer(gtt_relocs);
 	exec[1].alignment = 0;
 	exec[1].offset = 0;
 	exec[1].flags = 0;
 	exec[1].rsvd1 = 0;
 	exec[1].rsvd2 = 0;
 
-	execbuf.buffers_ptr = (uintptr_t)exec;
+	execbuf.buffers_ptr = to_user_pointer(exec);
 	execbuf.buffer_count = 2;
 	execbuf.batch_start_offset = 0;
 	execbuf.batch_len = special_batch_len;
@@ -228,10 +224,8 @@ static void do_test(int fd, bool faulting_reloc)
 
 		relocs_bo_handle[i] = gem_create(fd, 4096);
 		gem_write(fd, relocs_bo_handle[i], 0, reloc, sizeof(reloc));
-		gtt_relocs_ptr[i] = gem_mmap(fd, relocs_bo_handle[i], 4096,
-				      PROT_READ | PROT_WRITE);
-		igt_assert(gtt_relocs_ptr[i]);
-
+		gtt_relocs_ptr[i] = gem_mmap__gtt(fd, relocs_bo_handle[i], 4096,
+						  PROT_READ | PROT_WRITE);
 	}
 
 	/* repeat must be smaller than 4096/small_pitch */
@@ -293,7 +287,7 @@ static void do_forked_test(int fd, unsigned flags)
 		igt_fork_helper(&thrasher) {
 			while (1) {
 				usleep(1000);
-				igt_drop_caches_set(val);
+				igt_drop_caches_set(fd, val);
 			}
 		}
 	}
@@ -327,7 +321,8 @@ igt_main
 	memset(blob, 'A', sizeof(blob));
 
 	igt_fixture {
-		fd = drm_open_any();
+		fd = drm_open_driver(DRIVER_INTEL);
+		igt_require_gem(fd);
 
 		bufmgr = drm_intel_bufmgr_gem_init(fd, 4096);
 		/* disable reuse, otherwise the test fails */
