@@ -34,6 +34,7 @@
  * ranged-buffer-flush paths in the kernel.
  */
 
+#include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -45,11 +46,6 @@
 
 #include <drm.h>
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "intel_chipset.h"
-#include "intel_io.h"
-#include "igt_aux.h"
 
 IGT_TEST_DESCRIPTION("Test pread behavior when getting values out of"
 		     " just-drawn-to buffers.");
@@ -122,12 +118,26 @@ verify_small_read(drm_intel_bo *bo, uint32_t val)
 	}
 }
 
+typedef igt_hang_t (*do_hang)(int fd);
+
+static igt_hang_t no_hang(int fd)
+{
+	return (igt_hang_t){0};
+}
+
+static igt_hang_t bcs_hang(int fd)
+{
+	return igt_hang_ring(fd, batch->gen >= 6 ? I915_EXEC_BLT : I915_EXEC_DEFAULT);
+}
+
 static void do_test(int fd, int cache_level,
 		    drm_intel_bo *src[2],
 		    const uint32_t start[2],
 		    drm_intel_bo *tmp[2],
-		    int loop)
+		    int loop, do_hang do_hang_func)
 {
+	igt_hang_t hang;
+
 	if (cache_level != -1) {
 		gem_set_caching(fd, tmp[0]->handle, cache_level);
 		gem_set_caching(fd, tmp[1]->handle, cache_level);
@@ -136,32 +146,48 @@ static void do_test(int fd, int cache_level,
 	do {
 		/* First, do a full-buffer read after blitting */
 		intel_copy_bo(batch, tmp[0], src[0], width*height*4);
+		hang = do_hang_func(fd);
 		verify_large_read(tmp[0], start[0]);
+		igt_post_hang_ring(fd, hang);
 		intel_copy_bo(batch, tmp[0], src[1], width*height*4);
+		hang = do_hang_func(fd);
 		verify_large_read(tmp[0], start[1]);
+		igt_post_hang_ring(fd, hang);
 
 		intel_copy_bo(batch, tmp[0], src[0], width*height*4);
+		hang = do_hang_func(fd);
 		verify_small_read(tmp[0], start[0]);
+		igt_post_hang_ring(fd, hang);
 		intel_copy_bo(batch, tmp[0], src[1], width*height*4);
+		hang = do_hang_func(fd);
 		verify_small_read(tmp[0], start[1]);
+		igt_post_hang_ring(fd, hang);
 
 		intel_copy_bo(batch, tmp[0], src[0], width*height*4);
+		hang = do_hang_func(fd);
 		verify_large_read(tmp[0], start[0]);
+		igt_post_hang_ring(fd, hang);
 
 		intel_copy_bo(batch, tmp[0], src[0], width*height*4);
 		intel_copy_bo(batch, tmp[1], src[1], width*height*4);
+		hang = do_hang_func(fd);
 		verify_large_read(tmp[0], start[0]);
 		verify_large_read(tmp[1], start[1]);
+		igt_post_hang_ring(fd, hang);
 
 		intel_copy_bo(batch, tmp[0], src[0], width*height*4);
 		intel_copy_bo(batch, tmp[1], src[1], width*height*4);
+		hang = do_hang_func(fd);
 		verify_large_read(tmp[1], start[1]);
 		verify_large_read(tmp[0], start[0]);
+		igt_post_hang_ring(fd, hang);
 
 		intel_copy_bo(batch, tmp[1], src[0], width*height*4);
 		intel_copy_bo(batch, tmp[0], src[1], width*height*4);
+		hang = do_hang_func(fd);
 		verify_large_read(tmp[0], start[1]);
 		verify_large_read(tmp[1], start[0]);
+		igt_post_hang_ring(fd, hang);
 	} while (--loop);
 }
 
@@ -171,11 +197,22 @@ int fd;
 igt_main
 {
 	const uint32_t start[2] = {0, 1024 * 1024 / 4};
+	const struct {
+		const char *name;
+		int cache;
+	} tests[] = {
+		{ "default", -1 },
+		{ "uncached", 0 },
+		{ "snooped", 1 },
+		{ "display", 2 },
+		{ NULL, -1 },
+	}, *t;
 
 	igt_skip_on_simulation();
 
 	igt_fixture {
-		fd = drm_open_any();
+		fd = drm_open_driver(DRIVER_INTEL);
+		igt_require_gem(fd);
 
 		bufmgr = drm_intel_bufmgr_gem_init(fd, 4096);
 		drm_intel_bufmgr_gem_enable_reuse(bufmgr);
@@ -188,40 +225,17 @@ igt_main
 		dst[1] = drm_intel_bo_alloc(bufmgr, "dst bo", size, 4096);
 	}
 
-	igt_subtest("normal")
-		do_test(fd, -1, src, start, dst, 1);
+	for (t = tests; t->name; t++) {
+		igt_subtest_f("%s-normal", t->name)
+			do_test(fd, t->cache, src, start, dst, 1, no_hang);
 
-	igt_subtest("interruptible") {
 		igt_fork_signal_helper();
-		do_test(fd, -1, src, start, dst, 100);
+		igt_subtest_f("%s-interruptible", t->name)
+			do_test(fd, t->cache, src, start, dst, 100, no_hang);
 		igt_stop_signal_helper();
-	}
 
-	igt_subtest("normal-uncached")
-		do_test(fd, 0, src, start, dst, 1);
-
-	igt_subtest("interruptible-uncached") {
-		igt_fork_signal_helper();
-		do_test(fd, 0, src, start, dst, 100);
-		igt_stop_signal_helper();
-	}
-
-	igt_subtest("normal-snoop")
-		do_test(fd, 1, src, start, dst, 1);
-
-	igt_subtest("interruptible-snoop") {
-		igt_fork_signal_helper();
-		do_test(fd, 1, src, start, dst, 100);
-		igt_stop_signal_helper();
-	}
-
-	igt_subtest("normal-display")
-		do_test(fd, 2, src, start, dst, 1);
-
-	igt_subtest("interruptible-display") {
-		igt_fork_signal_helper();
-		do_test(fd, 2, src, start, dst, 100);
-		igt_stop_signal_helper();
+		igt_subtest_f("%s-hang", t->name)
+			do_test(fd, t->cache, src, start, dst, 1, bcs_hang);
 	}
 
 	igt_fixture {
