@@ -49,10 +49,29 @@
  * In short: designed for maximum evilness.
  */
 
-#include "rendercopy.h"
+#include "igt.h"
+#include <stdlib.h>
+#include <sys/ioctl.h>
+#include <stdio.h>
+#include <string.h>
+#include <fcntl.h>
+#include <inttypes.h>
+#include <errno.h>
+#include <sys/stat.h>
+#include <sys/time.h>
+
+#include <drm.h>
+
+#include "intel_bufmgr.h"
+
+IGT_TEST_DESCRIPTION("General gem coherency test.");
 
 #define CMD_POLY_STIPPLE_OFFSET       0x7906
 
+#define DUCTAPE 0xdead0001
+#define TILESZ	0xdead0002
+#define CHCK_RENDER 0xdead0003
+
 /** TODO:
  * - beat on relaxed fencing (i.e. mappable/fenceable tracking in the kernel)
  * - render copy (to check fence tracking and cache coherency management by the
@@ -100,7 +119,7 @@ struct option_struct options;
 #define BUSY_BUF_SIZE		(256*4096)
 #define TILE_BYTES(size)	((size)*(size)*sizeof(uint32_t))
 
-static struct scratch_buf buffers[2][MAX_BUFS];
+static struct igt_buf buffers[2][MAX_BUFS];
 /* tile i is at logical position tile_permutation[i] */
 static unsigned *tile_permutation;
 static unsigned num_buffers = 0;
@@ -116,7 +135,7 @@ struct {
 	unsigned max_failed_reads;
 } stats;
 
-static void tile2xy(struct scratch_buf *buf, unsigned tile, unsigned *x, unsigned *y)
+static void tile2xy(struct igt_buf *buf, unsigned tile, unsigned *x, unsigned *y)
 {
 	igt_assert(tile < buf->num_tiles);
 	*x = (tile*options.tile_size) % (buf->stride/sizeof(uint32_t));
@@ -141,11 +160,7 @@ static void emit_blt(drm_intel_bo *src_bo, uint32_t src_tiling, unsigned src_pit
 	}
 
 	/* copy lower half to upper half */
-	BEGIN_BATCH(8);
-	OUT_BATCH(XY_SRC_COPY_BLT_CMD |
-		  XY_SRC_COPY_BLT_WRITE_ALPHA |
-		  XY_SRC_COPY_BLT_WRITE_RGB |
-		  cmd_bits);
+	BLIT_COPY_BATCH_START(cmd_bits);
 	OUT_BATCH((3 << 24) | /* 32 bits */
 		  (0xcc << 16) | /* copy ROP */
 		  dst_pitch);
@@ -157,8 +172,8 @@ static void emit_blt(drm_intel_bo *src_bo, uint32_t src_tiling, unsigned src_pit
 	OUT_RELOC_FENCED(src_bo, I915_GEM_DOMAIN_RENDER, 0, 0);
 	ADVANCE_BATCH();
 
-	if (IS_GEN6(devid) || IS_GEN7(devid)) {
-		BEGIN_BATCH(3);
+	if (batch->gen >= 6) {
+		BEGIN_BATCH(3, 0);
 		OUT_BATCH(XY_SETUP_CLIP_BLT_CMD);
 		OUT_BATCH(0);
 		OUT_BATCH(0);
@@ -173,21 +188,21 @@ static void keep_gpu_busy(void)
 	int tmp;
 
 	tmp = 1 << gpu_busy_load;
-	igt_assert(tmp <= 1024);
+	igt_assert_lte(tmp, 1024);
 
 	emit_blt(busy_bo, 0, 4096, 0, 0, tmp, 128,
 		 busy_bo, 0, 4096, 0, 128);
 }
 
-static void set_to_cpu_domain(struct scratch_buf *buf, int writing)
+static void set_to_cpu_domain(struct igt_buf *buf, int writing)
 {
 	gem_set_domain(drm_fd, buf->bo->handle, I915_GEM_DOMAIN_CPU,
 		       writing ? I915_GEM_DOMAIN_CPU : 0);
 }
 
 static unsigned int copyfunc_seq = 0;
-static void (*copyfunc)(struct scratch_buf *src, unsigned src_x, unsigned src_y,
-			struct scratch_buf *dst, unsigned dst_x, unsigned dst_y,
+static void (*copyfunc)(struct igt_buf *src, unsigned src_x, unsigned src_y,
+			struct igt_buf *dst, unsigned dst_x, unsigned dst_y,
 			unsigned logical_tile_no);
 
 /* stride, x, y in units of uint32_t! */
@@ -206,18 +221,15 @@ static void cpucpy2d(uint32_t *src, unsigned src_stride, unsigned src_x, unsigne
 			    + i*options.tile_size + j;
 			uint32_t tmp = src[src_ofs];
 			if (tmp != expect) {
-			    printf("mismatch at tile %i pos %i, read %i, expected %i, diff %i\n",
-				    logical_tile_no, i*options.tile_size + j, tmp, expect, (int) tmp - expect);
-			    if (options.trace_tile >= 0 && options.fail)
-				    igt_fail(1);
+			    igt_info("mismatch at tile %i pos %i, read %i, expected %i, diff %i\n", logical_tile_no, i * options.tile_size + j, tmp, expect, (int)tmp - expect);
+			    igt_fail_on(options.trace_tile >= 0 && options.fail);
 			    failed++;
 			}
 			/* when not aborting, correct any errors */
 			dst[dst_ofs] = expect;
 		}
 	}
-	if (failed && options.fail)
-		igt_fail(1);
+	igt_fail_on(failed && options.fail);
 
 	if (failed > stats.max_failed_reads)
 		stats.max_failed_reads = failed;
@@ -225,8 +237,8 @@ static void cpucpy2d(uint32_t *src, unsigned src_stride, unsigned src_x, unsigne
 		stats.num_failed++;
 }
 
-static void cpu_copyfunc(struct scratch_buf *src, unsigned src_x, unsigned src_y,
-			 struct scratch_buf *dst, unsigned dst_x, unsigned dst_y,
+static void cpu_copyfunc(struct igt_buf *src, unsigned src_x, unsigned src_y,
+			 struct igt_buf *dst, unsigned dst_x, unsigned dst_y,
 			 unsigned logical_tile_no)
 {
 	igt_assert(batch->ptr == batch->buffer);
@@ -244,8 +256,8 @@ static void cpu_copyfunc(struct scratch_buf *src, unsigned src_x, unsigned src_y
 		 logical_tile_no);
 }
 
-static void prw_copyfunc(struct scratch_buf *src, unsigned src_x, unsigned src_y,
-			 struct scratch_buf *dst, unsigned dst_x, unsigned dst_y,
+static void prw_copyfunc(struct igt_buf *src, unsigned src_x, unsigned src_y,
+			 struct igt_buf *dst, unsigned dst_x, unsigned dst_y,
 			 unsigned logical_tile_no)
 {
 	uint32_t tmp_tile[options.tile_size*options.tile_size];
@@ -288,8 +300,8 @@ static void prw_copyfunc(struct scratch_buf *src, unsigned src_x, unsigned src_y
 	}
 }
 
-static void blitter_copyfunc(struct scratch_buf *src, unsigned src_x, unsigned src_y,
-			     struct scratch_buf *dst, unsigned dst_x, unsigned dst_y,
+static void blitter_copyfunc(struct igt_buf *src, unsigned src_x, unsigned src_y,
+			     struct igt_buf *dst, unsigned dst_x, unsigned dst_y,
 			     unsigned logical_tile_no)
 {
 	static unsigned keep_gpu_busy_counter = 0;
@@ -318,22 +330,27 @@ static void blitter_copyfunc(struct scratch_buf *src, unsigned src_x, unsigned s
 	}
 }
 
-static void render_copyfunc(struct scratch_buf *src, unsigned src_x, unsigned src_y,
-			    struct scratch_buf *dst, unsigned dst_x, unsigned dst_y,
+static void render_copyfunc(struct igt_buf *src, unsigned src_x, unsigned src_y,
+			    struct igt_buf *dst, unsigned dst_x, unsigned dst_y,
 			    unsigned logical_tile_no)
 {
 	static unsigned keep_gpu_busy_counter = 0;
-	render_copyfunc_t rendercopy = get_render_copyfunc(devid);
+	igt_render_copyfunc_t rendercopy = igt_get_render_copyfunc(devid);
 
 	/* check both edges of the fence usage */
 	if (keep_gpu_busy_counter & 1)
 		keep_gpu_busy();
 
-	if (rendercopy)
-		rendercopy(batch, src, src_x, src_y,
+	if (rendercopy) {
+		/*
+		 * Flush outstanding blts so that they don't end up on
+		 * the render ring when that's not allowed (gen6+).
+		 */
+		intel_batchbuffer_flush(batch);
+		rendercopy(batch, NULL, src, src_x, src_y,
 		     options.tile_size, options.tile_size,
 		     dst, dst_x, dst_y);
-	else
+	} else
 		blitter_copyfunc(src, src_x, src_y,
 				 dst, dst_x, dst_y,
 				 logical_tile_no);
@@ -348,35 +365,35 @@ static void next_copyfunc(int tile)
 {
 	if (fence_storm) {
 		if (tile == options.trace_tile)
-			printf(" using fence storm\n");
+			igt_info(" using fence storm\n");
 		return;
 	}
 
 	if (copyfunc_seq % 61 == 0
 			&& options.forced_tiling != I915_TILING_NONE) {
 		if (tile == options.trace_tile)
-			printf(" using fence storm\n");
+			igt_info(" using fence storm\n");
 		fence_storm = num_fences;
 		copyfunc = blitter_copyfunc;
 	} else if (copyfunc_seq % 17 == 0) {
 		if (tile == options.trace_tile)
-			printf(" using cpu\n");
+			igt_info(" using cpu\n");
 		copyfunc = cpu_copyfunc;
 	} else if (copyfunc_seq % 19 == 0) {
 		if (tile == options.trace_tile)
-			printf(" using prw\n");
+			igt_info(" using prw\n");
 		copyfunc = prw_copyfunc;
 	} else if (copyfunc_seq % 3 == 0 && options.use_render) {
 		if (tile == options.trace_tile)
-			printf(" using render\n");
+			igt_info(" using render\n");
 		copyfunc = render_copyfunc;
 	} else if (options.use_blt){
 		if (tile == options.trace_tile)
-			printf(" using blitter\n");
+			igt_info(" using blitter\n");
 		copyfunc = blitter_copyfunc;
 	} else if (options.use_render){
 		if (tile == options.trace_tile)
-			printf(" using render\n");
+			igt_info(" using render\n");
 		copyfunc = render_copyfunc;
 	} else {
 		copyfunc = cpu_copyfunc;
@@ -438,28 +455,28 @@ static void fan_in_and_check(void)
 	}
 }
 
-static void sanitize_stride(struct scratch_buf *buf)
+static void sanitize_stride(struct igt_buf *buf)
 {
 
-	if (buf_height(buf) > options.max_dimension)
+	if (igt_buf_height(buf) > options.max_dimension)
 		buf->stride = buf->size / options.max_dimension;
 
-	if (buf_height(buf) < options.tile_size)
+	if (igt_buf_height(buf) < options.tile_size)
 		buf->stride = buf->size / options.tile_size;
 
-	if (buf_width(buf) < options.tile_size)
+	if (igt_buf_width(buf) < options.tile_size)
 		buf->stride = options.tile_size * sizeof(uint32_t);
 
 	igt_assert(buf->stride <= 8192);
-	igt_assert(buf_width(buf) <= options.max_dimension);
-	igt_assert(buf_height(buf) <= options.max_dimension);
+	igt_assert(igt_buf_width(buf) <= options.max_dimension);
+	igt_assert(igt_buf_height(buf) <= options.max_dimension);
 
-	igt_assert(buf_width(buf) >= options.tile_size);
-	igt_assert(buf_height(buf) >= options.tile_size);
+	igt_assert(igt_buf_width(buf) >= options.tile_size);
+	igt_assert(igt_buf_height(buf) >= options.tile_size);
 
 }
 
-static void init_buffer(struct scratch_buf *buf, unsigned size)
+static void init_buffer(struct igt_buf *buf, unsigned size)
 {
 	buf->bo = drm_intel_bo_alloc(bufmgr, "tiled bo", size, 4096);
 	buf->size = size;
@@ -484,12 +501,12 @@ static void init_buffer(struct scratch_buf *buf, unsigned size)
 
 static void exchange_buf(void *array, unsigned i, unsigned j)
 {
-	struct scratch_buf *buf_arr, tmp;
+	struct igt_buf *buf_arr, tmp;
 	buf_arr = array;
 
-	memcpy(&tmp, &buf_arr[i], sizeof(struct scratch_buf));
-	memcpy(&buf_arr[i], &buf_arr[j], sizeof(struct scratch_buf));
-	memcpy(&buf_arr[j], &tmp, sizeof(struct scratch_buf));
+	memcpy(&tmp, &buf_arr[i], sizeof(struct igt_buf));
+	memcpy(&buf_arr[i], &buf_arr[j], sizeof(struct igt_buf));
+	memcpy(&buf_arr[j], &tmp, sizeof(struct igt_buf));
 }
 
 
@@ -541,27 +558,22 @@ static void init_set(unsigned set)
 			       buffers[set][i].stride);
 
 		if (options.trace_tile != -1 && i == options.trace_tile/options.tiles_per_buf)
-			printf("changing buffer %i containing tile %i: tiling %i, stride %i\n", i,
-					options.trace_tile,
-					buffers[set][i].tiling, buffers[set][i].stride);
+			igt_info("changing buffer %i containing tile %i: tiling %i, stride %i\n", i, options.trace_tile, buffers[set][i].tiling, buffers[set][i].stride);
 	}
 }
 
 static void exchange_uint(void *array, unsigned i, unsigned j)
 {
 	unsigned *i_arr = array;
-	unsigned i_tmp;
 
-	i_tmp = i_arr[i];
-	i_arr[i] = i_arr[j];
-	i_arr[j] = i_tmp;
+	igt_swap(i_arr[i], i_arr[j]);
 }
 
 static void copy_tiles(unsigned *permutation)
 {
 	unsigned src_tile, src_buf_idx, src_x, src_y;
 	unsigned dst_tile, dst_buf_idx, dst_x, dst_y;
-	struct scratch_buf *src_buf, *dst_buf;
+	struct igt_buf *src_buf, *dst_buf;
 	int i, idx;
 	for (i = 0; i < num_total_tiles; i++) {
 		/* tile_permutation is independent of current_permutation, so
@@ -580,9 +592,7 @@ static void copy_tiles(unsigned *permutation)
 		tile2xy(dst_buf, dst_tile, &dst_x, &dst_y);
 
 		if (options.trace_tile == i)
-			printf("copying tile %i from %i (%i, %i) to %i (%i, %i)", i,
-				tile_permutation[i], src_buf_idx, src_tile,
-				permutation[idx], dst_buf_idx, dst_tile);
+			igt_info("copying tile %i from %i (%i, %i) to %i (%i, %i)", i, tile_permutation[i], src_buf_idx, src_tile, permutation[idx], dst_buf_idx, dst_tile);
 
 		if (options.no_hw) {
 			cpucpy2d(src_buf->data,
@@ -603,28 +613,204 @@ static void copy_tiles(unsigned *permutation)
 	intel_batchbuffer_flush(batch);
 }
 
-static int get_num_fences(void)
-{
-	int val;
-
-	val = gem_available_fences(drm_fd);
-
-	printf ("total %d fences\n", val);
-	igt_assert(val > 4);
-
-	return val - 2;
-}
-
 static void sanitize_tiles_per_buf(void)
 {
 	if (options.tiles_per_buf > options.scratch_buf_size / TILE_BYTES(options.tile_size))
 		options.tiles_per_buf = options.scratch_buf_size / TILE_BYTES(options.tile_size);
 }
 
-static void parse_options(int argc, char **argv)
+static int parse_options(int opt, int opt_index, void *data)
 {
-	int c, tmp;
-	int option_index = 0;
+	int tmp;
+
+	switch(opt) {
+		case 'd':
+			options.no_hw = 1;
+			igt_info("no-hw debug mode\n");
+			break;
+		case 'S':
+			options.use_signal_helper = 0;
+			igt_info("disabling that pesky nuisance who keeps interrupting us\n");
+			break;
+		case 's':
+			tmp = atoi(optarg);
+			if (tmp < options.tile_size*8192)
+				igt_info("scratch buffer size needs to be at least %i\n", options.tile_size * 8192);
+			else if (tmp & (tmp - 1)) {
+				igt_info("scratch buffer size needs to be a power-of-two\n");
+			} else {
+				igt_info("fixed scratch buffer size to %u\n", tmp);
+				options.scratch_buf_size = tmp;
+				sanitize_tiles_per_buf();
+			}
+			break;
+		case 'g':
+			tmp = atoi(optarg);
+			if (tmp < 0 || tmp > 10)
+				igt_info("gpu busy load needs to be bigger than 0 and smaller than 10\n");
+			else {
+				igt_info("gpu busy load factor set to %i\n", tmp);
+				gpu_busy_load = options.gpu_busy_load = tmp;
+			}
+			break;
+		case 'c':
+			options.num_buffers = atoi(optarg);
+			igt_info("buffer count set to %i\n", options.num_buffers);
+			break;
+		case 't':
+			options.trace_tile = atoi(optarg);
+			igt_info("tracing tile %i\n", options.trace_tile);
+			break;
+		case 'r':
+			options.use_render = 0;
+			igt_info("disabling render copy\n");
+			break;
+		case 'b':
+			options.use_blt = 0;
+			igt_info("disabling blt copy\n");
+			break;
+		case 'u':
+			options.forced_tiling = I915_TILING_NONE;
+			igt_info("disabling tiling\n");
+			break;
+		case 'x':
+			if (options.use_cpu_maps) {
+				igt_info("tiling not possible with cpu maps\n");
+			} else {
+				options.forced_tiling = I915_TILING_X;
+				igt_info("using only X-tiling\n");
+			}
+			break;
+		case 'm':
+			options.use_cpu_maps = 1;
+			options.forced_tiling = I915_TILING_NONE;
+			igt_info("disabling tiling\n");
+			break;
+		case 'o':
+			options.total_rounds = atoi(optarg);
+			igt_info("total rounds %i\n", options.total_rounds);
+			break;
+		case 'f':
+			options.fail = 0;
+			igt_info("not failing when detecting errors\n");
+			break;
+		case 'p':
+			options.tiles_per_buf = atoi(optarg);
+			igt_info("tiles per buffer %i\n", options.tiles_per_buf);
+			break;
+		case DUCTAPE:
+			options.ducttape = 0;
+			igt_info("applying duct-tape\n");
+			break;
+		case TILESZ:
+			options.tile_size = atoi(optarg);
+			sanitize_tiles_per_buf();
+			igt_info("til size %i\n", options.tile_size);
+			break;
+		case CHCK_RENDER:
+			options.check_render_cpyfn = 1;
+			igt_info("checking render copy function\n");
+			break;
+	}
+
+	/* actually 32767, according to docs, but that kills our nice pot calculations. */
+	options.max_dimension = 16*1024;
+	if (options.use_render) {
+		if (IS_GEN2(devid) || IS_GEN3(devid))
+			options.max_dimension = 2048;
+		else
+			options.max_dimension = 8192;
+	}
+	igt_info("Limiting buffer to %dx%d\n", options.max_dimension, options.max_dimension);
+
+	return 0;
+}
+
+static void init(void)
+{
+	int i;
+	unsigned tmp;
+
+	if (options.num_buffers == 0) {
+		tmp = gem_aperture_size(drm_fd);
+		tmp = min(256 * (1024 * 1024), tmp);
+		num_buffers = 2 * tmp / options.scratch_buf_size / 3;
+		num_buffers /= 2;
+		igt_info("using %u buffers\n", num_buffers);
+	} else
+		num_buffers = options.num_buffers;
+
+	bufmgr = drm_intel_bufmgr_gem_init(drm_fd, 4096);
+	drm_intel_bufmgr_gem_enable_reuse(bufmgr);
+	drm_intel_bufmgr_gem_enable_fenced_relocs(bufmgr);
+	num_fences = gem_available_fences(drm_fd);
+	igt_assert_lt(4, num_fences);
+	batch = intel_batchbuffer_alloc(bufmgr, devid);
+
+	busy_bo = drm_intel_bo_alloc(bufmgr, "tiled bo", BUSY_BUF_SIZE, 4096);
+	if (options.forced_tiling >= 0)
+		gem_set_tiling(drm_fd, busy_bo->handle, options.forced_tiling, 4096);
+
+	for (i = 0; i < num_buffers; i++) {
+		init_buffer(&buffers[0][i], options.scratch_buf_size);
+		init_buffer(&buffers[1][i], options.scratch_buf_size);
+
+		num_total_tiles += buffers[0][i].num_tiles;
+	}
+	current_set = 0;
+
+	/* just in case it helps reproducability */
+	srandom(0xdeadbeef);
+}
+
+static void check_render_copyfunc(void)
+{
+	struct igt_buf src, dst;
+	uint32_t *ptr;
+	int i, j, pass;
+
+	if (!options.check_render_cpyfn)
+		return;
+
+	init_buffer(&src, options.scratch_buf_size);
+	init_buffer(&dst, options.scratch_buf_size);
+
+	for (pass = 0; pass < 16; pass++) {
+		int sx = random() % (igt_buf_width(&src)-options.tile_size);
+		int sy = random() % (igt_buf_height(&src)-options.tile_size);
+		int dx = random() % (igt_buf_width(&dst)-options.tile_size);
+		int dy = random() % (igt_buf_height(&dst)-options.tile_size);
+
+		if (options.use_cpu_maps)
+			set_to_cpu_domain(&src, 1);
+
+		memset(src.data, 0xff, options.scratch_buf_size);
+		for (j = 0; j < options.tile_size; j++) {
+			ptr = (uint32_t*)((char *)src.data + sx*4 + (sy+j) * src.stride);
+			for (i = 0; i < options.tile_size; i++)
+				ptr[i] = j * options.tile_size + i;
+		}
+
+		render_copyfunc(&src, sx, sy, &dst, dx, dy, 0);
+
+		if (options.use_cpu_maps)
+			set_to_cpu_domain(&dst, 0);
+
+		for (j = 0; j < options.tile_size; j++) {
+			ptr = (uint32_t*)((char *)dst.data + dx*4 + (dy+j) * dst.stride);
+			for (i = 0; i < options.tile_size; i++)
+				if (ptr[i] != j * options.tile_size + i) {
+					igt_info("render copyfunc mismatch at (%d, %d): found %d, expected %d\n", i, j, ptr[i], j * options.tile_size + i);
+				}
+		}
+	}
+}
+
+
+int main(int argc, char **argv)
+{
+	int i, j;
+	unsigned *current_permutation, *tmp_permutation;
 	static struct option long_options[] = {
 		{"no-hw", 0, 0, 'd'},
 		{"buf-size", 1, 0, 's'},
@@ -640,11 +826,8 @@ static void parse_options(int argc, char **argv)
 		{"rounds", 1, 0, 'o'},
 		{"no-fail", 0, 0, 'f'},
 		{"tiles-per-buf", 0, 0, 'p'},
-#define DUCTAPE 0xdead0001
 		{"remove-duct-tape", 0, 0, DUCTAPE},
-#define TILESZ	0xdead0002
 		{"tile-size", 1, 0, TILESZ},
-#define CHCK_RENDER 0xdead0003
 		{"check-render-cpyfn", 0, 0, CHCK_RENDER},
 		{NULL, 0, 0, 0},
 	};
@@ -666,209 +849,12 @@ static void parse_options(int argc, char **argv)
 	options.tiles_per_buf = options.scratch_buf_size / TILE_BYTES(options.tile_size);
 	options.check_render_cpyfn = 0;
 
-	while((c = getopt_long(argc, argv, "ds:g:c:t:rbuxmo:fp:",
-			       long_options, &option_index)) != -1) {
-		switch(c) {
-		case 'd':
-			options.no_hw = 1;
-			printf("no-hw debug mode\n");
-			break;
-		case 'S':
-			options.use_signal_helper = 0;
-			printf("disabling that pesky nuisance who keeps interrupting us\n");
-			break;
-		case 's':
-			tmp = atoi(optarg);
-			if (tmp < options.tile_size*8192)
-				printf("scratch buffer size needs to be at least %i\n",
-				       options.tile_size*8192);
-			else if (tmp & (tmp - 1)) {
-				printf("scratch buffer size needs to be a power-of-two\n");
-			} else {
-				printf("fixed scratch buffer size to %u\n", tmp);
-				options.scratch_buf_size = tmp;
-				sanitize_tiles_per_buf();
-			}
-			break;
-		case 'g':
-			tmp = atoi(optarg);
-			if (tmp < 0 || tmp > 10)
-				printf("gpu busy load needs to be bigger than 0 and smaller than 10\n");
-			else {
-				printf("gpu busy load factor set to %i\n", tmp);
-				gpu_busy_load = options.gpu_busy_load = tmp;
-			}
-			break;
-		case 'c':
-			options.num_buffers = atoi(optarg);
-			printf("buffer count set to %i\n", options.num_buffers);
-			break;
-		case 't':
-			options.trace_tile = atoi(optarg);
-			printf("tracing tile %i\n", options.trace_tile);
-			break;
-		case 'r':
-			options.use_render = 0;
-			printf("disabling render copy\n");
-			break;
-		case 'b':
-			options.use_blt = 0;
-			printf("disabling blt copy\n");
-			break;
-		case 'u':
-			options.forced_tiling = I915_TILING_NONE;
-			printf("disabling tiling\n");
-			break;
-		case 'x':
-			if (options.use_cpu_maps) {
-				printf("tiling not possible with cpu maps\n");
-			} else {
-				options.forced_tiling = I915_TILING_X;
-				printf("using only X-tiling\n");
-			}
-			break;
-		case 'm':
-			options.use_cpu_maps = 1;
-			options.forced_tiling = I915_TILING_NONE;
-			printf("disabling tiling\n");
-			break;
-		case 'o':
-			options.total_rounds = atoi(optarg);
-			printf("total rounds %i\n", options.total_rounds);
-			break;
-		case 'f':
-			options.fail = 0;
-			printf("not failing when detecting errors\n");
-			break;
-		case 'p':
-			options.tiles_per_buf = atoi(optarg);
-			printf("tiles per buffer %i\n", options.tiles_per_buf);
-			break;
-		case DUCTAPE:
-			options.ducttape = 0;
-			printf("applying duct-tape\n");
-			break;
-		case TILESZ:
-			options.tile_size = atoi(optarg);
-			sanitize_tiles_per_buf();
-			printf("til size %i\n", options.tile_size);
-			break;
-		case CHCK_RENDER:
-			options.check_render_cpyfn = 1;
-			printf("checking render copy function\n");
-			break;
-		default:
-			printf("unkown command options\n");
-			break;
-		}
-	}
+	igt_simple_init_parse_opts(&argc, argv,"ds:g:c:t:rbuxmo:fp:",
+				   long_options, NULL, parse_options, NULL);
 
-	if (optind < argc)
-		printf("unkown command options\n");
-
-	/* actually 32767, according to docs, but that kills our nice pot calculations. */
-	options.max_dimension = 16*1024;
-	if (options.use_render) {
-		if (IS_GEN2(devid) || IS_GEN3(devid))
-			options.max_dimension = 2048;
-		else
-			options.max_dimension = 8192;
-	}
-	printf("Limiting buffer to %dx%d\n",
-	       options.max_dimension, options.max_dimension);
-}
-
-static void init(void)
-{
-	int i;
-	unsigned tmp;
-
-	if (options.num_buffers == 0) {
-		tmp = gem_aperture_size(drm_fd);
-		tmp = tmp > 256*(1024*1024) ? 256*(1024*1024) : tmp;
-		num_buffers = 2 * tmp / options.scratch_buf_size / 3;
-		num_buffers /= 2;
-		printf("using %u buffers\n", num_buffers);
-	} else
-		num_buffers = options.num_buffers;
-
-	bufmgr = drm_intel_bufmgr_gem_init(drm_fd, 4096);
-	drm_intel_bufmgr_gem_enable_reuse(bufmgr);
-	drm_intel_bufmgr_gem_enable_fenced_relocs(bufmgr);
-	num_fences = get_num_fences();
-	batch = intel_batchbuffer_alloc(bufmgr, devid);
-
-	busy_bo = drm_intel_bo_alloc(bufmgr, "tiled bo", BUSY_BUF_SIZE, 4096);
-	if (options.forced_tiling >= 0)
-		gem_set_tiling(drm_fd, busy_bo->handle, options.forced_tiling, 4096);
-
-	for (i = 0; i < num_buffers; i++) {
-		init_buffer(&buffers[0][i], options.scratch_buf_size);
-		init_buffer(&buffers[1][i], options.scratch_buf_size);
-
-		num_total_tiles += buffers[0][i].num_tiles;
-	}
-	current_set = 0;
-
-	/* just in case it helps reproducability */
-	srandom(0xdeadbeef);
-}
-
-static void check_render_copyfunc(void)
-{
-	struct scratch_buf src, dst;
-	uint32_t *ptr;
-	int i, j, pass;
-
-	if (!options.check_render_cpyfn)
-		return;
-
-	init_buffer(&src, options.scratch_buf_size);
-	init_buffer(&dst, options.scratch_buf_size);
-
-	for (pass = 0; pass < 16; pass++) {
-		int sx = random() % (buf_width(&src)-options.tile_size);
-		int sy = random() % (buf_height(&src)-options.tile_size);
-		int dx = random() % (buf_width(&dst)-options.tile_size);
-		int dy = random() % (buf_height(&dst)-options.tile_size);
-
-		if (options.use_cpu_maps)
-			set_to_cpu_domain(&src, 1);
-
-		memset(src.data, 0xff, options.scratch_buf_size);
-		for (j = 0; j < options.tile_size; j++) {
-			ptr = (uint32_t*)((char *)src.data + sx*4 + (sy+j) * src.stride);
-			for (i = 0; i < options.tile_size; i++)
-				ptr[i] = j * options.tile_size + i;
-		}
-
-		render_copyfunc(&src, sx, sy, &dst, dx, dy, 0);
-
-		if (options.use_cpu_maps)
-			set_to_cpu_domain(&dst, 0);
-
-		for (j = 0; j < options.tile_size; j++) {
-			ptr = (uint32_t*)((char *)dst.data + dx*4 + (dy+j) * dst.stride);
-			for (i = 0; i < options.tile_size; i++)
-				if (ptr[i] != j * options.tile_size + i) {
-					printf("render copyfunc mismatch at (%d, %d): found %d, expected %d\n",
-					       i, j, ptr[i], j*options.tile_size + i);
-				}
-		}
-	}
-}
-
-
-int main(int argc, char **argv)
-{
-	int i, j;
-	unsigned *current_permutation, *tmp_permutation;
-
-	drm_fd = drm_open_any();
+	drm_fd = drm_open_driver(DRIVER_INTEL);
 	devid = intel_get_drm_devid(drm_fd);
 
-	parse_options(argc, argv);
-
 	/* start our little helper early before too may allocations occur */
 	if (options.use_signal_helper)
 		igt_fork_signal_helper();
@@ -887,10 +873,10 @@ int main(int argc, char **argv)
 	fan_out();
 
 	for (i = 0; i < options.total_rounds; i++) {
-		printf("round %i\n", i);
+		igt_info("round %i\n", i);
 		if (i % 64 == 63) {
 			fan_in_and_check();
-			printf("everything correct after %i rounds\n", i + 1);
+			igt_info("everything correct after %i rounds\n", i + 1);
 		}
 
 		target_set = (current_set + 1) & 1;
@@ -913,8 +899,7 @@ int main(int argc, char **argv)
 
 	fan_in_and_check();
 
-	fprintf(stderr, "num failed tiles %u, max incoherent bytes %zd\n",
-		stats.num_failed, stats.max_failed_reads*sizeof(uint32_t));
+	igt_info("num failed tiles %u, max incoherent bytes %zd\n", stats.num_failed, stats.max_failed_reads * sizeof(uint32_t));
 
 	intel_batchbuffer_free(batch);
 	drm_intel_bufmgr_destroy(bufmgr);
@@ -923,5 +908,5 @@ int main(int argc, char **argv)
 
 	igt_stop_signal_helper();
 
-	return 0;
+	igt_exit();
 }
