@@ -86,23 +86,9 @@ sync_ring(int fd, unsigned ring, int num_children, int timeout)
 	int num_engines = 0;
 
 	if (ring == ~0u) {
-		const struct intel_execution_engine *e;
-
-		for (e = intel_execution_engines; e->name; e++) {
-			if (e->exec_id == 0)
-				continue;
-
-			if (!gem_has_ring(fd, e->exec_id | e->flags))
-				continue;
-
-			if (e->exec_id == I915_EXEC_BSD) {
-				int is_bsd2 = e->flags != 0;
-				if (gem_has_bsd2(fd) != is_bsd2)
-					continue;
-			}
-
-			names[num_engines] = e->name;
-			engines[num_engines++] = e->exec_id | e->flags;
+		for_each_physical_engine(fd, ring) {
+			names[num_engines] = e__->name;
+			engines[num_engines++] = ring;
 			if (num_engines == ARRAY_SIZE(engines))
 				break;
 		}
@@ -152,6 +138,45 @@ sync_ring(int fd, unsigned ring, int num_children, int timeout)
 	igt_assert_eq(intel_detect_and_clear_missed_interrupts(fd), 0);
 }
 
+static void
+idle_ring(int fd, unsigned ring, int timeout)
+{
+	const uint32_t bbe = MI_BATCH_BUFFER_END;
+	struct drm_i915_gem_exec_object2 object;
+	struct drm_i915_gem_execbuffer2 execbuf;
+	double start, elapsed;
+	unsigned long cycles;
+
+	gem_require_ring(fd, ring);
+
+	memset(&object, 0, sizeof(object));
+	object.handle = gem_create(fd, 4096);
+	gem_write(fd, object.handle, 0, &bbe, sizeof(bbe));
+
+	memset(&execbuf, 0, sizeof(execbuf));
+	execbuf.buffers_ptr = to_user_pointer(&object);
+	execbuf.buffer_count = 1;
+	execbuf.flags = ring;
+	gem_execbuf(fd, &execbuf);
+	gem_sync(fd, object.handle);
+
+	intel_detect_and_clear_missed_interrupts(fd);
+	start = gettime();
+	cycles = 0;
+	do {
+		do {
+			gem_execbuf(fd, &execbuf);
+			gem_quiescent_gpu(fd);
+		} while (++cycles & 1023);
+	} while ((elapsed = gettime() - start) < timeout);
+	igt_assert_eq(intel_detect_and_clear_missed_interrupts(fd), 0);
+
+	igt_info("Completed %ld cycles: %.3f us\n",
+		 cycles, elapsed*1e6/cycles);
+
+	gem_close(fd, object.handle);
+}
+
 static void
 store_ring(int fd, unsigned ring, int num_children, int timeout)
 {
@@ -161,26 +186,12 @@ store_ring(int fd, unsigned ring, int num_children, int timeout)
 	int num_engines = 0;
 
 	if (ring == ~0u) {
-		const struct intel_execution_engine *e;
-
-		for (e = intel_execution_engines; e->name; e++) {
-			if (e->exec_id == 0)
+		for_each_physical_engine(fd, ring) {
+			if (!gem_can_store_dword(fd, ring))
 				continue;
 
-			if (!gem_has_ring(fd, e->exec_id | e->flags))
-				continue;
-
-			if (!gem_can_store_dword(fd, e->exec_id | e->flags))
-				continue;
-
-			if (e->exec_id == I915_EXEC_BSD) {
-				int is_bsd2 = e->flags != 0;
-				if (gem_has_bsd2(fd) != is_bsd2)
-					continue;
-			}
-
-			names[num_engines] = e->name;
-			engines[num_engines++] = e->exec_id | e->flags;
+			names[num_engines] = e__->name;
+			engines[num_engines++] = ring;
 			if (num_engines == ARRAY_SIZE(engines))
 				break;
 		}
@@ -463,31 +474,17 @@ store_many(int fd, unsigned ring, int timeout)
 	intel_detect_and_clear_missed_interrupts(fd);
 
 	if (ring == ~0u) {
-		const struct intel_execution_engine *e;
-
-		for (e = intel_execution_engines; e->name; e++) {
-			if (e->exec_id == 0)
+		for_each_physical_engine(fd, ring) {
+			if (!gem_can_store_dword(fd, ring))
 				continue;
 
-			if (!gem_has_ring(fd, e->exec_id | e->flags))
-				continue;
-
-			if (!gem_can_store_dword(fd, e->exec_id | e->flags))
-				continue;
-
-			if (e->exec_id == I915_EXEC_BSD) {
-				int is_bsd2 = e->flags != 0;
-				if (gem_has_bsd2(fd) != is_bsd2)
-					continue;
-			}
-
 			igt_fork(child, 1)
 				__store_many(fd,
-					     e->exec_id | e->flags,
+					     ring,
 					     timeout,
 					     &shared[n]);
 
-			names[n++] = e->name;
+			names[n++] = e__->name;
 		}
 		igt_waitchildren();
 	} else {
@@ -508,24 +505,11 @@ store_many(int fd, unsigned ring, int timeout)
 static void
 sync_all(int fd, int num_children, int timeout)
 {
-	const struct intel_execution_engine *e;
-	unsigned engines[16];
+	unsigned engines[16], engine;
 	int num_engines = 0;
 
-	for (e = intel_execution_engines; e->name; e++) {
-		if (e->exec_id == 0)
-			continue;
-
-		if (!gem_has_ring(fd, e->exec_id | e->flags))
-			continue;
-
-		if (e->exec_id == I915_EXEC_BSD) {
-			int is_bsd2 = e->flags != 0;
-			if (gem_has_bsd2(fd) != is_bsd2)
-				continue;
-		}
-
-		engines[num_engines++] = e->exec_id | e->flags;
+	for_each_physical_engine(fd, engine) {
+		engines[num_engines++] = engine;
 		if (num_engines == ARRAY_SIZE(engines))
 			break;
 	}
@@ -573,27 +557,15 @@ static void
 store_all(int fd, int num_children, int timeout)
 {
 	const int gen = intel_gen(intel_get_drm_devid(fd));
-	const struct intel_execution_engine *e;
 	unsigned engines[16];
 	int num_engines = 0;
+	unsigned int ring;
 
-	for (e = intel_execution_engines; e->name; e++) {
-		if (e->exec_id == 0)
+	for_each_physical_engine(fd, ring) {
+		if (!gem_can_store_dword(fd, ring))
 			continue;
 
-		if (!gem_has_ring(fd, e->exec_id | e->flags))
-			continue;
-
-		if (!gem_can_store_dword(fd, e->exec_id))
-			continue;
-
-		if (e->exec_id == I915_EXEC_BSD) {
-			int is_bsd2 = e->flags != 0;
-			if (gem_has_bsd2(fd) != is_bsd2)
-				continue;
-		}
-
-		engines[num_engines++] = e->exec_id | e->flags;
+		engines[num_engines++] = ring;
 		if (num_engines == ARRAY_SIZE(engines))
 			break;
 	}
@@ -698,23 +670,9 @@ preempt(int fd, unsigned ring, int num_children, int timeout)
 	uint32_t ctx[2];
 
 	if (ring == ~0u) {
-		const struct intel_execution_engine *e;
-
-		for (e = intel_execution_engines; e->name; e++) {
-			if (e->exec_id == 0)
-				continue;
-
-			if (!gem_has_ring(fd, e->exec_id | e->flags))
-				continue;
-
-			if (e->exec_id == I915_EXEC_BSD) {
-				int is_bsd2 = e->flags != 0;
-				if (gem_has_bsd2(fd) != is_bsd2)
-					continue;
-			}
-
-			names[num_engines] = e->name;
-			engines[num_engines++] = e->exec_id | e->flags;
+		for_each_physical_engine(fd, ring) {
+			names[num_engines] = e__->name;
+			engines[num_engines++] = ring;
 			if (num_engines == ARRAY_SIZE(engines))
 				break;
 		}
@@ -802,6 +760,8 @@ igt_main
 	for (e = intel_execution_engines; e->name; e++) {
 		igt_subtest_f("%s", e->name)
 			sync_ring(fd, e->exec_id | e->flags, 1, 150);
+		igt_subtest_f("idle-%s", e->name)
+			idle_ring(fd, e->exec_id | e->flags, 150);
 		igt_subtest_f("store-%s", e->name)
 			store_ring(fd, e->exec_id | e->flags, 1, 150);
 		igt_subtest_f("many-%s", e->name)
@@ -839,6 +799,7 @@ igt_main
 
 	igt_subtest_group {
 		igt_fixture {
+			gem_require_contexts(fd);
 			igt_require(gem_scheduler_has_ctx_priority(fd));
 			igt_require(gem_scheduler_has_preemption(fd));
 		}
