@@ -44,6 +44,8 @@
  * Note that this test will use swap in an effort to test all of ram.
  */
 
+#include "igt.h"
+#include "igt_x86.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -56,10 +58,6 @@
 
 #include <drm.h>
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "intel_io.h"
-#include "igt_aux.h"
 
 IGT_TEST_DESCRIPTION("Test swizzling by testing pwrite does the inverse of"
 		     " pread.");
@@ -82,7 +80,8 @@ create_bo_and_fill(int fd)
 	gem_set_tiling(fd, handle, current_tiling_mode, WIDTH * sizeof(uint32_t));
 
 	/* Fill the BO with dwords starting at start_val */
-	data = gem_mmap(fd, handle, sizeof(linear), PROT_READ | PROT_WRITE);
+	data = gem_mmap__gtt(fd, handle, sizeof(linear),
+			     PROT_READ | PROT_WRITE);
 	for (i = 0; i < WIDTH*HEIGHT; i++)
 		data[i] = i;
 	munmap(data, sizeof(linear));
@@ -101,19 +100,25 @@ create_bo(int fd)
 	return handle;
 }
 
+static void copy_wc_page(void *dst, const void *src)
+{
+	igt_memcpy_from_wc(dst, src, PAGE_SIZE);
+}
+
 igt_simple_main
 {
-	int fd;
-	uint32_t *data;
-	int i, j;
 	uint32_t tiling, swizzle;
-	uint32_t handle, handle_target;
 	int count;
+	int fd;
 	
-	fd = drm_open_any();
+	fd = drm_open_driver(DRIVER_INTEL);
 	count = SLOW_QUICK(intel_get_total_ram_mb() * 9 / 10, 8) ;
 
-	for (i = 0; i < count/2; i++) {
+	for (int i = 0; i < count/2; i++) {
+		uint32_t handle, handle_target;
+		char *data;
+		int n;
+
 		current_tiling_mode = I915_TILING_X;
 
 		handle = create_bo_and_fill(fd);
@@ -125,11 +130,18 @@ igt_simple_main
 		gem_write(fd, handle_target, 0, linear, sizeof(linear));
 
 		/* Check the target bo's contents. */
-		data = gem_mmap(fd, handle_target, sizeof(linear), PROT_READ | PROT_WRITE);
-		for (j = 0; j < WIDTH*HEIGHT; j++)
-			igt_assert_f(data[j] == j,
-				     "mismatch at %i: %i\n",
-				     j, data[j]);
+		data = gem_mmap__gtt(fd, handle_target, sizeof(linear), PROT_READ);
+		n = 0;
+		for (int pfn = 0; pfn < sizeof(linear)/PAGE_SIZE; pfn++) {
+			uint32_t page[PAGE_SIZE/sizeof(uint32_t)];
+			copy_wc_page(page, data + PAGE_SIZE*pfn);
+			for (int j = 0; j < PAGE_SIZE/sizeof(uint32_t); j++) {
+				igt_assert_f(page[j] == n,
+					     "mismatch at %i: %i\n",
+					     n, page[j]);
+				n++;
+			}
+		}
 		munmap(data, sizeof(linear));
 
 		/* Leak both bos so that we use all of system mem! */
