@@ -21,18 +21,18 @@
  * IN THE SOFTWARE.
  */
 
+#include "igt.h"
 #include <errno.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <string.h>
+#include <fcntl.h>
 
-#include "drmtest.h"
-#include "igt_debugfs.h"
-#include "igt_kms.h"
 
 typedef struct {
 	int drm_fd;
 	igt_display_t display;
+	int gen;
 } data_t;
 
 typedef struct {
@@ -55,35 +55,42 @@ typedef struct {
 	struct igt_fb red_fb, blue_fb;
 } pageflip_test_t;
 
+typedef struct {
+	data_t *data;
+	int x, y;
+	int w, h;
+	struct igt_fb biggreen_fb, smallred_fb, smallblue_fb;
+} gen9_test_t;
+
 static void
 functional_test_init(functional_test_t *test, igt_output_t *output, enum pipe pipe)
 {
 	data_t *data = test->data;
 	drmModeModeInfo *mode;
 
-	test->pipe_crc = igt_pipe_crc_new(pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
+	test->pipe_crc = igt_pipe_crc_new(data->drm_fd, pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
 
 	igt_output_set_pipe(output, pipe);
 
 	mode = igt_output_get_mode(output);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
 				DRM_FORMAT_XRGB8888,
-				false, /* tiled */
+				LOCAL_DRM_FORMAT_MOD_NONE,
 				0.0, 0.0, 0.0,
 				&test->black_fb);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
 				DRM_FORMAT_XRGB8888,
-				false, /* tiled */
+				LOCAL_DRM_FORMAT_MOD_NONE,
 				0.0, 0.0, 1.0,
 				&test->blue_fb);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
 				DRM_FORMAT_XRGB8888,
-				false, /* tiled */
+				LOCAL_DRM_FORMAT_MOD_NONE,
 				1.0, 1.0, 0.0,
 				&test->yellow_fb);
 	igt_create_color_fb(data->drm_fd, 100, 100,
 				DRM_FORMAT_XRGB8888,
-				false, /* tiled */
+				LOCAL_DRM_FORMAT_MOD_NONE,
 				1.0, 0.0, 0.0,
 				&test->red_fb);
 
@@ -125,11 +132,9 @@ functional_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	functional_test_t test = { .data = data };
 	igt_display_t *display = &data->display;
 	igt_plane_t *primary, *sprite;
-	int ret;
 	int num_primary = 0, num_cursor = 0;
 	int i;
 
-	igt_assert(data->display.has_universal_planes);
 	igt_skip_on(pipe >= display->n_pipes);
 
 	igt_info("Testing connector %s using pipe %s\n", igt_output_name(output),
@@ -144,16 +149,16 @@ functional_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	 * boolean and show up in userspace as the wrong type.
 	 */
 	for (i = 0; i < display->pipes[pipe].n_planes; i++)
-		if (display->pipes[pipe].planes[i].is_primary)
+		if (display->pipes[pipe].planes[i].type == DRM_PLANE_TYPE_PRIMARY)
 			num_primary++;
-		else if (display->pipes[pipe].planes[i].is_cursor)
+		else if (display->pipes[pipe].planes[i].type == DRM_PLANE_TYPE_CURSOR)
 			num_cursor++;
 
-	igt_assert(num_primary == 1);
-	igt_assert(num_cursor <= 1);
+	igt_assert_eq(num_primary, 1);
+	igt_assert_lte(num_cursor, 1);
 
-	primary = igt_output_get_plane(output, IGT_PLANE_PRIMARY);
-	sprite = igt_output_get_plane(output, IGT_PLANE_2);
+	primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
+	sprite = igt_output_get_plane_type(output, DRM_PLANE_TYPE_OVERLAY);
 	if (!sprite) {
 		functional_test_fini(&test, output);
 		igt_skip("No sprite plane available\n");
@@ -217,10 +222,9 @@ functional_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	igt_display_commit2(display, COMMIT_UNIVERSAL);
 
 	/* Step 10: Enable crtc (fb = -1), take CRC (CRC 7) */
-	ret = drmModeSetCrtc(data->drm_fd, output->config.crtc->crtc_id, -1,
-			     0, 0, &output->config.connector->connector_id,
-			     1, test.mode);
-	igt_assert(ret == 0);
+	igt_assert(drmModeSetCrtc(data->drm_fd, output->config.crtc->crtc_id, -1,
+				  0, 0, &output->config.connector->connector_id,
+				  1, test.mode) == 0);
 	igt_pipe_crc_collect_crc(test.pipe_crc, &test.crc_7);
 
 	/* Step 11: Disable primary plane */
@@ -238,16 +242,15 @@ functional_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	igt_display_commit2(display, COMMIT_LEGACY);
 
 	/* Step 14: Universal API, set primary completely offscreen (CRC 9) */
-	ret = drmModeSetPlane(data->drm_fd, primary->drm_plane->plane_id,
-			      output->config.crtc->crtc_id,
-			      test.blue_fb.fb_id, 0,
-			      9000, 9000,
-			      test.mode->hdisplay,
-			      test.mode->vdisplay,
-			      IGT_FIXED(0,0), IGT_FIXED(0,0),
-			      IGT_FIXED(test.mode->hdisplay,0),
-			      IGT_FIXED(test.mode->vdisplay,0));
-	igt_assert(ret == 0);
+	igt_assert(drmModeSetPlane(data->drm_fd, primary->drm_plane->plane_id,
+				   output->config.crtc->crtc_id,
+				   test.blue_fb.fb_id, 0,
+				   9000, 9000,
+				   test.mode->hdisplay,
+				   test.mode->vdisplay,
+				   IGT_FIXED(0,0), IGT_FIXED(0,0),
+				   IGT_FIXED(test.mode->hdisplay,0),
+				   IGT_FIXED(test.mode->vdisplay,0)) == 0);
 	igt_pipe_crc_collect_crc(test.pipe_crc, &test.crc_9);
 
 	/*
@@ -264,37 +267,37 @@ functional_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	igt_display_commit2(display, COMMIT_LEGACY);
 
 	/* Blue bg + red sprite should be same under both types of API's */
-	igt_assert(igt_crc_equal(&test.crc_2, &test.crc_4));
+	igt_assert_crc_equal(&test.crc_2, &test.crc_4);
 
 	/* Disabling primary plane should be same as black primary */
-	igt_assert(igt_crc_equal(&test.crc_1, &test.crc_5));
+	igt_assert_crc_equal(&test.crc_1, &test.crc_5);
 
 	/* Re-enabling primary should return to blue properly */
-	igt_assert(igt_crc_equal(&test.crc_2, &test.crc_6));
+	igt_assert_crc_equal(&test.crc_2, &test.crc_6);
 
 	/*
 	 * We should be able to setup plane FB's while CRTC is disabled and
 	 * then have them pop up correctly when the CRTC is re-enabled.
 	 */
-	igt_assert(igt_crc_equal(&test.crc_2, &test.crc_7));
+	igt_assert_crc_equal(&test.crc_2, &test.crc_7);
 
 	/*
 	 * We should be able to modeset with the primary plane off
 	 * successfully
 	 */
-	igt_assert(igt_crc_equal(&test.crc_3, &test.crc_8));
+	igt_assert_crc_equal(&test.crc_3, &test.crc_8);
 
 	/*
 	 * We should be able to move the primary plane completely offscreen
 	 * and have it disable successfully.
 	 */
-	igt_assert(igt_crc_equal(&test.crc_5, &test.crc_9));
+	igt_assert_crc_equal(&test.crc_5, &test.crc_9);
 
 	/*
 	 * We should be able to explicitly disable an already
 	 * implicitly-disabled primary plane
 	 */
-	igt_assert(igt_crc_equal(&test.crc_5, &test.crc_10));
+	igt_assert_crc_equal(&test.crc_5, &test.crc_10);
 
 	igt_plane_set_fb(primary, NULL);
 	igt_plane_set_fb(sprite, NULL);
@@ -313,19 +316,19 @@ sanity_test_init(sanity_test_t *test, igt_output_t *output, enum pipe pipe)
 	mode = igt_output_get_mode(output);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
 			    DRM_FORMAT_XRGB8888,
-			    false, /* tiled */
+			    LOCAL_DRM_FORMAT_MOD_NONE,
 			    0.0, 0.0, 1.0,
 			    &test->blue_fb);
 	igt_create_color_fb(data->drm_fd,
 			    mode->hdisplay + 100, mode->vdisplay + 100,
 			    DRM_FORMAT_XRGB8888,
-			    false, /* tiled */
+			    LOCAL_DRM_FORMAT_MOD_NONE,
 			    0.0, 0.0, 1.0,
 			    &test->oversized_fb);
 	igt_create_color_fb(data->drm_fd,
 			    mode->hdisplay - 100, mode->vdisplay - 100,
 			    DRM_FORMAT_XRGB8888,
-			    false, /* tiled */
+			    LOCAL_DRM_FORMAT_MOD_NONE,
 			    0.0, 0.0, 1.0,
 			    &test->undersized_fb);
 
@@ -357,14 +360,17 @@ sanity_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	sanity_test_t test = { .data = data };
 	igt_plane_t *primary;
 	drmModeModeInfo *mode;
-	int i, ret = 0;
+	int i;
+	int expect;
+
+	igt_skip_on(pipe >= data->display.n_pipes);
 
 	igt_output_set_pipe(output, pipe);
 	mode = igt_output_get_mode(output);
 
 	sanity_test_init(&test, output, pipe);
 
-	primary = igt_output_get_plane(output, IGT_PLANE_PRIMARY);
+	primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
 
 	/* Use legacy API to set a mode with a blue FB */
 	igt_plane_set_fb(primary, &test.blue_fb);
@@ -372,57 +378,55 @@ sanity_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 
 	/*
 	 * Try to use universal plane API to set primary plane that
-	 * doesn't cover CRTC (should fail).
+	 * doesn't cover CRTC (should fail on pre-gen9 and succeed on
+	 * gen9+).
 	 */
 	igt_plane_set_fb(primary, &test.undersized_fb);
-	ret = igt_display_try_commit2(&data->display, COMMIT_UNIVERSAL);
-	igt_assert(ret == -EINVAL);
+	expect = (data->gen < 9) ? -EINVAL : 0;
+	igt_assert(igt_display_try_commit2(&data->display, COMMIT_UNIVERSAL) == expect);
 
 	/* Same as above, but different plane positioning. */
 	igt_plane_set_position(primary, 100, 100);
-	ret = igt_display_try_commit2(&data->display, COMMIT_UNIVERSAL);
-	igt_assert(ret == -EINVAL);
+	igt_assert(igt_display_try_commit2(&data->display, COMMIT_UNIVERSAL) == expect);
 
 	igt_plane_set_position(primary, 0, 0);
 
-	/* Try to use universal plane API to scale down (should fail) */
-	ret = drmModeSetPlane(data->drm_fd, primary->drm_plane->plane_id,
-			      output->config.crtc->crtc_id,
-			      test.oversized_fb.fb_id, 0,
-			      0, 0,
-			      mode->hdisplay + 100,
-			      mode->vdisplay + 100,
-			      IGT_FIXED(0,0), IGT_FIXED(0,0),
-			      IGT_FIXED(mode->hdisplay,0),
-			      IGT_FIXED(mode->vdisplay,0));
-	igt_assert(ret == -ERANGE);
+	/* Try to use universal plane API to scale down (should fail on pre-gen9) */
+	expect = (data->gen < 9) ? -ERANGE : 0;
+	igt_assert(drmModeSetPlane(data->drm_fd, primary->drm_plane->plane_id,
+				   output->config.crtc->crtc_id,
+				   test.oversized_fb.fb_id, 0,
+				   0, 0,
+				   mode->hdisplay + 100,
+				   mode->vdisplay + 100,
+				   IGT_FIXED(0,0), IGT_FIXED(0,0),
+				   IGT_FIXED(mode->hdisplay,0),
+				   IGT_FIXED(mode->vdisplay,0)) == expect);
 
-	/* Try to use universal plane API to scale up (should fail) */
-	ret = drmModeSetPlane(data->drm_fd, primary->drm_plane->plane_id,
-			      output->config.crtc->crtc_id,
-			      test.oversized_fb.fb_id, 0,
-			      0, 0,
-			      mode->hdisplay,
-			      mode->vdisplay,
-			      IGT_FIXED(0,0), IGT_FIXED(0,0),
-			      IGT_FIXED(mode->hdisplay - 100,0),
-			      IGT_FIXED(mode->vdisplay - 100,0));
-	igt_assert(ret == -ERANGE);
+	/* Try to use universal plane API to scale up (should fail on pre-gen9) */
+	igt_assert(drmModeSetPlane(data->drm_fd, primary->drm_plane->plane_id,
+				   output->config.crtc->crtc_id,
+				   test.oversized_fb.fb_id, 0,
+				   0, 0,
+				   mode->hdisplay,
+				   mode->vdisplay,
+				   IGT_FIXED(0,0), IGT_FIXED(0,0),
+				   IGT_FIXED(mode->hdisplay - 100,0),
+				   IGT_FIXED(mode->vdisplay - 100,0)) == expect);
 
 	/* Find other crtcs and try to program our primary plane on them */
 	for (i = 0; i < test.moderes->count_crtcs; i++)
 		if (test.moderes->crtcs[i] != output->config.crtc->crtc_id) {
-			ret = drmModeSetPlane(data->drm_fd,
-					      primary->drm_plane->plane_id,
-					      test.moderes->crtcs[i],
-					      test.blue_fb.fb_id, 0,
-					      0, 0,
-					      mode->hdisplay,
-					      mode->vdisplay,
-					      IGT_FIXED(0,0), IGT_FIXED(0,0),
-					      IGT_FIXED(mode->hdisplay,0),
-					      IGT_FIXED(mode->vdisplay,0));
-			igt_assert(ret == -EINVAL);
+			igt_assert(drmModeSetPlane(data->drm_fd,
+						   primary->drm_plane->plane_id,
+						   test.moderes->crtcs[i],
+						   test.blue_fb.fb_id, 0,
+						   0, 0,
+						   mode->hdisplay,
+						   mode->vdisplay,
+						   IGT_FIXED(0,0), IGT_FIXED(0,0),
+						   IGT_FIXED(mode->hdisplay,0),
+						   IGT_FIXED(mode->vdisplay,0)) == -EINVAL);
 		}
 
 	igt_plane_set_fb(primary, NULL);
@@ -440,12 +444,12 @@ pageflip_test_init(pageflip_test_t *test, igt_output_t *output, enum pipe pipe)
 	mode = igt_output_get_mode(output);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
 			    DRM_FORMAT_XRGB8888,
-			    false, /* tiled */
+			    LOCAL_DRM_FORMAT_MOD_NONE,
 			    1.0, 0.0, 0.0,
 			    &test->red_fb);
 	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
 			    DRM_FORMAT_XRGB8888,
-			    false, /* tiled */
+			    LOCAL_DRM_FORMAT_MOD_NONE,
 			    0.0, 0.0, 1.0,
 			    &test->blue_fb);
 }
@@ -466,16 +470,18 @@ pageflip_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	pageflip_test_t test = { .data = data };
 	igt_plane_t *primary;
 	struct timeval timeout = { .tv_sec = 0, .tv_usec = 500 };
-	drmEventContext evctx = { .version = DRM_EVENT_CONTEXT_VERSION };
+	drmEventContext evctx = { .version = 2 };
 
 	fd_set fds;
 	int ret = 0;
 
+	igt_skip_on(pipe >= data->display.n_pipes);
+
 	igt_output_set_pipe(output, pipe);
 
 	pageflip_test_init(&test, output, pipe);
 
-	primary = igt_output_get_plane(output, IGT_PLANE_PRIMARY);
+	primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
 
 	/* Use legacy API to set a mode with a blue FB */
 	igt_plane_set_fb(primary, &test.blue_fb);
@@ -491,9 +497,8 @@ pageflip_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	 * Note that crtc->primary->fb = NULL causes flip to return EBUSY for
 	 * historical reasons...
 	 */
-	ret = drmModePageFlip(data->drm_fd, output->config.crtc->crtc_id,
-			      test.red_fb.fb_id, 0, NULL);
-	igt_assert(ret == -EBUSY);
+	igt_assert(drmModePageFlip(data->drm_fd, output->config.crtc->crtc_id,
+				   test.red_fb.fb_id, 0, NULL) == -EBUSY);
 
 	/* Turn primary plane back on */
 	igt_plane_set_fb(primary, &test.blue_fb);
@@ -506,10 +511,9 @@ pageflip_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	 * completes, which we don't have a good way to specifically test for,
 	 * but at least we can make sure that nothing blows up.
 	 */
-	ret = drmModePageFlip(data->drm_fd, output->config.crtc->crtc_id,
-			      test.red_fb.fb_id, DRM_MODE_PAGE_FLIP_EVENT,
-			      &test);
-	igt_assert(ret == 0);
+	igt_assert(drmModePageFlip(data->drm_fd, output->config.crtc->crtc_id,
+				   test.red_fb.fb_id, DRM_MODE_PAGE_FLIP_EVENT,
+				   &test) == 0);
 	igt_plane_set_fb(primary, NULL);
 	igt_display_commit2(&data->display, COMMIT_UNIVERSAL);
 
@@ -519,57 +523,288 @@ pageflip_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
 	do {
 		ret = select(data->drm_fd + 1, &fds, NULL, NULL, &timeout);
 	} while (ret < 0 && errno == EINTR);
-	igt_assert(ret == 1);
-	ret = drmHandleEvent(data->drm_fd, &evctx);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 1);
+	igt_assert(drmHandleEvent(data->drm_fd, &evctx) == 0);
 
 	igt_plane_set_fb(primary, NULL);
 	pageflip_test_fini(&test, output);
 }
 
+static void
+cursor_leak_test_fini(data_t *data,
+		      igt_output_t *output,
+		      struct igt_fb *bg,
+		      struct igt_fb *curs)
+{
+	int i;
+
+	igt_remove_fb(data->drm_fd, bg);
+	for (i = 0; i < 10; i++)
+		igt_remove_fb(data->drm_fd, &curs[i]);
+
+	igt_output_set_pipe(output, PIPE_ANY);
+}
+
+static int
+i915_gem_fb_count(data_t *data)
+{
+	char buf[1024];
+	FILE *fp;
+	int fd;
+	int count = 0;
+
+	fd = igt_debugfs_open(data->drm_fd, "i915_gem_framebuffer", O_RDONLY);
+	fp = fdopen(fd, "r");
+	igt_require(fp);
+	while (fgets(buf, sizeof(buf), fp) != NULL)
+		count++;
+	fclose(fp);
+	close(fd);
+
+	return count;
+}
+
+static void
+cursor_leak_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
+{
+	igt_display_t *display = &data->display;
+	igt_plane_t *primary, *cursor;
+	drmModeModeInfo *mode;
+	struct igt_fb background_fb;
+	struct igt_fb cursor_fb[10];
+	int i;
+	int r, g, b;
+	int count1, count2;
+
+	igt_skip_on(pipe >= display->n_pipes);
+	igt_require(display->has_cursor_plane);
+
+	igt_output_set_pipe(output, pipe);
+	mode = igt_output_get_mode(output);
+
+	/* Count GEM framebuffers before creating our cursor FB's */
+	count1 = i915_gem_fb_count(data);
+
+	/* Black background FB */
+	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
+			    DRM_FORMAT_XRGB8888,
+			    false,
+			    0.0, 0.0, 0.0,
+			    &background_fb);
+
+	/* Random color cursors */
+	for (i = 0; i < 10; i++) {
+		r = rand() % 0xFF;
+		g = rand() % 0xFF;
+		b = rand() % 0xFF;
+		igt_create_color_fb(data->drm_fd, 64, 64,
+				    DRM_FORMAT_ARGB8888,
+				    false,
+				    (double)r / 0xFF,
+				    (double)g / 0xFF,
+				    (double)b / 0xFF,
+				    &cursor_fb[i]);
+	}
+
+	primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
+	cursor = igt_output_get_plane_type(output, DRM_PLANE_TYPE_CURSOR);
+	if (!primary || !cursor) {
+		cursor_leak_test_fini(data, output, &background_fb, cursor_fb);
+		igt_skip("Primary and/or cursor are unavailable\n");
+	}
+
+
+	igt_plane_set_fb(primary, &background_fb);
+	igt_display_commit2(display, COMMIT_LEGACY);
+
+	igt_plane_set_position(cursor, 100, 100);
+
+	/*
+	 * Exercise both legacy and universal code paths.  Note that legacy
+	 * handling in the kernel redirects through universal codepaths
+	 * internally, so that redirection is where we're most worried about
+	 * leaking.
+	 */
+	for (i = 0; i < 10; i++) {
+		igt_plane_set_fb(cursor, &cursor_fb[i]);
+		igt_display_commit2(display, COMMIT_UNIVERSAL);
+	}
+	for (i = 0; i < 10; i++) {
+		igt_plane_set_fb(cursor, &cursor_fb[i]);
+		igt_display_commit2(display, COMMIT_LEGACY);
+	}
+
+	/* Release our framebuffer handles before we take a second count */
+	igt_plane_set_fb(primary, NULL);
+	igt_plane_set_fb(cursor, NULL);
+	igt_display_commit2(display, COMMIT_LEGACY);
+	cursor_leak_test_fini(data, output, &background_fb, cursor_fb);
+
+	/* We should be back to the same framebuffer count as when we started */
+	count2 = i915_gem_fb_count(data);
+
+	igt_assert_eq(count1, count2);
+}
+
+static void
+gen9_test_init(gen9_test_t *test, igt_output_t *output, enum pipe pipe)
+{
+	data_t *data = test->data;
+	drmModeModeInfo *mode;
+
+	igt_output_set_pipe(output, pipe);
+
+	mode = igt_output_get_mode(output);
+	test->w = mode->hdisplay / 2;
+	test->h = mode->vdisplay / 2;
+	test->x = mode->hdisplay / 4;
+	test->y = mode->vdisplay / 4;
+
+	/* Initial framebuffer of full CRTC size */
+	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
+			    DRM_FORMAT_XRGB8888,
+			    LOCAL_DRM_FORMAT_MOD_NONE,
+			    0.0, 1.0, 0.0,
+			    &test->biggreen_fb);
+
+	/* Framebuffers that only cover a quarter of the CRTC size */
+	igt_create_color_fb(data->drm_fd, test->w, test->h,
+			    DRM_FORMAT_XRGB8888,
+			    LOCAL_DRM_FORMAT_MOD_NONE,
+			    1.0, 0.0, 0.0,
+			    &test->smallred_fb);
+	igt_create_color_fb(data->drm_fd, test->w, test->h,
+			    DRM_FORMAT_XRGB8888,
+			    LOCAL_DRM_FORMAT_MOD_NONE,
+			    0.0, 0.0, 1.0,
+			    &test->smallblue_fb);
+}
+
+static void
+gen9_test_fini(gen9_test_t *test, igt_output_t *output)
+{
+	igt_remove_fb(test->data->drm_fd, &test->biggreen_fb);
+	igt_remove_fb(test->data->drm_fd, &test->smallred_fb);
+	igt_remove_fb(test->data->drm_fd, &test->smallblue_fb);
+
+	igt_output_set_pipe(output, PIPE_ANY);
+	igt_display_commit2(&test->data->display, COMMIT_LEGACY);
+}
+
+/*
+ * Test features specific to gen9+ platforms (i.e., primary plane
+ * windowing)
+ */
+static void
+gen9_test_pipe(data_t *data, enum pipe pipe, igt_output_t *output)
+{
+	gen9_test_t test = { .data = data };
+	igt_plane_t *primary;
+
+	int ret = 0;
+
+	igt_skip_on(data->gen < 9);
+	igt_skip_on(pipe >= data->display.n_pipes);
+
+	igt_output_set_pipe(output, pipe);
+
+	gen9_test_init(&test, output, pipe);
+
+	primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
+
+	/* Start with a full-screen primary plane */
+	igt_plane_set_fb(primary, &test.biggreen_fb);
+	igt_display_commit2(&data->display, COMMIT_LEGACY);
+
+	/* Set primary to windowed size/position */
+	igt_plane_set_fb(primary, &test.smallblue_fb);
+	igt_plane_set_position(primary, test.x, test.y);
+	igt_plane_set_size(primary, test.w, test.h);
+	igt_display_commit2(&data->display, COMMIT_UNIVERSAL);
+
+	/*
+	 * SetPlane update to another framebuffer of the same size
+	 * should succeed
+	 */
+	igt_plane_set_fb(primary, &test.smallred_fb);
+	igt_plane_set_position(primary, test.x, test.y);
+	igt_plane_set_size(primary, test.w, test.h);
+	igt_display_commit2(&data->display, COMMIT_UNIVERSAL);
+
+	/* PageFlip should also succeed */
+	ret = drmModePageFlip(data->drm_fd, output->config.crtc->crtc_id,
+			      test.smallblue_fb.fb_id, 0, NULL);
+	igt_assert_eq(ret, 0);
+
+	igt_plane_set_fb(primary, NULL);
+	igt_plane_set_position(primary, 0, 0);
+	gen9_test_fini(&test, output);
+}
+
 static void
 run_tests_for_pipe(data_t *data, enum pipe pipe)
 {
 	igt_output_t *output;
 
+	igt_fixture {
+		int valid_tests = 0;
+
+		igt_skip_on(pipe >= data->display.n_pipes);
+
+		for_each_valid_output_on_pipe(&data->display, pipe, output)
+			valid_tests++;
+
+		igt_require_f(valid_tests, "no valid crtc/connector combinations found\n");
+	}
+
 	igt_subtest_f("universal-plane-pipe-%s-functional",
 		      kmstest_pipe_name(pipe))
-		for_each_connected_output(&data->display, output)
+		for_each_valid_output_on_pipe(&data->display, pipe, output)
 			functional_test_pipe(data, pipe, output);
 
 	igt_subtest_f("universal-plane-pipe-%s-sanity",
 		      kmstest_pipe_name(pipe))
-		for_each_connected_output(&data->display, output)
+		for_each_valid_output_on_pipe(&data->display, pipe, output)
 			sanity_test_pipe(data, pipe, output);
 
 	igt_subtest_f("disable-primary-vs-flip-pipe-%s",
 		      kmstest_pipe_name(pipe))
-		for_each_connected_output(&data->display, output)
+		for_each_valid_output_on_pipe(&data->display, pipe, output)
 			pageflip_test_pipe(data, pipe, output);
+
+	igt_subtest_f("cursor-fb-leak-pipe-%s",
+		      kmstest_pipe_name(pipe))
+		for_each_valid_output_on_pipe(&data->display, pipe, output)
+			cursor_leak_test_pipe(data, pipe, output);
+
+	igt_subtest_f("universal-plane-gen9-features-pipe-%s",
+		      kmstest_pipe_name(pipe))
+		for_each_valid_output_on_pipe(&data->display, pipe, output)
+			gen9_test_pipe(data, pipe, output);
 }
 
 static data_t data;
 
 igt_main
 {
-	int num_pipes;
+	enum pipe pipe;
 
 	igt_skip_on_simulation();
 
 	igt_fixture {
-		data.drm_fd = drm_open_any_master();
+		data.drm_fd = drm_open_driver_master(DRIVER_INTEL);
+		data.gen = intel_gen(intel_get_drm_devid(data.drm_fd));
 
 		kmstest_set_vt_graphics_mode();
 
-		igt_require_pipe_crc();
+		igt_require_pipe_crc(data.drm_fd);
 		igt_display_init(&data.display, data.drm_fd);
-
-		igt_require(data.display.has_universal_planes);
 	}
 
-	num_pipes = igt_display_get_n_pipes(&data.display);
-	for (int pipe = 0; pipe < num_pipes; pipe++)
-		run_tests_for_pipe(&data, pipe);
+	for_each_pipe_static(pipe) {
+		igt_subtest_group
+			run_tests_for_pipe(&data, pipe);
+	}
 
 	igt_fixture {
 		igt_display_fini(&data.display);
