@@ -35,11 +35,13 @@
 #include <dirent.h>
 #include <time.h>
 #include <poll.h>
+#include <sched.h>
 
 #include "igt.h"
 #include "igt_core.h"
 #include "igt_perf.h"
 #include "igt_sysfs.h"
+#include "igt_pm.h"
 #include "sw_sync.h"
 
 IGT_TEST_DESCRIPTION("Test the i915 pmu perf interface");
@@ -72,23 +74,47 @@ static int open_group(uint64_t config, int group)
 static void
 init(int gem_fd, const struct intel_execution_engine2 *e, uint8_t sample)
 {
-	int fd;
+	int fd, err = 0;
+	bool exists;
 
-	fd = open_pmu(__I915_PMU_ENGINE(e->class, e->instance, sample));
+	errno = 0;
+	fd = perf_i915_open(__I915_PMU_ENGINE(e->class, e->instance, sample));
+	if (fd < 0)
+		err = errno;
 
-	close(fd);
+	exists = gem_has_engine(gem_fd, e->class, e->instance);
+	if (intel_gen(intel_get_drm_devid(gem_fd)) < 6 &&
+	    sample == I915_SAMPLE_SEMA)
+		exists = false;
+
+	if (exists) {
+		igt_assert_eq(err, 0);
+		igt_assert_fd(fd);
+		close(fd);
+	} else {
+		igt_assert_lt(fd, 0);
+		igt_assert_eq(err, ENODEV);
+	}
 }
 
-static uint64_t pmu_read_single(int fd)
+static uint64_t __pmu_read_single(int fd, uint64_t *ts)
 {
 	uint64_t data[2];
 
 	igt_assert_eq(read(fd, data, sizeof(data)), sizeof(data));
 
+	if (ts)
+		*ts = data[1];
+
 	return data[0];
 }
 
-static void pmu_read_multi(int fd, unsigned int num, uint64_t *val)
+static uint64_t pmu_read_single(int fd)
+{
+	return __pmu_read_single(fd, NULL);
+}
+
+static uint64_t pmu_read_multi(int fd, unsigned int num, uint64_t *val)
 {
 	uint64_t buf[2 + num];
 	unsigned int i;
@@ -97,13 +123,20 @@ static void pmu_read_multi(int fd, unsigned int num, uint64_t *val)
 
 	for (i = 0; i < num; i++)
 		val[i] = buf[2 + i];
+
+	return buf[1];
 }
 
+#define __assert_within_epsilon(x, ref, tol_up, tol_down) \
+	igt_assert_f((double)(x) <= (1.0 + (tol_up)) * (double)(ref) && \
+		     (double)(x) >= (1.0 - (tol_down)) * (double)(ref), \
+		     "'%s' != '%s' (%f not within +%f%%/-%f%% tolerance of %f)\n",\
+		     #x, #ref, (double)(x), \
+		     (tol_up) * 100.0, (tol_down) * 100.0, \
+		     (double)(ref))
+
 #define assert_within_epsilon(x, ref, tolerance) \
-	igt_assert_f((double)(x) <= (1.0 + (tolerance)) * (double)(ref) && \
-		     (double)(x) >= (1.0 - (tolerance)) * (double)(ref), \
-		     "'%s' != '%s' (%f not within %f%% tolerance of %f)\n",\
-		     #x, #ref, (double)(x), (tolerance) * 100.0, (double)(ref))
+	__assert_within_epsilon(x, ref, tolerance, tolerance)
 
 /*
  * Helper for cases where we assert on time spent sleeping (directly or
@@ -130,8 +163,29 @@ static unsigned int e2ring(int gem_fd, const struct intel_execution_engine2 *e)
 	return gem_class_instance_to_eb_flags(gem_fd, e->class, e->instance);
 }
 
+#define TEST_BUSY (1)
+#define FLAG_SYNC (2)
+#define TEST_TRAILING_IDLE (4)
+#define TEST_RUNTIME_PM (8)
+#define FLAG_LONG (16)
+#define FLAG_HANG (32)
+
+static void end_spin(int fd, igt_spin_t *spin, unsigned int flags)
+{
+	if (!spin)
+		return;
+
+	igt_spin_batch_end(spin);
+
+	if (flags & FLAG_SYNC)
+		gem_sync(fd, spin->handle);
+
+	if (flags & TEST_TRAILING_IDLE)
+		usleep(batch_duration_ns / 5000);
+}
+
 static void
-single(int gem_fd, const struct intel_execution_engine2 *e, bool busy)
+single(int gem_fd, const struct intel_execution_engine2 *e, unsigned int flags)
 {
 	unsigned long slept;
 	igt_spin_t *spin;
@@ -140,20 +194,46 @@ single(int gem_fd, const struct intel_execution_engine2 *e, bool busy)
 
 	fd = open_pmu(I915_PMU_ENGINE_BUSY(e->class, e->instance));
 
-	if (busy)
+	if (flags & TEST_BUSY)
 		spin = igt_spin_batch_new(gem_fd, 0, e2ring(gem_fd, e), 0);
 	else
 		spin = NULL;
 
-	slept = measured_usleep(batch_duration_ns / 1000);
-	igt_spin_batch_end(spin);
-
 	val = pmu_read_single(fd);
+	slept = measured_usleep(batch_duration_ns / 1000);
+	if (flags & TEST_TRAILING_IDLE)
+		end_spin(gem_fd, spin, flags);
+	val = pmu_read_single(fd) - val;
+
+	if (flags & FLAG_HANG)
+		igt_force_gpu_reset(gem_fd);
+	else
+		end_spin(gem_fd, spin, FLAG_SYNC);
+
+	assert_within_epsilon(val, flags & TEST_BUSY ? slept : 0.f, tolerance);
+
+	/* Check for idle after hang. */
+	if (flags & FLAG_HANG) {
+		/* Sleep for a bit for reset unwind to settle. */
+		usleep(500e3);
+		/*
+		 * Ensure batch was executing before reset, meaning it must be
+		 * idle by now. Unless it did not even manage to start before we
+		 * triggered the reset, in which case the idleness check below
+		 * might fail. The latter is very unlikely since there are two
+		 * sleeps during which it had an opportunity to start.
+		 */
+		igt_assert(!gem_bo_busy(gem_fd, spin->handle));
+		val = pmu_read_single(fd);
+		slept = measured_usleep(batch_duration_ns / 1000);
+		val = pmu_read_single(fd) - val;
+
+		assert_within_epsilon(val, 0, tolerance);
+	}
 
 	igt_spin_batch_free(gem_fd, spin);
 	close(fd);
 
-	assert_within_epsilon(val, busy ? slept : 0.f, tolerance);
 	gem_quiescent_gpu(gem_fd);
 }
 
@@ -161,8 +241,8 @@ static void
 busy_start(int gem_fd, const struct intel_execution_engine2 *e)
 {
 	unsigned long slept;
+	uint64_t val, ts[2];
 	igt_spin_t *spin;
-	uint64_t val;
 	int fd;
 
 	/*
@@ -181,13 +261,15 @@ busy_start(int gem_fd, const struct intel_execution_engine2 *e)
 
 	fd = open_pmu(I915_PMU_ENGINE_BUSY(e->class, e->instance));
 
+	val = __pmu_read_single(fd, &ts[0]);
 	slept = measured_usleep(batch_duration_ns / 1000);
-	val = pmu_read_single(fd);
+	val = __pmu_read_single(fd, &ts[1]) - val;
+	igt_debug("slept=%lu perf=%"PRIu64"\n", slept, ts[1] - ts[0]);
 
 	igt_spin_batch_free(gem_fd, spin);
 	close(fd);
 
-	assert_within_epsilon(val, slept, tolerance);
+	assert_within_epsilon(val, ts[1] - ts[0], tolerance);
 	gem_quiescent_gpu(gem_fd);
 }
 
@@ -200,8 +282,8 @@ static void
 busy_double_start(int gem_fd, const struct intel_execution_engine2 *e)
 {
 	unsigned long slept;
+	uint64_t val, val2, ts[2];
 	igt_spin_t *spin[2];
-	uint64_t val, val2;
 	uint32_t ctx;
 	int fd;
 
@@ -228,8 +310,10 @@ busy_double_start(int gem_fd, const struct intel_execution_engine2 *e)
 	 */
 	fd = open_pmu(I915_PMU_ENGINE_BUSY(e->class, e->instance));
 
+	val = __pmu_read_single(fd, &ts[0]);
 	slept = measured_usleep(batch_duration_ns / 1000);
-	val = pmu_read_single(fd);
+	val = __pmu_read_single(fd, &ts[1]) - val;
+	igt_debug("slept=%lu perf=%"PRIu64"\n", slept, ts[1] - ts[0]);
 
 	igt_spin_batch_end(spin[0]);
 	igt_spin_batch_end(spin[1]);
@@ -241,7 +325,7 @@ busy_double_start(int gem_fd, const struct intel_execution_engine2 *e)
 	usleep(batch_duration_ns / 1000);
 	val2 = pmu_read_single(fd) - val2;
 
-	igt_info("busy=%lu idle=%lu\n", val, val2);
+	igt_info("busy=%"PRIu64" idle=%"PRIu64"\n", val, val2);
 
 	igt_spin_batch_free(gem_fd, spin[0]);
 	igt_spin_batch_free(gem_fd, spin[1]);
@@ -250,13 +334,13 @@ busy_double_start(int gem_fd, const struct intel_execution_engine2 *e)
 
 	gem_context_destroy(gem_fd, ctx);
 
-	assert_within_epsilon(val, slept, tolerance);
+	assert_within_epsilon(val, ts[1] - ts[0], tolerance);
 	igt_assert_eq(val2, 0);
 
 	gem_quiescent_gpu(gem_fd);
 }
 
-static void log_busy(int fd, unsigned int num_engines, uint64_t *val)
+static void log_busy(unsigned int num_engines, uint64_t *val)
 {
 	char buf[1024];
 	int rem = sizeof(buf);
@@ -277,14 +361,15 @@ static void log_busy(int fd, unsigned int num_engines, uint64_t *val)
 
 static void
 busy_check_all(int gem_fd, const struct intel_execution_engine2 *e,
-	       const unsigned int num_engines)
+	       const unsigned int num_engines, unsigned int flags)
 {
 	const struct intel_execution_engine2 *e_;
+	uint64_t tval[2][num_engines];
+	unsigned int busy_idx = 0, i;
 	uint64_t val[num_engines];
 	int fd[num_engines];
 	unsigned long slept;
 	igt_spin_t *spin;
-	unsigned int busy_idx, i;
 
 	i = 0;
 	fd[0] = -1;
@@ -302,15 +387,21 @@ busy_check_all(int gem_fd, const struct intel_execution_engine2 *e,
 	igt_assert_eq(i, num_engines);
 
 	spin = igt_spin_batch_new(gem_fd, 0, e2ring(gem_fd, e), 0);
+	pmu_read_multi(fd[0], num_engines, tval[0]);
 	slept = measured_usleep(batch_duration_ns / 1000);
-	igt_spin_batch_end(spin);
-
-	pmu_read_multi(fd[0], num_engines, val);
-	log_busy(fd[0], num_engines, val);
+	if (flags & TEST_TRAILING_IDLE)
+		end_spin(gem_fd, spin, flags);
+	pmu_read_multi(fd[0], num_engines, tval[1]);
 
+	end_spin(gem_fd, spin, FLAG_SYNC);
 	igt_spin_batch_free(gem_fd, spin);
 	close(fd[0]);
 
+	for (i = 0; i < num_engines; i++)
+		val[i] = tval[1][i] - tval[0][i];
+
+	log_busy(num_engines, val);
+
 	assert_within_epsilon(val[busy_idx], slept, tolerance);
 	for (i = 0; i < num_engines; i++) {
 		if (i == busy_idx)
@@ -321,18 +412,32 @@ busy_check_all(int gem_fd, const struct intel_execution_engine2 *e,
 }
 
 static void
-most_busy_check_all(int gem_fd, const struct intel_execution_engine2 *e,
-		    const unsigned int num_engines)
+__submit_spin_batch(int gem_fd,
+		    struct drm_i915_gem_exec_object2 *obj,
+		    const struct intel_execution_engine2 *e)
 {
+	struct drm_i915_gem_execbuffer2 eb = {
+		.buffer_count = 1,
+		.buffers_ptr = to_user_pointer(obj),
+		.flags = e2ring(gem_fd, e),
+	};
+
+	gem_execbuf(gem_fd, &eb);
+}
+
+static void
+most_busy_check_all(int gem_fd, const struct intel_execution_engine2 *e,
+		    const unsigned int num_engines, unsigned int flags)
+{
+	struct drm_i915_gem_exec_object2 obj = {};
 	const struct intel_execution_engine2 *e_;
+	uint64_t tval[2][num_engines];
 	uint64_t val[num_engines];
 	int fd[num_engines];
 	unsigned long slept;
 	igt_spin_t *spin = NULL;
 	unsigned int idle_idx, i;
 
-	gem_require_engine(gem_fd, e->class, e->instance);
-
 	i = 0;
 	for_each_engine_class_instance(fd, e_) {
 		if (!gem_has_engine(gem_fd, e_->class, e_->instance))
@@ -341,18 +446,11 @@ most_busy_check_all(int gem_fd, const struct intel_execution_engine2 *e,
 		if (e == e_) {
 			idle_idx = i;
 		} else if (spin) {
-			struct drm_i915_gem_exec_object2 obj = {
-				.handle = spin->handle
-			};
-			struct drm_i915_gem_execbuffer2 eb = {
-				.buffer_count = 1,
-				.buffers_ptr = to_user_pointer(&obj),
-				.flags = e2ring(gem_fd, e_),
-			};
-			gem_execbuf(gem_fd, &eb);
+			__submit_spin_batch(gem_fd, &obj, e_);
 		} else {
 			spin = igt_spin_batch_new(gem_fd, 0,
 						  e2ring(gem_fd, e_), 0);
+			obj.handle = spin->handle;
 		}
 
 		val[i++] = I915_PMU_ENGINE_BUSY(e_->class, e_->instance);
@@ -363,15 +461,21 @@ most_busy_check_all(int gem_fd, const struct intel_execution_engine2 *e,
 	for (i = 0; i < num_engines; i++)
 		fd[i] = open_group(val[i], fd[0]);
 
+	pmu_read_multi(fd[0], num_engines, tval[0]);
 	slept = measured_usleep(batch_duration_ns / 1000);
-	igt_spin_batch_end(spin);
-
-	pmu_read_multi(fd[0], num_engines, val);
-	log_busy(fd[0], num_engines, val);
+	if (flags & TEST_TRAILING_IDLE)
+		end_spin(gem_fd, spin, flags);
+	pmu_read_multi(fd[0], num_engines, tval[1]);
 
+	end_spin(gem_fd, spin, FLAG_SYNC);
 	igt_spin_batch_free(gem_fd, spin);
 	close(fd[0]);
 
+	for (i = 0; i < num_engines; i++)
+		val[i] = tval[1][i] - tval[0][i];
+
+	log_busy(num_engines, val);
+
 	for (i = 0; i < num_engines; i++) {
 		if (i == idle_idx)
 			assert_within_epsilon(val[i], 0.0f, tolerance);
@@ -382,9 +486,12 @@ most_busy_check_all(int gem_fd, const struct intel_execution_engine2 *e,
 }
 
 static void
-all_busy_check_all(int gem_fd, const unsigned int num_engines)
+all_busy_check_all(int gem_fd, const unsigned int num_engines,
+		   unsigned int flags)
 {
+	struct drm_i915_gem_exec_object2 obj = {};
 	const struct intel_execution_engine2 *e;
+	uint64_t tval[2][num_engines];
 	uint64_t val[num_engines];
 	int fd[num_engines];
 	unsigned long slept;
@@ -397,18 +504,11 @@ all_busy_check_all(int gem_fd, const unsigned int num_engines)
 			continue;
 
 		if (spin) {
-			struct drm_i915_gem_exec_object2 obj = {
-				.handle = spin->handle
-			};
-			struct drm_i915_gem_execbuffer2 eb = {
-				.buffer_count = 1,
-				.buffers_ptr = to_user_pointer(&obj),
-				.flags = e2ring(gem_fd, e),
-			};
-			gem_execbuf(gem_fd, &eb);
+			__submit_spin_batch(gem_fd, &obj, e);
 		} else {
 			spin = igt_spin_batch_new(gem_fd, 0,
 						  e2ring(gem_fd, e), 0);
+			obj.handle = spin->handle;
 		}
 
 		val[i++] = I915_PMU_ENGINE_BUSY(e->class, e->instance);
@@ -419,48 +519,58 @@ all_busy_check_all(int gem_fd, const unsigned int num_engines)
 	for (i = 0; i < num_engines; i++)
 		fd[i] = open_group(val[i], fd[0]);
 
+	pmu_read_multi(fd[0], num_engines, tval[0]);
 	slept = measured_usleep(batch_duration_ns / 1000);
-	igt_spin_batch_end(spin);
-
-	pmu_read_multi(fd[0], num_engines, val);
-	log_busy(fd[0], num_engines, val);
+	if (flags & TEST_TRAILING_IDLE)
+		end_spin(gem_fd, spin, flags);
+	pmu_read_multi(fd[0], num_engines, tval[1]);
 
+	end_spin(gem_fd, spin, FLAG_SYNC);
 	igt_spin_batch_free(gem_fd, spin);
 	close(fd[0]);
 
+	for (i = 0; i < num_engines; i++)
+		val[i] = tval[1][i] - tval[0][i];
+
+	log_busy(num_engines, val);
+
 	for (i = 0; i < num_engines; i++)
 		assert_within_epsilon(val[i], slept, tolerance);
 	gem_quiescent_gpu(gem_fd);
 }
 
 static void
-no_sema(int gem_fd, const struct intel_execution_engine2 *e, bool busy)
+no_sema(int gem_fd, const struct intel_execution_engine2 *e, unsigned int flags)
 {
 	igt_spin_t *spin;
-	uint64_t val[2];
+	uint64_t val[2][2];
 	int fd;
 
 	fd = open_group(I915_PMU_ENGINE_SEMA(e->class, e->instance), -1);
 	open_group(I915_PMU_ENGINE_WAIT(e->class, e->instance), fd);
 
-	if (busy) {
+	if (flags & TEST_BUSY)
 		spin = igt_spin_batch_new(gem_fd, 0, e2ring(gem_fd, e), 0);
-		igt_spin_batch_set_timeout(spin, batch_duration_ns);
-	} else {
-		usleep(batch_duration_ns / 1000);
-	}
+	else
+		spin = NULL;
 
-	if (busy)
-		gem_sync(gem_fd, spin->handle);
+	pmu_read_multi(fd, 2, val[0]);
+	measured_usleep(batch_duration_ns / 1000);
+	if (flags & TEST_TRAILING_IDLE)
+		end_spin(gem_fd, spin, flags);
+	pmu_read_multi(fd, 2, val[1]);
 
-	pmu_read_multi(fd, 2, val);
+	val[0][0] = val[1][0] - val[0][0];
+	val[0][1] = val[1][1] - val[0][1];
 
-	if (busy)
+	if (spin) {
+		end_spin(gem_fd, spin, FLAG_SYNC);
 		igt_spin_batch_free(gem_fd, spin);
+	}
 	close(fd);
 
-	assert_within_epsilon(val[0], 0.0f, tolerance);
-	assert_within_epsilon(val[1], 0.0f, tolerance);
+	assert_within_epsilon(val[0][0], 0.0f, tolerance);
+	assert_within_epsilon(val[0][1], 0.0f, tolerance);
 }
 
 #define MI_INSTR(opcode, flags) (((opcode) << 23) | (flags))
@@ -469,7 +579,8 @@ no_sema(int gem_fd, const struct intel_execution_engine2 *e, bool busy)
 #define   MI_SEMAPHORE_SAD_GTE_SDD	(1<<12)
 
 static void
-sema_wait(int gem_fd, const struct intel_execution_engine2 *e)
+sema_wait(int gem_fd, const struct intel_execution_engine2 *e,
+	  unsigned int flags)
 {
 	struct drm_i915_gem_relocation_entry reloc[2] = {};
 	struct drm_i915_gem_exec_object2 obj[2] = {};
@@ -478,7 +589,7 @@ sema_wait(int gem_fd, const struct intel_execution_engine2 *e)
 	unsigned long slept;
 	uint32_t *obj_ptr;
 	uint32_t batch[16];
-	uint64_t val[2];
+	uint64_t val[2], ts[2];
 	int fd;
 
 	igt_require(intel_gen(intel_get_drm_devid(gem_fd)) >= 8);
@@ -536,17 +647,25 @@ sema_wait(int gem_fd, const struct intel_execution_engine2 *e)
 
 	fd = open_pmu(I915_PMU_ENGINE_SEMA(e->class, e->instance));
 
+	val[0] = pmu_read_single(fd);
+
 	gem_execbuf(gem_fd, &eb);
 	do { /* wait for the batch to start executing */
 		usleep(5e3);
 	} while (!obj_ptr[1]);
-	usleep(5e3); /* wait for the register sampling */
 
-	val[0] = pmu_read_single(fd);
+	igt_assert_f(igt_wait(pmu_read_single(fd) != val[0], 10, 1),
+		     "sampling failed to start withing 10ms");
+
+	val[0] = __pmu_read_single(fd, &ts[0]);
 	slept = measured_usleep(batch_duration_ns / 1000);
-	val[1] = pmu_read_single(fd);
-	igt_debug("slept %.3fms, sampled %.3fms\n",
-		  slept*1e-6, (val[1] - val[0])*1e-6);
+	if (flags & TEST_TRAILING_IDLE)
+		obj_ptr[0] = 1;
+	val[1] = __pmu_read_single(fd, &ts[1]);
+	igt_debug("slept %.3fms (perf %.3fms), sampled %.3fms\n",
+		  slept * 1e-6,
+		  (ts[1] - ts[0]) * 1e-6,
+		  (val[1] - val[0]) * 1e-6);
 
 	obj_ptr[0] = 1;
 	gem_sync(gem_fd, bb_handle);
@@ -749,11 +868,13 @@ static void
 multi_client(int gem_fd, const struct intel_execution_engine2 *e)
 {
 	uint64_t config = I915_PMU_ENGINE_BUSY(e->class, e->instance);
-	unsigned int slept;
+	unsigned long slept[2];
+	uint64_t val[2], ts[2], perf_slept[2];
 	igt_spin_t *spin;
-	uint64_t val[2];
 	int fd[2];
 
+	gem_quiescent_gpu(gem_fd);
+
 	fd[0] = open_pmu(config);
 
 	/*
@@ -764,21 +885,26 @@ multi_client(int gem_fd, const struct intel_execution_engine2 *e)
 	fd[1] = open_pmu(config);
 
 	spin = igt_spin_batch_new(gem_fd, 0, e2ring(gem_fd, e), 0);
-	igt_spin_batch_set_timeout(spin, 2 * batch_duration_ns);
 
-	slept = measured_usleep(batch_duration_ns / 1000);
-	val[1] = pmu_read_single(fd[1]);
+	val[0] = val[1] = __pmu_read_single(fd[0], &ts[0]);
+	slept[1] = measured_usleep(batch_duration_ns / 1000);
+	val[1] = __pmu_read_single(fd[1], &ts[1]) - val[1];
+	perf_slept[1] = ts[1] - ts[0];
+	igt_debug("slept=%lu perf=%"PRIu64"\n", slept[1], perf_slept[1]);
 	close(fd[1]);
 
+	slept[0] = measured_usleep(batch_duration_ns / 1000) + slept[1];
+	val[0] = __pmu_read_single(fd[0], &ts[1]) - val[0];
+	perf_slept[0] = ts[1] - ts[0];
+	igt_debug("slept=%lu perf=%"PRIu64"\n", slept[0], perf_slept[0]);
+
+	igt_spin_batch_end(spin);
 	gem_sync(gem_fd, spin->handle);
-
-	val[0] = pmu_read_single(fd[0]);
-
 	igt_spin_batch_free(gem_fd, spin);
 	close(fd[0]);
 
-	assert_within_epsilon(val[0], 2 * batch_duration_ns, tolerance);
-	assert_within_epsilon(val[1], slept, tolerance);
+	assert_within_epsilon(val[0], perf_slept[0], tolerance);
+	assert_within_epsilon(val[1], perf_slept[1], tolerance);
 }
 
 /**
@@ -858,19 +984,30 @@ static bool cpu0_hotplug_support(void)
 
 static void cpu_hotplug(int gem_fd)
 {
-	struct timespec start = { };
-	igt_spin_t *spin;
-	uint64_t val, ref;
-	int fd;
+	igt_spin_t *spin[2];
+	uint64_t ts[2];
+	uint64_t val;
+	int link[2];
+	int fd, ret;
+	int cur = 0;
+	char buf;
 
+	igt_skip_on(IS_BROXTON(intel_get_drm_devid(gem_fd)));
 	igt_require(cpu0_hotplug_support());
 
-	fd = perf_i915_open(I915_PMU_ENGINE_BUSY(I915_ENGINE_CLASS_RENDER, 0));
-	igt_assert(fd >= 0);
+	fd = open_pmu(I915_PMU_ENGINE_BUSY(I915_ENGINE_CLASS_RENDER, 0));
 
-	spin = igt_spin_batch_new(gem_fd, 0, I915_EXEC_RENDER, 0);
+	/*
+	 * Create two spinners so test can ensure shorter gaps in engine
+	 * busyness as it is terminating one and re-starting the other.
+	 */
+	spin[0] = igt_spin_batch_new(gem_fd, 0, I915_EXEC_RENDER, 0);
+	spin[1] = __igt_spin_batch_new(gem_fd, 0, I915_EXEC_RENDER, 0);
 
-	igt_nsec_elapsed(&start);
+	val = __pmu_read_single(fd, &ts[0]);
+
+	ret = pipe2(link, O_NONBLOCK);
+	igt_assert_eq(ret, 0);
 
 	/*
 	 * Toggle online status of all the CPUs in a child process and ensure
@@ -879,40 +1016,94 @@ static void cpu_hotplug(int gem_fd)
 	igt_fork(child, 1) {
 		int cpu = 0;
 
+		close(link[0]);
+
 		for (;;) {
 			char name[128];
 			int cpufd;
 
-			sprintf(name, "/sys/devices/system/cpu/cpu%d/online",
-				cpu);
+			igt_assert_lt(snprintf(name, sizeof(name),
+					       "/sys/devices/system/cpu/cpu%d/online",
+					       cpu), sizeof(name));
 			cpufd = open(name, O_WRONLY);
 			if (cpufd == -1) {
 				igt_assert(cpu > 0);
+				/*
+				 * Signal parent that we cycled through all
+				 * CPUs and we are done.
+				 */
+				igt_assert_eq(write(link[1], "*", 1), 1);
+				break;
+			}
+
+			/* Offline followed by online a CPU. */
+
+			ret = write(cpufd, "0", 2);
+			if (ret < 0) {
+				/*
+				 * If we failed to offline a CPU we don't want
+				 * to proceed.
+				 */
+				igt_warn("Failed to offline cpu%u! (%d)\n",
+					 cpu, errno);
+				igt_assert_eq(write(link[1], "s", 1), 1);
 				break;
 			}
-			igt_assert_eq(write(cpufd, "0", 2), 2);
 
 			usleep(1e6);
 
-			igt_assert_eq(write(cpufd, "1", 2), 2);
+			ret = write(cpufd, "1", 2);
+			if (ret < 0) {
+				/*
+				 * Failed to bring a CPU back online is fatal
+				 * for the sanity of a test run so stop further
+				 * testing.
+				 */
+				igt_warn("Failed to online cpu%u! (%d)\n",
+					 cpu, errno);
+				igt_fatal_error();
+			}
 
 			close(cpufd);
 			cpu++;
 		}
 	}
 
+	close(link[1]);
+
+	/*
+	 * Very long batches can be declared as GPU hangs so emit shorter ones
+	 * until the CPU core shuffler finishes one loop.
+	 */
+	for (;;) {
+		usleep(500e3);
+		end_spin(gem_fd, spin[cur], 0);
+
+		/* Check if the child is signaling completion. */
+		ret = read(link[0], &buf, 1);
+		if ( ret == 1 || (ret < 0 && errno != EAGAIN))
+			break;
+
+		igt_spin_batch_free(gem_fd, spin[cur]);
+		spin[cur] = __igt_spin_batch_new(gem_fd, 0, I915_EXEC_RENDER,
+						 0);
+		cur ^= 1;
+	}
+
+	val = __pmu_read_single(fd, &ts[1]) - val;
+
+	end_spin(gem_fd, spin[0], FLAG_SYNC);
+	end_spin(gem_fd, spin[1], FLAG_SYNC);
+	igt_spin_batch_free(gem_fd, spin[0]);
+	igt_spin_batch_free(gem_fd, spin[1]);
 	igt_waitchildren();
-
-	igt_spin_batch_end(spin);
-	gem_sync(gem_fd, spin->handle);
-
-	ref = igt_nsec_elapsed(&start);
-	val = pmu_read_single(fd);
-
-	igt_spin_batch_free(gem_fd, spin);
 	close(fd);
+	close(link[0]);
 
-	assert_within_epsilon(val, ref, tolerance);
+	/* Skip if child signals a problem with offlining a CPU. */
+	igt_skip_on(buf == 's');
+
+	assert_within_epsilon(val, ts[1] - ts[0], tolerance);
 }
 
 static void
@@ -1026,9 +1217,8 @@ static void
 test_frequency(int gem_fd)
 {
 	uint32_t min_freq, max_freq, boost_freq;
-	uint64_t val[2], start[2];
+	uint64_t val[2], start[2], slept;
 	double min[2], max[2];
-	unsigned long slept;
 	igt_spin_t *spin;
 	int fd, sysfs;
 
@@ -1059,11 +1249,11 @@ test_frequency(int gem_fd)
 
 	gem_quiescent_gpu(gem_fd); /* Idle to be sure the change takes effect */
 	spin = igt_spin_batch_new(gem_fd, 0, I915_EXEC_RENDER, 0);
-	pmu_read_multi(fd, 2, start);
 
-	slept = measured_usleep(batch_duration_ns / 1000);
+	slept = pmu_read_multi(fd, 2, start);
+	measured_usleep(batch_duration_ns / 1000);
+	slept = pmu_read_multi(fd, 2, val) - slept;
 
-	pmu_read_multi(fd, 2, val);
 	min[0] = 1e9*(val[0] - start[0]) / slept;
 	min[1] = 1e9*(val[1] - start[1]) / slept;
 
@@ -1085,11 +1275,11 @@ test_frequency(int gem_fd)
 
 	gem_quiescent_gpu(gem_fd);
 	spin = igt_spin_batch_new(gem_fd, 0, I915_EXEC_RENDER, 0);
-	pmu_read_multi(fd, 2, start);
 
-	slept = measured_usleep(batch_duration_ns / 1000);
+	slept = pmu_read_multi(fd, 2, start);
+	measured_usleep(batch_duration_ns / 1000);
+	slept = pmu_read_multi(fd, 2, val) - slept;
 
-	pmu_read_multi(fd, 2, val);
 	max[0] = 1e9*(val[0] - start[0]) / slept;
 	max[1] = 1e9*(val[1] - start[1]) / slept;
 
@@ -1109,8 +1299,13 @@ test_frequency(int gem_fd)
 		 min[0], min[1]);
 	igt_info("Max frequency: requested %.1f, actual %.1f\n",
 		 max[0], max[1]);
+
 	assert_within_epsilon(min[0], min_freq, tolerance);
-	assert_within_epsilon(max[0], max_freq, tolerance);
+	/*
+	 * On thermally throttled devices we cannot be sure maximum frequency
+	 * can be reached so use larger tolerance downards.
+	 */
+	__assert_within_epsilon(max[0], max_freq, tolerance, 0.15f);
 }
 
 static bool wait_for_rc6(int fd)
@@ -1135,24 +1330,51 @@ static bool wait_for_rc6(int fd)
 }
 
 static void
-test_rc6(int gem_fd)
+test_rc6(int gem_fd, unsigned int flags)
 {
 	int64_t duration_ns = 2e9;
-	uint64_t idle, busy, prev;
-	unsigned int slept;
+	uint64_t idle, busy, prev, ts[2];
+	unsigned long slept;
 	int fd, fw;
 
+	gem_quiescent_gpu(gem_fd);
+
 	fd = open_pmu(I915_PMU_RC6_RESIDENCY);
 
-	gem_quiescent_gpu(gem_fd);
+	if (flags & TEST_RUNTIME_PM) {
+		drmModeRes *res;
+
+		res = drmModeGetResources(gem_fd);
+		igt_assert(res);
+
+		/* force all connectors off */
+		kmstest_set_vt_graphics_mode();
+		kmstest_unset_all_crtcs(gem_fd, res);
+		drmModeFreeResources(res);
+
+		igt_require(igt_setup_runtime_pm());
+		igt_require(igt_wait_for_pm_status(IGT_RUNTIME_PM_STATUS_SUSPENDED));
+
+		/*
+		 * Sleep for a bit to see if once woken up estimated RC6 hasn't
+		 * drifted to far in advance of real RC6.
+		 */
+		if (flags & FLAG_LONG) {
+			pmu_read_single(fd);
+			sleep(5);
+			pmu_read_single(fd);
+		}
+	}
+
 	igt_require(wait_for_rc6(fd));
 
-	/* Go idle and check full RC6. */
-	prev = pmu_read_single(fd);
+	/* While idle check full RC6. */
+	prev = __pmu_read_single(fd, &ts[0]);
 	slept = measured_usleep(duration_ns / 1000);
-	idle = pmu_read_single(fd);
+	idle = __pmu_read_single(fd, &ts[1]);
+	igt_debug("slept=%lu perf=%"PRIu64"\n", slept, ts[1] - ts[0]);
 
-	assert_within_epsilon(idle - prev, slept, tolerance);
+	assert_within_epsilon(idle - prev, ts[1] - ts[0], tolerance);
 
 	/* Wake up device and check no RC6. */
 	fw = igt_open_forcewake_handle(gem_fd);
@@ -1169,6 +1391,209 @@ test_rc6(int gem_fd)
 	assert_within_epsilon(busy - prev, 0.0, tolerance);
 }
 
+static void
+test_enable_race(int gem_fd, const struct intel_execution_engine2 *e)
+{
+	uint64_t config = I915_PMU_ENGINE_BUSY(e->class, e->instance);
+	struct igt_helper_process engine_load = { };
+	const uint32_t bbend = MI_BATCH_BUFFER_END;
+	struct drm_i915_gem_exec_object2 obj = { };
+	struct drm_i915_gem_execbuffer2 eb = { };
+	int fd;
+
+	igt_require(gem_has_execlists(gem_fd));
+	igt_require(gem_has_engine(gem_fd, e->class, e->instance));
+
+	obj.handle = gem_create(gem_fd, 4096);
+	gem_write(gem_fd, obj.handle, 0, &bbend, sizeof(bbend));
+
+	eb.buffer_count = 1;
+	eb.buffers_ptr = to_user_pointer(&obj);
+	eb.flags = e2ring(gem_fd, e);
+
+	/*
+	 * This test is probabilistic so run in a few times to increase the
+	 * chance of hitting the race.
+	 */
+	igt_until_timeout(10) {
+		/*
+		 * Defeat the busy stats delayed disable, we need to guarantee
+		 * we are the first PMU user.
+		 */
+		gem_quiescent_gpu(gem_fd);
+		sleep(2);
+
+		/* Apply interrupt-heavy load on the engine. */
+		igt_fork_helper(&engine_load) {
+			for (;;)
+				gem_execbuf(gem_fd, &eb);
+		}
+
+		/* Wait a bit to allow engine load to start. */
+		usleep(500e3);
+
+		/* Enable the PMU. */
+		fd = open_pmu(config);
+
+		/* Stop load and close the PMU. */
+		igt_stop_helper(&engine_load);
+		close(fd);
+	}
+
+	/* Cleanup. */
+	gem_close(gem_fd, obj.handle);
+	gem_quiescent_gpu(gem_fd);
+}
+
+static double __error(double val, double ref)
+{
+	igt_assert(ref > 1e-5 /* smallval */);
+	return (100.0 * val / ref) - 100.0;
+}
+
+static void __rearm_spin_batch(igt_spin_t *spin)
+{
+	const uint32_t mi_arb_chk = 0x5 << 23;
+
+       *spin->batch = mi_arb_chk;
+       __sync_synchronize();
+}
+
+#define div_round_up(a, b) (((a) + (b) - 1) / (b))
+
+static void
+accuracy(int gem_fd, const struct intel_execution_engine2 *e,
+	 unsigned long target_busy_pct)
+{
+	const unsigned int min_test_loops = 7;
+	const unsigned long min_test_us = 1e6;
+	unsigned long busy_us = 2500;
+	unsigned long idle_us = 100 * (busy_us - target_busy_pct *
+				busy_us / 100) / target_busy_pct;
+	unsigned long pwm_calibration_us;
+	unsigned long test_us;
+	double busy_r, expected;
+	uint64_t val[2];
+	uint64_t ts[2];
+	int link[2];
+	int fd;
+
+	/* Sampling platforms cannot reach the high accuracy criteria. */
+	igt_require(gem_has_execlists(gem_fd));
+
+	while (idle_us < 2500) {
+		busy_us *= 2;
+		idle_us *= 2;
+	}
+
+	pwm_calibration_us = min_test_loops * (busy_us + idle_us);
+	while (pwm_calibration_us < min_test_us)
+		pwm_calibration_us += busy_us + idle_us;
+	test_us = min_test_loops * (idle_us + busy_us);
+	while (test_us < min_test_us)
+		test_us += busy_us + idle_us;
+
+	igt_info("calibration=%luus, test=%luus; ratio=%.2f%% (%luus/%luus)\n",
+		 pwm_calibration_us, test_us,
+		 (double)busy_us / (busy_us + idle_us) * 100.0,
+		 busy_us, idle_us);
+
+	assert_within_epsilon((double)busy_us / (busy_us + idle_us),
+				(double)target_busy_pct / 100.0, tolerance);
+
+	igt_assert(pipe(link) == 0);
+
+	/* Emit PWM pattern on the engine from a child. */
+	igt_fork(child, 1) {
+		struct sched_param rt = { .sched_priority = 99 };
+		const unsigned long timeout[] = {
+			pwm_calibration_us * 1000, test_us * 2 * 1000
+		};
+		struct drm_i915_gem_exec_object2 obj = {};
+		uint64_t total_busy_ns = 0, total_idle_ns = 0;
+		igt_spin_t *spin;
+		int ret;
+
+		/* We need the best sleep accuracy we can get. */
+		ret = sched_setscheduler(0,
+					 SCHED_FIFO | SCHED_RESET_ON_FORK,
+					 &rt);
+		if (ret)
+			igt_warn("Failed to set scheduling policy!\n");
+
+		/* Allocate our spin batch and idle it. */
+		spin = igt_spin_batch_new(gem_fd, 0, e2ring(gem_fd, e), 0);
+		obj.handle = spin->handle;
+		__submit_spin_batch(gem_fd, &obj, e); /* record its location */
+		igt_spin_batch_end(spin);
+		gem_sync(gem_fd, obj.handle);
+		obj.flags |= EXEC_OBJECT_PINNED;
+
+		/* 1st pass is calibration, second pass is the test. */
+		for (int pass = 0; pass < ARRAY_SIZE(timeout); pass++) {
+			uint64_t busy_ns = -total_busy_ns;
+			uint64_t idle_ns = -total_idle_ns;
+			struct timespec test_start = { };
+
+			igt_nsec_elapsed(&test_start);
+			do {
+				struct timespec t_busy = { };
+				unsigned int target_idle_us;
+
+				igt_nsec_elapsed(&t_busy);
+
+				/* Restart the spinbatch. */
+				__rearm_spin_batch(spin);
+				__submit_spin_batch(gem_fd, &obj, e);
+				measured_usleep(busy_us);
+				igt_spin_batch_end(spin);
+				gem_sync(gem_fd, obj.handle);
+
+				total_busy_ns += igt_nsec_elapsed(&t_busy);
+
+				target_idle_us =
+					(100 * total_busy_ns / target_busy_pct - (total_busy_ns + total_idle_ns)) / 1000;
+				total_idle_ns += measured_usleep(target_idle_us);
+			} while (igt_nsec_elapsed(&test_start) < timeout[pass]);
+
+			busy_ns += total_busy_ns;
+			idle_ns += total_idle_ns;
+
+			expected = (double)busy_ns / (busy_ns + idle_ns);
+			igt_info("%u: busy %"PRIu64"us, idle %"PRIu64"us: %.2f%% (target: %lu%%)\n",
+				 pass, busy_ns / 1000, idle_ns / 1000,
+				 100 * expected, target_busy_pct);
+			write(link[1], &expected, sizeof(expected));
+		}
+
+		igt_spin_batch_free(gem_fd, spin);
+	}
+
+	/* Let the child run. */
+	read(link[0], &expected, sizeof(expected));
+	assert_within_epsilon(expected, target_busy_pct/100., 0.05);
+
+	/* Collect engine busyness for an interesting part of child runtime. */
+	fd = open_pmu(I915_PMU_ENGINE_BUSY(e->class, e->instance));
+	val[0] = __pmu_read_single(fd, &ts[0]);
+	read(link[0], &expected, sizeof(expected));
+	val[1] = __pmu_read_single(fd, &ts[1]);
+	close(fd);
+
+	close(link[1]);
+	close(link[0]);
+
+	igt_waitchildren();
+
+	busy_r = (double)(val[1] - val[0]) / (ts[1] - ts[0]);
+
+	igt_info("error=%.2f%% (%.2f%% vs %.2f%%)\n",
+		 __error(busy_r, expected), 100 * busy_r, 100 * expected);
+
+	assert_within_epsilon(busy_r, expected, 0.15);
+	assert_within_epsilon(1 - busy_r, 1 - expected, 0.15);
+}
+
 igt_main
 {
 	const unsigned int num_other_metrics =
@@ -1197,8 +1622,11 @@ igt_main
 		invalid_init();
 
 	for_each_engine_class_instance(fd, e) {
+		const unsigned int pct[] = { 2, 50, 98 };
+
 		/**
-		 * Test that a single engine metric can be initialized.
+		 * Test that a single engine metric can be initialized or it
+		 * is correctly rejected.
 		 */
 		igt_subtest_f("init-busy-%s", e->name)
 			init(fd, e, I915_SAMPLE_BUSY);
@@ -1209,46 +1637,112 @@ igt_main
 		igt_subtest_f("init-sema-%s", e->name)
 			init(fd, e, I915_SAMPLE_SEMA);
 
-		/**
-		 * Test that engines show no load when idle.
-		 */
-		igt_subtest_f("idle-%s", e->name)
-			single(fd, e, false);
+		igt_subtest_group {
+			igt_fixture {
+				gem_require_engine(fd, e->class, e->instance);
+			}
 
-		/**
-		 * Test that a single engine reports load correctly.
-		 */
-		igt_subtest_f("busy-%s", e->name)
-			single(fd, e, true);
+			/**
+			 * Test that engines show no load when idle.
+			 */
+			igt_subtest_f("idle-%s", e->name)
+				single(fd, e, 0);
 
-		/**
-		 * Test that when one engine is loaded other report no load.
-		 */
-		igt_subtest_f("busy-check-all-%s", e->name)
-			busy_check_all(fd, e, num_engines);
+			/**
+			 * Test that a single engine reports load correctly.
+			 */
+			igt_subtest_f("busy-%s", e->name)
+				single(fd, e, TEST_BUSY);
+			igt_subtest_f("busy-idle-%s", e->name)
+				single(fd, e, TEST_BUSY | TEST_TRAILING_IDLE);
 
-		/**
-		 * Test that when all except one engine are loaded all loads
-		 * are correctly reported.
-		 */
-		igt_subtest_f("most-busy-check-all-%s", e->name)
-			most_busy_check_all(fd, e, num_engines);
+			/**
+			 * Test that when one engine is loaded other report no
+			 * load.
+			 */
+			igt_subtest_f("busy-check-all-%s", e->name)
+				busy_check_all(fd, e, num_engines, TEST_BUSY);
+			igt_subtest_f("busy-idle-check-all-%s", e->name)
+				busy_check_all(fd, e, num_engines,
+					       TEST_BUSY | TEST_TRAILING_IDLE);
 
-		/**
-		 * Test that semphore counters report no activity on idle
-		 * or busy engines.
-		 */
-		igt_subtest_f("idle-no-semaphores-%s", e->name)
-			no_sema(fd, e, false);
+			/**
+			 * Test that when all except one engine are loaded all
+			 * loads are correctly reported.
+			 */
+			igt_subtest_f("most-busy-check-all-%s", e->name)
+				most_busy_check_all(fd, e, num_engines,
+						    TEST_BUSY);
+			igt_subtest_f("most-busy-idle-check-all-%s", e->name)
+				most_busy_check_all(fd, e, num_engines,
+						    TEST_BUSY |
+						    TEST_TRAILING_IDLE);
 
-		igt_subtest_f("busy-no-semaphores-%s", e->name)
-			no_sema(fd, e, true);
+			/**
+			 * Test that semphore counters report no activity on
+			 * idle or busy engines.
+			 */
+			igt_subtest_f("idle-no-semaphores-%s", e->name)
+				no_sema(fd, e, 0);
 
-		/**
-		 * Test that semaphore waits are correctly reported.
-		 */
-		igt_subtest_f("semaphore-wait-%s", e->name)
-			sema_wait(fd, e);
+			igt_subtest_f("busy-no-semaphores-%s", e->name)
+				no_sema(fd, e, TEST_BUSY);
+
+			igt_subtest_f("busy-idle-no-semaphores-%s", e->name)
+				no_sema(fd, e, TEST_BUSY | TEST_TRAILING_IDLE);
+
+			/**
+			 * Test that semaphore waits are correctly reported.
+			 */
+			igt_subtest_f("semaphore-wait-%s", e->name)
+				sema_wait(fd, e, TEST_BUSY);
+
+			igt_subtest_f("semaphore-wait-idle-%s", e->name)
+				sema_wait(fd, e,
+					  TEST_BUSY | TEST_TRAILING_IDLE);
+
+			/**
+			 * Check that two perf clients do not influence each
+			 * others observations.
+			 */
+			igt_subtest_f("multi-client-%s", e->name)
+				multi_client(fd, e);
+
+			/**
+			* Check that reported usage is correct when PMU is
+			* enabled after the batch is running.
+			*/
+			igt_subtest_f("busy-start-%s", e->name)
+				busy_start(fd, e);
+
+			/**
+			 * Check that reported usage is correct when PMU is
+			 * enabled after two batches are running.
+			 */
+			igt_subtest_f("busy-double-start-%s", e->name) {
+				gem_require_contexts(fd);
+				busy_double_start(fd, e);
+			}
+
+			/**
+			 * Check that the PMU can be safely enabled in face of
+			 * interrupt-heavy engine load.
+			 */
+			igt_subtest_f("enable-race-%s", e->name)
+				test_enable_race(fd, e);
+
+			/**
+			 * Check engine busyness accuracy is as expected.
+			 */
+			for (i = 0; i < ARRAY_SIZE(pct); i++) {
+				igt_subtest_f("busy-accuracy-%u-%s",
+					      pct[i], e->name)
+					accuracy(fd, e, pct[i]);
+			}
+
+			igt_subtest_f("busy-hang-%s", e->name)
+				single(fd, e, TEST_BUSY | FLAG_HANG);
+		}
 
 		/**
 		 * Test that event waits are correctly reported.
@@ -1256,27 +1750,6 @@ igt_main
 		if (e->class == I915_ENGINE_CLASS_RENDER)
 			igt_subtest_f("event-wait-%s", e->name)
 				event_wait(fd, e);
-
-		/**
-		 * Check that two perf clients do not influence each others
-		 * observations.
-		 */
-		igt_subtest_f("multi-client-%s", e->name)
-			multi_client(fd, e);
-
-		/**
-		 * Check that reported usage is correct when PMU is enabled
-		 * after the batch is running.
-		 */
-		igt_subtest_f("busy-start-%s", e->name)
-			busy_start(fd, e);
-
-		/**
-		 * Check that reported usage is correct when PMU is enabled
-		 * after two batches are running.
-		 */
-		igt_subtest_f("busy-double-start-%s", e->name)
-			busy_double_start(fd, e);
 	}
 
 	/**
@@ -1284,7 +1757,10 @@ igt_main
 	 * correctly reported.
 	 */
 	igt_subtest("all-busy-check-all")
-		all_busy_check_all(fd, num_engines);
+		all_busy_check_all(fd, num_engines, TEST_BUSY);
+	igt_subtest("all-busy-idle-check-all")
+		all_busy_check_all(fd, num_engines,
+				   TEST_BUSY | TEST_TRAILING_IDLE);
 
 	/**
 	 * Test that non-engine counters can be initialized and read. Apart
@@ -1323,13 +1799,19 @@ igt_main
 	 * Test RC6 residency reporting.
 	 */
 	igt_subtest("rc6")
-		test_rc6(fd);
+		test_rc6(fd, 0);
+
+	igt_subtest("rc6-runtime-pm")
+		test_rc6(fd, TEST_RUNTIME_PM);
+
+	igt_subtest("rc6-runtime-pm-long")
+		test_rc6(fd, TEST_RUNTIME_PM | FLAG_LONG);
 
 	/**
 	 * Check render nodes are counted.
 	 */
 	igt_subtest_group {
-		int render_fd;
+		int render_fd = -1;
 
 		igt_fixture {
 			render_fd = drm_open_driver_render(DRIVER_INTEL);
@@ -1339,8 +1821,20 @@ igt_main
 		}
 
 		for_each_engine_class_instance(fd, e) {
-			igt_subtest_f("render-node-busy-%s", e->name)
-				single(fd, e, true);
+			igt_subtest_group {
+				igt_fixture {
+					gem_require_engine(render_fd,
+							   e->class,
+							   e->instance);
+				}
+
+				igt_subtest_f("render-node-busy-%s", e->name)
+					single(fd, e, TEST_BUSY);
+				igt_subtest_f("render-node-busy-idle-%s",
+					      e->name)
+					single(fd, e,
+					       TEST_BUSY | TEST_TRAILING_IDLE);
+			}
 		}
 
 		igt_fixture {
