@@ -38,7 +38,7 @@
  * decode, but never crash in the process.
  */
 
-#define _GNU_SOURCE
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -50,10 +50,14 @@
 #include <err.h>
 #include <assert.h>
 #include <intel_bufmgr.h>
+#include <zlib.h>
+#include <ctype.h>
 
 #include "intel_chipset.h"
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
 #include "instdone.h"
+#include "intel_reg.h"
+#include "drmtest.h"
 
 static uint32_t
 print_head(unsigned int reg)
@@ -94,14 +98,15 @@ print_instdone(uint32_t devid, unsigned int instdone, unsigned int instdone1)
 	static int once;
 
 	if (!once) {
-		init_instdone_definitions(devid);
+		if (!init_instdone_definitions(devid))
+			return;
 		once = 1;
 	}
 
 	for (i = 0; i < num_instdone_bits; i++) {
 		int busy = 0;
 
-		if (instdone_bits[i].reg == INST_DONE_1) {
+		if (instdone_bits[i].reg == INSTDONE_1) {
 			if (!(instdone1 & instdone_bits[i].bit))
 				busy = 1;
 		} else {
@@ -241,6 +246,78 @@ print_pgtbl_err(unsigned int reg, unsigned int devid)
 	}
 }
 
+static void print_ivb_error(unsigned int reg, unsigned int devid)
+{
+	if (reg & (1 << 0))
+		printf("    TLB page fault error (GTT entry not valid)\n");
+	if (reg & (1 << 1))
+		printf("    Invalid physical address in RSTRM interface (PAVP)\n");
+	if (reg & (1 << 2))
+		printf("    Invalid page directory entry error\n");
+	if (reg & (1 << 3))
+		printf("    Invalid physical address in ROSTRM interface (PAVP)\n");
+	if (reg & (1 << 4))
+		printf("    TLB page VTD translation generated an error\n");
+	if (reg & (1 << 5))
+		printf("    Invalid physical address in WRITE interface (PAVP)\n");
+	if (reg & (1 << 6))
+		printf("    Page directory VTD translation generated error\n");
+	if (reg & (1 << 8))
+		printf("    Cacheline containing a PD was marked as invalid\n");
+	if (IS_HASWELL(devid) && (reg >> 10) & 0x1f)
+		printf("    %d pending page faults\n", (reg >> 10) & 0x1f);
+}
+
+static void print_snb_error(unsigned int reg)
+{
+	if (reg & (1 << 0))
+		printf("    TLB page fault error (GTT entry not valid)\n");
+	if (reg & (1 << 1))
+		printf("    Context page GTT translation generated a fault (GTT entry not valid)\n");
+	if (reg & (1 << 2))
+		printf("    Invalid page directory entry error\n");
+	if (reg & (1 << 3))
+		printf("    HWS page GTT translation generated a page fault (GTT entry not valid)\n");
+	if (reg & (1 << 4))
+		printf("    TLB page VTD translation generated an error\n");
+	if (reg & (1 << 5))
+		printf("    Context page VTD translation generated an error\n");
+	if (reg & (1 << 6))
+		printf("    Page directory VTD translation generated error\n");
+	if (reg & (1 << 7))
+		printf("    HWS page VTD translation generated an error\n");
+	if (reg & (1 << 8))
+		printf("    Cacheline containing a PD was marked as invalid\n");
+}
+
+static void print_bdw_error(unsigned int reg, unsigned int devid)
+{
+	print_ivb_error(reg, devid);
+
+	if (reg & (1 << 10))
+		printf("    Non WB memory type for Advanced Context\n");
+	if (reg & (1 << 11))
+		printf("    PASID not enabled\n");
+	if (reg & (1 << 12))
+		printf("    PASID boundary violation\n");
+	if (reg & (1 << 13))
+		printf("    PASID not valid\n");
+	if (reg & (1 << 14))
+		printf("    PASID was zero for untranslated request\n");
+	if (reg & (1 << 15))
+		printf("    Context was not marked as present when doing DMA\n");
+}
+
+static void
+print_error(unsigned int reg, unsigned int devid)
+{
+	switch (intel_gen(devid)) {
+	case 8: return print_bdw_error(reg, devid);
+	case 7: return print_ivb_error(reg, devid);
+	case 6: return print_snb_error(reg);
+	}
+}
+
 static void
 print_snb_fence(unsigned int devid, uint64_t fence)
 {
@@ -274,7 +351,7 @@ print_i915_fence(unsigned int devid, uint64_t fence)
 
 	printf("    %svalid, %c-tiled, pitch: %i, start: 0x%08x, size: %i\n",
 			fence & 1 ? "" : "in",
-			fence & 12 ? 'y' : 'x',
+			fence & (1<<12) ? 'y' : 'x',
 			(1<<((fence>>4)&0xf))*tile_width,
 			(uint32_t)fence & 0xff00000,
 			1<<(20 + ((fence>>8)&0xf)));
@@ -285,7 +362,7 @@ print_i830_fence(unsigned int devid, uint64_t fence)
 {
 	printf("    %svalid, %c-tiled, pitch: %i, start: 0x%08x, size: %i\n",
 			fence & 1 ? "" : "in",
-			fence & 12 ? 'y' : 'x',
+			fence & (1<<12) ? 'y' : 'x',
 			(1<<((fence>>4)&0xf))*128,
 			(uint32_t)fence & 0x7f80000,
 			1<<(19 + ((fence>>8)&0xf)));
@@ -305,18 +382,185 @@ print_fence(unsigned int devid, uint64_t fence)
 	}
 }
 
-#define MAX_RINGS 10 /* I really hope this never... */
-uint32_t head[MAX_RINGS];
-int head_ndx = 0;
-int num_rings = -1;
-static void print_batch(int is_batch, const char *ring_name, uint32_t gtt_offset)
+static void
+print_fault_reg(unsigned devid, uint32_t reg)
 {
-	const char *buffer_type[2] = {  "ringbuffer", "batchbuffer" };
-	if (is_batch) {
-		printf("%s (%s) at 0x%08x\n", buffer_type[is_batch], ring_name, gtt_offset);
-	} else {
-		printf("%s (%s) at 0x%08x; HEAD points to: 0x%08x\n", buffer_type[is_batch], ring_name, gtt_offset, head[head_ndx++ % num_rings] + gtt_offset);
+	const char *gen7_types[] = { "Page",
+				     "Invalid PD",
+				     "Unloaded PD",
+				     "Invalid and Unloaded PD" };
+
+	const char *gen8_types[] = { "PTE",
+				     "PDE",
+				     "PDPE",
+				     "PML4E" };
+
+	const char *engine[] = { "GFX", "MFX0", "MFX1", "VEBX",
+				 "BLT", "Unknown", "Unknown", "Unknown" };
+
+	if (intel_gen(devid) < 7)
+		return;
+
+	if (reg & (1 << 0))
+		printf("    Valid\n");
+	else
+		return;
+
+	if (intel_gen(devid) < 8)
+		printf("    %s Fault (%s)\n", gen7_types[reg >> 1 & 0x3],
+		       reg & (1 << 11) ? "GGTT" : "PPGTT");
+	else
+		printf("    Invalid %s Fault\n", gen8_types[reg >> 1 & 0x3]);
+
+	if (intel_gen(devid) < 8)
+		printf("    Address 0x%08x\n", reg & ~((1 << 12)-1));
+	else
+		printf("    Engine %s\n", engine[reg >> 12 & 0x7]);
+
+	printf("    Source ID %d\n", reg >> 3 & 0xff);
+}
+
+static void
+print_fault_data(unsigned devid, uint32_t data1, uint32_t data0)
+{
+	uint64_t address;
+
+	if (intel_gen(devid) < 8)
+		return;
+
+	address = ((uint64_t)(data0) << 12) | ((uint64_t)data1 & 0xf) << 44;
+	printf("    Address 0x%016" PRIx64 " %s\n", address,
+	       data1 & (1 << 4) ? "GGTT" : "PPGTT");
+}
+
+#define MAX_RINGS 10 /* I really hope this never... */
+
+static bool maybe_ascii(const void *data, int check)
+{
+	const char *c = data;
+	while (check--) {
+		if (!isprint(*c++))
+			return false;
 	}
+	return true;
+}
+
+static void decode(struct drm_intel_decode *ctx,
+		   const char *buffer_name,
+		   const char *ring_name,
+		   uint64_t gtt_offset,
+		   uint32_t head_offset,
+		   uint32_t *data, int *count,
+		   int decode)
+{
+	if (!*count)
+		return;
+
+	printf("%s (%s) at 0x%08x_%08x", buffer_name, ring_name,
+	       (unsigned)(gtt_offset >> 32),
+	       (unsigned)(gtt_offset & 0xffffffff));
+	if (head_offset != -1)
+		printf("; HEAD points to: 0x%08x_%08x",
+		       (unsigned)((head_offset + gtt_offset) >> 32),
+		       (unsigned)((head_offset + gtt_offset) & 0xffffffff));
+	printf("\n");
+
+	if (decode) {
+		drm_intel_decode_set_batch_pointer(ctx, data, gtt_offset,
+						   *count);
+		drm_intel_decode(ctx);
+	} else if (maybe_ascii(data, 16)) {
+		printf("%*s\n", 4 * *count, (char *)data);
+	} else {
+		for (int i = 0; i + 4 <= *count; i += 4)
+			printf("[%04x] %08x %08x %08x %08x\n",
+			       4*i, data[i], data[i+1], data[i+2], data[i+3]);
+	}
+	*count = 0;
+}
+
+static int zlib_inflate(uint32_t **ptr, int len)
+{
+	struct z_stream_s zstream;
+	void *out;
+
+	memset(&zstream, 0, sizeof(zstream));
+
+	zstream.next_in = (unsigned char *)*ptr;
+	zstream.avail_in = 4*len;
+
+	if (inflateInit(&zstream) != Z_OK)
+		return 0;
+
+	out = malloc(128*4096); /* approximate obj size */
+	zstream.next_out = out;
+	zstream.avail_out = 128*4096;
+
+	do {
+		switch (inflate(&zstream, Z_SYNC_FLUSH)) {
+		case Z_STREAM_END:
+			goto end;
+		case Z_OK:
+			break;
+		default:
+			inflateEnd(&zstream);
+			return 0;
+		}
+
+		if (zstream.avail_out)
+			break;
+
+		out = realloc(out, 2*zstream.total_out);
+		if (out == NULL) {
+			inflateEnd(&zstream);
+			return 0;
+		}
+
+		zstream.next_out = (unsigned char *)out + zstream.total_out;
+		zstream.avail_out = zstream.total_out;
+	} while (1);
+end:
+	inflateEnd(&zstream);
+	free(*ptr);
+	*ptr = out;
+	return zstream.total_out / 4;
+}
+
+static int ascii85_decode(const char *in, uint32_t **out, bool inflate)
+{
+	int len = 0, size = 1024;
+
+	*out = realloc(*out, sizeof(uint32_t)*size);
+	if (*out == NULL)
+		return 0;
+
+	while (*in >= '!' && *in <= 'z') {
+		uint32_t v = 0;
+
+		if (len == size) {
+			size *= 2;
+			*out = realloc(*out, sizeof(uint32_t)*size);
+			if (*out == NULL)
+				return 0;
+		}
+
+		if (*in == 'z') {
+			in++;
+		} else {
+			v += in[0] - 33; v *= 85;
+			v += in[1] - 33; v *= 85;
+			v += in[2] - 33; v *= 85;
+			v += in[3] - 33; v *= 85;
+			v += in[4] - 33;
+			in += 5;
+		}
+		(*out)[len++] = v;
+	}
+
+	if (!inflate)
+		return len;
+
+	return zlib_inflate(out, len);
 }
 
 static void
@@ -325,78 +569,110 @@ read_data_file(FILE *file)
 	struct drm_intel_decode *decode_ctx = NULL;
 	uint32_t devid = PCI_CHIP_I855_GM;
 	uint32_t *data = NULL;
+	uint32_t head[MAX_RINGS];
+	int head_idx = 0;
+	int num_rings = 0;
 	long long unsigned fence;
-	int data_size = 0, count = 0, line_number = 0, matched;
+	int data_size = 0, count = 0, matched;
 	char *line = NULL;
 	size_t line_size;
 	uint32_t offset, value, ring_length = 0;
-	uint32_t gtt_offset = 0, new_gtt_offset;
+	uint64_t gtt_offset = 0;
+	uint32_t head_offset = -1;
+	const char *buffer_name = "batch buffer";
 	char *ring_name = NULL;
-	int is_batch = 1;
+	int do_decode = 1;
 
 	while (getline(&line, &line_size, file) > 0) {
 		char *dashes;
-		line_number++;
+
+		if (line[0] == ':' || line[0] == '~') {
+			count = ascii85_decode(line+1, &data, line[0] == ':');
+			if (count == 0)
+				fprintf(stderr, "ASCII85 decode failed (%s - %s).\n",
+					ring_name, buffer_name);
+			decode(decode_ctx,
+			       buffer_name, ring_name,
+			       gtt_offset, head_offset,
+			       data, &count, do_decode);
+			continue;
+		}
 
 		dashes = strstr(line, "---");
 		if (dashes) {
-			char *new_ring_name = malloc(dashes - line);
+			const struct {
+				const char *match;
+				const char *name;
+				int do_decode;
+			} buffers[] = {
+				{ "ringbuffer", "ring", 1 },
+				{ "gtt_offset", "batch", 1 },
+				{ "hw context", "HW context", 1 },
+				{ "hw status", "HW status", 0 },
+				{ "wa context", "WA context", 1 },
+				{ "wa batchbuffer", "WA batch", 1 },
+				{ "user", "user", 0 },
+				{ "semaphores", "semaphores", 0 },
+				{ "guc log buffer", "GuC log", 0 },
+				{ },
+			}, *b;
+			char *new_ring_name;
+
+			new_ring_name = malloc(dashes - line);
 			strncpy(new_ring_name, line, dashes - line);
 			new_ring_name[dashes - line - 1] = '\0';
 
-			if (num_rings == -1)
-				num_rings = head_ndx;
+			decode(decode_ctx,
+			       buffer_name, ring_name,
+			       gtt_offset, head_offset,
+			       data, &count, do_decode);
+			gtt_offset = 0;
+			head_offset = -1;
 
-			matched = sscanf(dashes, "--- gtt_offset = 0x%08x\n",
-					&new_gtt_offset);
-			if (matched == 1) {
-				if (count) {
-					print_batch(is_batch, ring_name, gtt_offset);
-					drm_intel_decode_set_batch_pointer(decode_ctx,
-							data, gtt_offset,
-							count);
-					drm_intel_decode(decode_ctx);
-					count = 0;
+			free(ring_name);
+			ring_name = new_ring_name;
+
+			dashes += 4;
+			for (b = buffers; b->match; b++) {
+				uint32_t lo, hi;
+
+				if (strncasecmp(dashes, b->match,
+						strlen(b->match)))
+					continue;
+
+				dashes = strchr(dashes, '=');
+				if (!dashes)
+					break;
+
+				matched = sscanf(dashes, "= 0x%08x %08x\n",
+						 &hi, &lo);
+				if (matched > 0) {
+					gtt_offset = hi;
+					if (matched == 2) {
+						gtt_offset <<= 32;
+						gtt_offset |= lo;
+					}
 				}
-				gtt_offset = new_gtt_offset;
-				is_batch = 1;
-				free(ring_name);
-				ring_name = new_ring_name;
-				continue;
+
+				do_decode = b->do_decode;
+				buffer_name = b->name;
+				if (b == buffers)
+					head_offset = head[head_idx++];
+				break;
 			}
 
-			matched = sscanf(dashes, "--- ringbuffer = 0x%08x\n",
-					&new_gtt_offset);
-			if (matched == 1) {
-				if (count) {
-					print_batch(is_batch, ring_name, gtt_offset);
-					drm_intel_decode_set_batch_pointer(decode_ctx,
-							data, gtt_offset,
-							count);
-					drm_intel_decode(decode_ctx);
-					count = 0;
-				}
-				gtt_offset = new_gtt_offset;
-				is_batch = 0;
-				free(ring_name);
-				ring_name = new_ring_name;
-				continue;
-			}
+			continue;
 		}
 
 		matched = sscanf(line, "%08x : %08x", &offset, &value);
 		if (matched != 2) {
-			unsigned int reg;
+			unsigned int reg, reg2;
 
 			/* display reg section is after the ringbuffers, don't mix them */
-			if (count) {
-				print_batch(is_batch, ring_name, gtt_offset);
-				drm_intel_decode_set_batch_pointer(decode_ctx,
-						data, gtt_offset,
-						count);
-				drm_intel_decode(decode_ctx);
-				count = 0;
-			}
+			decode(decode_ctx,
+			       buffer_name, ring_name,
+			       gtt_offset, head_offset,
+			       data, &count, do_decode);
 
 			printf("%s", line);
 
@@ -422,7 +698,7 @@ read_data_file(FILE *file)
 
 			matched = sscanf(line, "  HEAD: 0x%08x\n", &reg);
 			if (matched == 1) {
-				head[head_ndx++] = print_head(reg);
+				head[num_rings++] = print_head(reg);
 			}
 
 			matched = sscanf(line, "  ACTHD: 0x%08x\n", &reg);
@@ -435,6 +711,10 @@ read_data_file(FILE *file)
 			if (matched == 1 && reg)
 				print_pgtbl_err(reg, devid);
 
+			matched = sscanf(line, "  ERROR: 0x%08x\n", &reg);
+			if (matched == 1 && reg)
+				print_error(reg, devid);
+
 			matched = sscanf(line, "  INSTDONE: 0x%08x\n", &reg);
 			if (matched == 1)
 				print_instdone(devid, reg, -1);
@@ -447,6 +727,14 @@ read_data_file(FILE *file)
 			if (matched == 2)
 				print_fence(devid, fence);
 
+			matched = sscanf(line, "  FAULT_REG: 0x%08x\n", &reg);
+			if (matched == 1 && reg)
+				print_fault_reg(devid, reg);
+
+			matched = sscanf(line, "  FAULT_TLB_DATA: 0x%08x 0x%08x\n", &reg, &reg2);
+			if (matched == 2)
+				print_fault_data(devid, reg, reg2);
+
 			continue;
 		}
 
@@ -464,19 +752,40 @@ read_data_file(FILE *file)
 		data[count-1] = value;
 	}
 
-	if (count) {
-		print_batch(is_batch, ring_name, gtt_offset);
-		drm_intel_decode_set_batch_pointer(decode_ctx,
-				data, gtt_offset,
-				count);
-		drm_intel_decode(decode_ctx);
-	}
+	decode(decode_ctx,
+	       buffer_name, ring_name,
+	       gtt_offset, head_offset,
+	       data, &count, do_decode);
 
 	free(data);
 	free(line);
 	free(ring_name);
 }
 
+static void setup_pager(void)
+{
+	int fds[2];
+
+	if (pipe(fds) == -1)
+		return;
+
+	switch (fork()) {
+	case -1:
+		break;
+	case 0:
+		close(fds[1]);
+		dup2(fds[0], 0);
+		execlp("less", "less", "-FRSi", NULL);
+		break;
+
+	default:
+		close(fds[0]);
+		dup2(fds[1], 1);
+		close(fds[1]);
+		break;
+	}
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -502,6 +811,9 @@ main(int argc, char *argv[])
 		return 1;
 	}
 
+	if (isatty(1))
+		setup_pager();
+
 	if (argc == 1) {
 		if (isatty(0)) {
 			path = "/sys/class/drm/card0/error";
@@ -575,4 +887,4 @@ main(int argc, char *argv[])
 	return 0;
 }
 
-/* vim: set ts=8 sw=8 tw=0 noet :*/
+/* vim: set ts=8 sw=8 tw=0 cino=:0,(0 noet :*/
