@@ -25,6 +25,7 @@
  *
  */
 
+#include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <getopt.h>
@@ -309,8 +310,8 @@ static const struct {
 	{ DEVICE_TYPE_COMPOSITE_OUTPUT, "Composite output" },
 	{ DEVICE_TYPE_DUAL_CHANNEL, "Dual channel" },
 	{ 1 << 7, "Content protection" },
-	{ DEVICE_TYPE_HIGH_SPEED_LINK, "High speel link" },
-	{ DEVICE_TYPE_LVDS_SINGALING, "LVDS signaling" },
+	{ DEVICE_TYPE_HIGH_SPEED_LINK, "High speed link" },
+	{ DEVICE_TYPE_LVDS_SIGNALING, "LVDS signaling" },
 	{ DEVICE_TYPE_TMDS_DVI_SIGNALING, "TMDS/DVI signaling" },
 	{ DEVICE_TYPE_VIDEO_SIGNALING, "Video signaling" },
 	{ DEVICE_TYPE_DISPLAYPORT_OUTPUT, "DisplayPort output" },
@@ -513,6 +514,8 @@ static void dump_general_definitions(struct context *context,
 	const struct bdb_general_definitions *defs = block->data;
 	int child_dev_num;
 
+	child_dev_num = (block->size - sizeof(*defs)) / defs->child_dev_size;
+
 	printf("\tCRT DDC GMBUS addr: 0x%02x\n", defs->crt_ddc_gmbus_pin);
 	printf("\tUse ACPI DPMS CRT power states: %s\n",
 	       YESNO(defs->dpms_acpi));
@@ -522,8 +525,8 @@ static void dump_general_definitions(struct context *context,
 	printf("\tBoot display type: 0x%02x%02x\n", defs->boot_display[1],
 	       defs->boot_display[0]);
 	printf("\tChild device size: %d\n", defs->child_dev_size);
+	printf("\tChild device count: %d\n", child_dev_num);
 
-	child_dev_num = (block->size - sizeof(*defs)) / defs->child_dev_size;
 	dump_child_devices(context, defs->devices,
 			   child_dev_num, defs->child_dev_size);
 }
@@ -534,9 +537,11 @@ static void dump_legacy_child_devices(struct context *context,
 	const struct bdb_legacy_child_devices *defs = block->data;
 	int child_dev_num;
 
-	printf("\tChild device size: %d\n", defs->child_dev_size);
-
 	child_dev_num = (block->size - sizeof(*defs)) / defs->child_dev_size;
+
+	printf("\tChild device size: %d\n", defs->child_dev_size);
+	printf("\tChild device count: %d\n", child_dev_num);
+
 	dump_child_devices(context, defs->devices,
 			   child_dev_num, defs->child_dev_size);
 }
@@ -1658,6 +1663,33 @@ static bool dump_section(struct context *context, int section_id)
 	return true;
 }
 
+/* print a description of the VBT of the form <bdb-version>-<vbt-signature> */
+static void print_description(struct context *context)
+{
+	const struct vbt_header *vbt = context->vbt;
+	const struct bdb_header *bdb = context->bdb;
+	char *desc = strndup((char *)vbt->signature, sizeof(vbt->signature));
+	char *p;
+
+	for (p = desc + strlen(desc) - 1; p >= desc && isspace(*p); p--)
+		*p = '\0';
+
+	for (p = desc; *p; p++) {
+		if (!isalnum(*p))
+			*p = '-';
+		else
+			*p = tolower(*p);
+	}
+
+	p = desc;
+	if (strncmp(p, "-vbt-", 5) == 0)
+		p += 5;
+
+	printf("%d-%s\n", bdb->version, p);
+
+	free (desc);
+}
+
 static void dump_headers(struct context *context)
 {
 	const struct vbt_header *vbt = context->vbt;
@@ -1720,6 +1752,8 @@ enum opt {
 	OPT_HEXDUMP,
 	OPT_BLOCK,
 	OPT_USAGE,
+	OPT_HEADER,
+	OPT_DESCRIBE,
 };
 
 static void usage(const char *toolname)
@@ -1731,6 +1765,8 @@ static void usage(const char *toolname)
 			" [--all-panels]"
 			" [--hexdump]"
 			" [--block=<block_no>]"
+			" [--header]"
+			" [--describe]"
 			" [--help]\n");
 }
 
@@ -1751,6 +1787,7 @@ int main(int argc, char **argv)
 	};
 	char *endp;
 	int block_number = -1;
+	bool header_only = false, describe = false;
 
 	static struct option options[] = {
 		{ "file",	required_argument,	NULL,	OPT_FILE },
@@ -1759,6 +1796,8 @@ int main(int argc, char **argv)
 		{ "all-panels",	no_argument,		NULL,	OPT_ALL_PANELS },
 		{ "hexdump",	no_argument,		NULL,	OPT_HEXDUMP },
 		{ "block",	required_argument,	NULL,	OPT_BLOCK },
+		{ "header",	no_argument,		NULL,	OPT_HEADER },
+		{ "describe",	no_argument,		NULL,	OPT_DESCRIBE },
 		{ "help",	no_argument,		NULL,	OPT_USAGE },
 		{ 0 }
 	};
@@ -1799,6 +1838,12 @@ int main(int argc, char **argv)
 				return EXIT_FAILURE;
 			}
 			break;
+		case OPT_HEADER:
+			header_only = true;
+			break;
+		case OPT_DESCRIBE:
+			describe = true;
+			break;
 		case OPT_END:
 			break;
 		case OPT_USAGE: /* fall-through */
@@ -1902,7 +1947,11 @@ int main(int argc, char **argv)
 		context.panel_type = 0;
 	}
 
-	if (block_number != -1) {
+	if (describe) {
+		print_description(&context);
+	} else if (header_only) {
+		dump_headers(&context);
+	} else if (block_number != -1) {
 		/* dump specific section only */
 		if (!dump_section(&context, block_number)) {
 			fprintf(stderr, "Block %d not found\n", block_number);
